/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug;
import CTL.Types.*;

/**
 * Represents an element of the continuous list, stores the PeerID,FID and method name.
 * @author Bjoern Gebhardt, Johannes Baumgarten, Jurij Merker, Stephan Kluge, Svetlana Li
 *
 */
public class CListContainer {
    
    private PeerID peerid; // the PeerID of the component waitign for a continuous message
    private FID fid; // the functionID of the called method
    private String method_name;	//the name of the called Method 
    private boolean eoc; //true if the message is a EOC call
    
    public CListContainer(PeerID peerid, FID fid, String method_name, boolean iseoc) {
        this.peerid = peerid;
        this.fid = fid;
        this.method_name = method_name;
        this.eoc = iseoc;
    }
    /**
     * Used to get the PeerID
     * @return PeerID the PeerID of the waiting component
     */
    public PeerID getPeerID() {
        return peerid;
    }
    
    /**
     * Used to get the FunctionID		
     * @return FID The FunctionID of the method called in the original message
     */
    public FID getFunctionID() {
        return fid;
    }
    
    /**
     * Used to get the name of the called method	
     * @return String The fully qualified name of the called method
     */
    public String getMethodName() {
    	return method_name;
    }
    
    
    /**
     * Used to save if the message is a EOC Message
     * @return boolean true if the call is a EOC call
     */
    public boolean isEOC(){
    	return eoc;
    }
}
