/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;

/**
 * realizes an element of the breakpoint list
 * @author Stephan Kluge
 *
 */
public class BreakpointListElement {
	
	private short id;
	private String packagename = null;
	private String classname;
	private String classpath;
	private String method;
	private String suffix = "";
	
    
    /**
     * Saves information about an element of the breakpointlist
     * @param id the id of a method
     * @param classpath the classpath of a class
     * @param method the name of a method with parameters
     */
	public BreakpointListElement(short id, String classpath, String method) {
		this.id = id;
		this.classname = SplitString.lastExpression(classpath, '.');
		this.packagename = SplitString.firstExpressions(classpath, '.');
		this.classpath = classpath;
		this.method = method;
	}
	
    
    
    /**
     * Saves information about an element of the breakpointlist
     * @param suffix "C" is a constructor and "S" a static method
     * @param id the id of a method
     * @param classpath the classpath of a class
     * @param method the name of a method with parameters
     */
	public BreakpointListElement(String suffix, short id, String classpath, String method) {
		this.classname = SplitString.lastExpression(classpath, '.');
		this.packagename = SplitString.firstExpressions(classpath, '.');
		this.classpath = classpath;
		this.id = id;
		this.method = method;
		this.suffix = suffix;
	}
	
    
    
	/**
	 * used to get the FunctionID
	 * @return FunctionID
	 */
	public int getID() {
		return id ;
	}
	
    
    
	/**
	 * used to get the classpath to the class
	 * @return String classpath
	 */
	public String getClasspath() {
		return classpath;
	}
	
    
    
	/**
	 * check if constructor
	 * @return boolean constructor
	 */
	public boolean getConstructor() {
		if(suffix.equals("C"))
			return true;
		else
			return false;
	}
	
    
    
	/**
	 * 
	 * @return The packagename
	 */
	public String getPackage() {
		return packagename;
	}
	
    
    
	/**
	 * used to get the method, at which to break
	 * @return String method at which to break
	 */
	public String getMethod() {
		return method;
	}
	
    
    
	/**
	 * 
	 * @return The FQCN <<package>>::<<classname>>:<<id>><<suffix>>
	 */
	public String getFQCN() {
		return packagename+ "::" +classname+ ":" +id +suffix;
	}
}
