/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.LinkedList;
import CTL.RUtil;
import CTL.Types.*;
import Debug.*;
import ReflWrap.ClassInfo;
import ReflWrap.MethodInfo;

/**
 * Searches methods and constructors in classes of the CTL4j and saves them as breakpoints
 * if desired by user
 * 
 * @author Stephan Kluge
 *
 */
public class CommandBreakOn {
    
    private Method[] all_methods;
    private Constructor[] all_constructors;
    public LinkedList<Short> index;
    private String classpath;
    private ClassInfo cinfo;
    private static String reached_breakpoint = "";
    public static LinkedList<BreakpointListElement> breakpointlist;
    
    
    /**
     * instances a new list for breakpoints
     */
    public CommandBreakOn() {
        breakpointlist = new LinkedList();
    }
    

    
    /**
     * Searches for multiple methods with the same name, but different parameters, in a class
     * @param classpath The path to a class with the notation «package».«class»
     * @param method_name The searched method name
     * @return Number of methods who was found in the class
     * 		   
     */
    public short searchMultipleMethods(String classpath, String method_name) {
    	try {
    		cinfo = new ClassInfo(classpath);
    		this.classpath = RUtil.getBase(classpath);
    		
    		if(this.classpath == null)
    			this.classpath = classpath;
    		cinfo = new ClassInfo(this.classpath);
    		
    		all_methods = cinfo.meth_here();
//    		for(int i = 0; i < all_methods.length; i++) {
//    			System.out.println(i+": "+all_methods[i].getName());
//    			System.out.println(ci.methodList().get(i));
//    		}
    		
    		index = new LinkedList();
    		
    		for(int i = 0; i < all_methods.length; i++) {
    			if(method_name.equals(all_methods[i].getName())) {
    				index.addLast((short)i);
    			}
    		}
    		return (short)index.size();
    	}
    	catch(ClassNotFoundException cnfe) {
//    		System.out.println("Class not found");
    		return 0;
    	}
    }
    
    
    
    /**
     * Checks if a certain class exists
     * @param component class to check
     * @return number of constructors
     * @throws ClassNotFoundException
     */
    public short searchClass(String classpath) throws ClassNotFoundException {
    	//System.out.println("Ausgabe von getBase in searchClass (Breakpoint setzen): "+RUtil.getBase(classpath));
    	//System.out.println("Ausgabe von getBase in searchClass (javaSys.CTestRI übergeben): "+RUtil.getBase("javaSys.CTestRI"));
    	
    	cinfo = new ClassInfo(classpath);
    	this.classpath = RUtil.getBase(classpath);
    	
    	if(this.classpath == null)
    		this.classpath = classpath;
    	
        cinfo = new ClassInfo(this.classpath);
        return searchConstructors();
    }
    
    
    
    /**
     * searches and saves the constructors of class to break on
     * @return number of constructors
     */
    private short searchConstructors() {

    		all_constructors = cinfo.constructors();
    		index = new LinkedList();
//    		System.out.println("classpath:" +classpath);
//    		System.out.println("all_const_länge: "+all_constructors.length);
//    		System.out.println("fid.länge: "+index.size());
//    		System.out.println("konst-name: "+all_constructors[0].getName());
    		
    		for(int i = 0; i < all_constructors.length; i++) {
    			if(classpath.equals(all_constructors[i].getName())) {
    				index.addLast((short)i);
    			}
    		}
    		return (short)index.size();
    }
    
    
    
    /**
     * Saves the selected constructor as breakpoint
     * @param index index of the constructor at which to break
     * @param constructor true if is constructor
     */
    public void setBreakpoint(short user_index, boolean constructor) {    	
    	short fid = 0;
    	
    	if(constructor)
    		breakpointlist.addLast(new BreakpointListElement("C", index.get(user_index), classpath, createConstructorAsString(index.get(user_index))));
    	else {
    		MethodInfo mi = new MethodInfo(all_methods[index.get(user_index)]);
    		
    		if(mi.isStatic()) {
    			for(int i = 0; i <= index.get(user_index); i++) {
    				MethodInfo m = new MethodInfo(all_methods[i]);
    				if(m.isStatic())
    					fid++;
    			}
    			breakpointlist.addLast(new BreakpointListElement("S", fid, classpath, createMethodAsString(index.get(user_index))));
    		}
    		else {
    			for(int i = 0; i <= index.get(user_index); i++) {
    				MethodInfo m = new MethodInfo(all_methods[i]);
    				if(!(m.isStatic()))
    					fid++;
    			}
    			breakpointlist.addLast(new BreakpointListElement(fid, classpath, createMethodAsString(index.get(user_index))));
    		}
    	}
    }
    
    
    
    private String createMethodAsString(short fid) {
    	String method = "";
    	String param = "";
    	Class[] all_parameters;
    	
    	all_parameters = all_methods[fid].getParameterTypes();
    		
    	for(int i = 0; i < all_parameters.length; i++) {
    		if(i == 0)
    			param = all_parameters[i].getSimpleName();
    		else if(i > 0)
    			param = param + ", " +all_parameters[i].getSimpleName();
    	}
    	method = all_methods[fid].getName()+ "(" +param+ ")";
    	return method;
    }
	
    
    
    private String createConstructorAsString(short fid) {
    	String method = SplitString.lastExpression(all_constructors[fid].getName(), '.');
    	String param = "";
    	Class[] all_parameters;
    	
    	
    	all_parameters = all_constructors[fid].getParameterTypes();
    		
    	for(int i = 0; i < all_parameters.length; i++) {
    		if(i == 0)
    			param = all_parameters[i].getSimpleName();
    		else if(i > 0)
    			param = param + ", " +all_parameters[i].getSimpleName();
    	}
    	method = method+ "(" +param+ ")";
    	return method;
    }
    
    
    
    /**
     * 
     * @return The multiple methods with the same name and their parameters as String
     */
    public String getMethodsAsString() {
    	String methods = "";
    	String param = "";
    	Class[] all_parameters;
    	
    	for(int i = 0; i < index.size(); i++) {
    		all_parameters = all_methods[index.get(i)].getParameterTypes();
    		
    		for(int j = 0; j < all_parameters.length; j++) {
    			if(j == 0)
    				param = all_parameters[j].getSimpleName();
    			else if(j > 0)
    				param = param + ", " +all_parameters[j].getSimpleName();
    		}
    		methods = methods + "\n" +(i+1)+ " " +all_methods[index.get(i)].getName()+
    				  "(" +param+ ")";
    	}
    	return methods;
    }
    
    
    
    /**
     * 
     * @return The multiple constructors with the same name and their parameters as String
     */
    public String getConstructorsAsString() {
    	String methods = "";
    	String param = "";
    	Class[] all_parameters;
    	
    	for(int i = 0; i < index.size(); i++) {
    		all_parameters = all_constructors[index.get(i)].getParameterTypes();
    		
    		for(int j = 0; j < all_parameters.length; j++) {
    			if(j == 0)
    				param = all_parameters[j].getSimpleName();
    			else if(j > 0)
    				param = param + ", " +all_parameters[j].getSimpleName();
    		}
    		methods = methods + "\n" +(i+1)+ " " +all_constructors[index.get(i)].getName()+
    				  "(" +param+ ")";
    	}
    	return methods;
    }
    
    
    
    /**
     * 
     * @return returns the saved breakpoints
     */
    public String getBreakpointList() {
    	String list = "\n\n\n\nBreakpoints:\n";
    	
    	for(int i = 0; i < breakpointlist.size(); i++) {	
    		list = list+ "\n" +(breakpointlist.get(i).getClasspath()) +
    			   		"." +(breakpointlist.get(i).getMethod());
    	}
    	return list;
    }
    
    
    
    /**
     * checks if the first element of the continuous list equals a saved breakpoint
     *
     */
    public void findBreakpoint() {
        while(true) {
        	if(EnvDebug.mirror.continuous_list.size() > 0) {
        		FID fid = EnvDebug.mirror.continuous_list.getFirst().getFunctionID();
        		String baseclass = "";
        		String fqcn = "";
        		
        		for(int i = 0; i < breakpointlist.size(); i++) {
//            		System.out.println("FID_class: "+fid._class());
//            		System.out.println("FID_name: "+fid.name());
//            		System.out.println("breakpoint class: "+(breakpointlist.get(i)).getClasspath());
//            		System.out.println("FID id: "+fid.ID());
//            		System.out.println("breakpoint id: "+(breakpointlist.get(i)).getID());
//            		
//            		System.out.println("Continuous-Liste--Methodenname: "+(EnvDebug.mirror.continuous_list.getFirst()).getMethodName());
//            		System.out.println("Mirror.id-list-klassenname: " +EnvDebug.mirror.getClassName(
//            				EnvDebug.mirror.getPartner((EnvDebug.mirror.continuous_list.getFirst()).getPeerID())));
//            		System.out.println("breakpoint FQCN: "+breakpointlist.get(i).getFQCN());
//            		System.out.println("------------------------------------------------------------------------------");
            		
//            		System.out.println("Ausgabe von getBase in find Breakpoint (fid._class übergeben): "+RUtil.getBase(fid._class()));
//            		System.out.println("Ausgabe von getBase in find Breakpoint (javaSys.CTestRI übergeben): "+RUtil.getBase("javaSys.CTestRI"));
            		if(!(fid._class().equals("answer"))) {
            			if(fid._class() != "") {
            				baseclass = RUtil.getBase(fid._class());
            				fqcn = createFQCN(baseclass, SplitString.lastExpression(fid.name(), ':'));
            				if(fqcn.equals(breakpointlist.get(i).getFQCN())) {
                				reached_breakpoint = breakpointlist.get(i).getClasspath()
                									 + "."
                									 +breakpointlist.get(i).getMethod();
            					breakpointlist.remove(i);
                				return;
                			}
            			}
            			else if(fid._class().equals("")) {
            				baseclass = RUtil.getBase(
            							EnvDebug.mirror.getClassName(
                    					EnvDebug.mirror.getPartner((
                    					EnvDebug.mirror.continuous_list.getFirst()).getPeerID())));
            				fqcn = createFQCN(baseclass, ""+fid.ID());
            				if(fqcn.equals(breakpointlist.get(i).getFQCN())) {
            					reached_breakpoint = breakpointlist.get(i).getClasspath()
								 					 + "."
								 					 +breakpointlist.get(i).getMethod();
            					breakpointlist.remove(i);
            					return;
            				}
            			}
            		}
                }
                EnvDebug.mirror.sendContinuousToFirstElement();
            }
        }
    }
    
    
    
    private String createFQCN(String classpath, String id) {
    	 String fqcn = SplitString.firstExpressions(classpath, '.')
 		   			  + "::"
 		   			  +SplitString.lastExpression(classpath, '.')
 		   			  + ":"
 		   			  +id;
    	 return fqcn;
    }
    
    
    /**
     * 
     * @return a reached breakpoint
     */
    public static String getReachedBreakpoint() {
    	return reached_breakpoint;
    }
}