/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;

import java.io.File;
import java.util.LinkedList;
import CTL.Env;
import CTL.Logger;
import CTL.Comm.TCPCommunicator;
import CTL.Types.Location;

/**
 * @author Stephan Kluge
 *
 */
public class CommandRun extends Thread {
    
    private PromptEvent promptevent;
    private Prompt prompt;
    private String component;
    
    
    /**
     * Constructs the reference to the classes Promptevent, Mirror and Prompt.
     * @param promptevent Reference to the class Promptevent
     * @param prompt Reference to the class Promp
     * @param component the program who shall be started
     */
    public CommandRun(PromptEvent promptevent, Prompt prompt, String component) {
        this.promptevent = promptevent;
        this.prompt = prompt;
        this.component = component;
    }
    
    
    
    /**
     * Runs a program at the distributed system
     */
    public void run() {
    	Env.log = new Logger("/tmp/debugger", Logger.DBG);

		String myHost = "localhost";
		if (new File("locs.txt").exists())
		{
			LinkedList<Location> locs = Location.parseFile("locs.txt");
			if (locs != null)
			{
				String hisHost = locs.get(0).host();
				myHost = TCPCommunicator.determineIP(hisHost, 22).toString();
			}
		}

		String myProg = component; // Specified by the user
		String cmd = "xterm -hold -e env CTL_DEBUGGER="+myHost+":"+DebuggerMain.ds.getPort()+
			" java -Dfile.encoding=ISO-8859-1 "+myProg; // For C just use myProg
		try
		{
			prompt.textarea.append("\n\nprogram \"" +component+ "\" started");
			System.out.println(NCLib.Process.exec(cmd));
		}
		catch (Exception e)
		{
			System.out.println("Could not start process:");
			e.printStackTrace();
			System.exit(1);
		}

		System.out.println("Received "+DebuggerMain.recv+" bytes.");
		//System.exit(0);
		System.out.println("program finished");
		prompt.textfieldWhite();
		prompt.textfield.setEditable(true);
		promptevent.setCommandRunStarted(false);
    }
}
