/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.LinkedList;
import CTL.Types.PeerID;





/**
 * Accepts a connection and passes it on to the MirrorConnectHandler
 * @author Jurij Merker,Johannes Baumgarten,Bjoern Gebhardt,Stephan Kluge,Svetlana Li
 *
 */
public class DebugServer extends Thread {

	public int port = -1;
	private ServerSocket ssock = null;
	private LinkedList<Socket> dudes = new LinkedList<Socket>();
	

	
	public DebugServer ()
	{	
		try
		{
			ssock = new ServerSocket(0, 25);
			System.out.println("ServerSocket: "+ssock.getLocalPort());
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		port = ssock.getLocalPort();
	}
					
/**
 * Waits for a connection and upon recieving data calls a new MirrorConnectHandler
 */
	public void run ()
	{
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		
		if (ssock == null)
			return;
		
		int i = 0;
		while (true)
		{
			try
			{
				Socket sock = ssock.accept();
				System.out.println("New connection from "+sock.getRemoteSocketAddress());
				dudes.add(sock);
				MirrorConnectHandler handle = new MirrorConnectHandler(sock, i);
				handle.start();
				i++;
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	/**
	 * used to get the port of the DebugServer
	 * @return int port the port of the Server
	 */
	public int getPort ()
	{
		return port;
	}

	/**
	 * used to get a PeerID with the port of the Server and the ip
	 * set to localhost
	 * @return PeerID of the Server
	 * @throws UnknownHostException
	 */
	public PeerID getPID () throws UnknownHostException
	{
		return new PeerID("localhost", port);
	}

	
	/**
	 * used to get the list of connected sockets
	 * @return LinkedList dudes contains the connected sockets
	 */
	public LinkedList<Socket> getDudes ()
	{
		return dudes;
	}
}
