/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;
/**
 * @author Johannes Baumgarten,Jurij Merker,Bjoern Gebhardt,Stephan Kluge,Svetlana Li
 * @version 1.2
 * 
 * Handles connections, updates the visualizer and passes the recieved data to the generated
 * NameDebug classes for further diassembling.
 */

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.Socket;
import java.nio.ByteOrder;
import CTL.*;
import CTL.Comm.BuffyIn;

import CTL.Streams.OIStream;
import CTL.Streams.OOStream;
import CTL.Types.*;
import Debug.*;


public class MirrorConnectHandler extends CTL.ConnectHandler 
{
	private int id = -1;
	private Socket sock = null;

		
	public MirrorConnectHandler(Socket sock,int id){
		super(null);	
		this.id = id;
		this.sock = sock;
	}
	
	/**
	 * run() is autmatically run, whenever a new thread is created.
	 * It recieves data from the DebugServer, extraxts the header and passes the header and
	 * the rest of the message ,which is stored as bit array, on.
	 */
	synchronized public void run ()
	{
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		
		try
		{
			BufferedInputStream read = new BufferedInputStream(sock.getInputStream());
			OIStream in = new OIStream(read);
			while (sock.isConnected())
			{
				Header head = in.serialRead(Header.class);
				int size = (int)head.getSize();
				System.out.println(id+": "+head);
		
				
				
				if (size > 64*1024*1024)
					return;

				if (size > 0)
				{
					byte[] buf = new byte[size];
					in.read(buf);		
					accept(buf,head,sock);
			
				}
			

			
			if (size==0){ // Wenn im Handshake der MagicString empfangen wird
						  // wird automatisch beantwortet
					 sendContinuous(sock);
					
				}
			}
			sock.close();
		}
		catch (Exception e)
		{
			//e.printStackTrace();
			return;
		}
	}
	
	/**
	 * sends a continuos message to the component connected at the given socket.
	 * @param sock socket at which the connection is established
	 */
	synchronized public void sendContinuous(Socket sock) {
		  
					
		try {
			PeerID tmp = new PeerID(sock.getLocalAddress().toString().substring(1), sock.getPort());
			Header head = new Header(0, Remote.CTRL, tmp, -1);
			OutputStream out0 = sock.getOutputStream();
			OOStream out1 = new OOStream(out0);
			out1.serialWrite(head);
			out1.flush();
		
		} 
		catch (Exception e) {
			System.out.println("Fehler in der Methode MirrorConnectHandler.sendContinuous(Socket sock)");
		}; 
	}
	
	

/**
 * accept is used for further disassembling of the message and evoke methods in the
 * visualizer
 * 
 * @param buf Byte Array of the Payload of the message
 * @param ihead Header of the recieved Message
 * @param sock Socket at which the connection is established
 * @throws IOException
 * @throws IllegalAccessException
 * @throws InvocationTargetException
 * @throws CTLException
 * @throws NoSuchMethodException
 * @throws ClassNotFoundException
 * @throws InstantiationException
 */
	synchronized public void accept (byte[]buf , Header ihead,Socket sock) throws 	IOException, IllegalAccessException,
											InvocationTargetException, CTLException,
											NoSuchMethodException, ClassNotFoundException,
											InstantiationException
{
	Env env = Env.newEnv();
//	Socket socket = sock; // Socket, at which the connection was recieved
	long objID; //ObjId of target object
	String debugger; //name of the debugger-Executable
	int pid = -1; // ProcessID
	int planguage = -1 ;// programming language of the component
	PeerID host = null; // PeerID of the sender
	GroupInfo gi; // GroupInfo of the service recieved during handshake
	FID dummy = new FID((short)0,"answer"); //Dummy FID which is used when adding RMI Answers to the continuous list
	

	

	BuffyIn in = new BuffyIn(buf);
	Header ohead = ihead;

			
	Env.log.msg(Logger.INFO, "Recv header: "+ohead);
	if (ohead.needSwap())
		in.changeEndianess(ByteOrder.LITTLE_ENDIAN);

	switch (ohead.tag())
	{
		

					
		case Remote.OPER: // Wird beim Handshake empfangen , wenn empfangen endet der timeout
			
			EnvDebug.mirror.setHandshakeRunning(false);
			if (ohead.size()>0){
				gi = in.i.serialRead(GroupInfo.class);
//				System.out.println("host wird erstellt");
				EnvDebug.vis.addHost(ohead.pid().toString()); //Peer darstellen lassen
			
			
			if (EnvDebug.mirror.counter==0) 
			{
				//nachricht des clients, folgt dem magicstring-paket
//				System.out.println("EOC ObjID des Clients : "+gi.id());
//				EnvDebug.mirror.setEOCList(gi.pid(),gi.id(),"");
				EnvDebug.mirror.tmp[EnvDebug.mirror.counter] = ohead.pid();
			    EnvDebug.mirror.setPSList(ohead.pid(),sock); //PeerID-socket speichern
				EnvDebug.mirror.setDBG(ohead.pid(),"null",0,0); //PeerID setzen Rest kommt sp??ter
				EnvDebug.mirror.counter++;
//				Erzeugt eine DummyKlasse von GroupDebug, welche beim Beenden des Service angesprochen wird
//				System.out.println("EOC Objekt wird hinzugefuegt");
				Class foo3 = Class.forName("javaSys.GroupDebug"); 
				Object bar2 = foo3.newInstance();
				Env.map.regObj(bar2,(int)gi.id());
//				System.out.println("GroupDebug mit ObjID :"+gi.id()+" registriert");
			}
			else
			{				
				//antwort des service mit GroupInfoObject
//				System.out.println("EOC ObjID des Service : "+gi.id());
				EnvDebug.mirror.setEOCList(gi.pid(),gi.id(),"");
				EnvDebug.mirror.tmp[EnvDebug.mirror.counter] = ohead.pid();
				EnvDebug.mirror.setPSList(ohead.pid(),sock); //PeerID,Socket speichern
				EnvDebug.mirror.addPair(EnvDebug.mirror.tmp[0],EnvDebug.mirror.tmp[1]); //PeerID-PeerID speichern
				EnvDebug.mirror.setDBG(ohead.pid(),"null",0,0); //PeerID setzen die restlichen Infos kommen sp??ter
				EnvDebug.mirror.counter=0;				
//				Erzeugt eine DummyKlasse von GroupDebug, welche beim Beenden des Service angesprochen wird
//				System.out.println("EOC Objekt wird hinzugefuegt");
				Class foo3 = Class.forName("javaSys.GroupDebug"); 
				Object bar2 = foo3.newInstance();
				Env.map.regObj(bar2,(int)gi.id());
//				System.out.println("GroupDebug mit ObjID :"+gi.id()+" registriert");
			}
			
			}
			
			 EnvDebug.mirror.sendContinuous(ohead.pid());
			
			break;
		
		case Remote.RMI: // wird bei Methodenaufrufen verwendet
			
					
			objID = in.i.readLong();// Liest die ObjectID aus der empfangenen Nachricht
//			System.out.println("ObjID des Calls :"+objID);
			procRMI(in.i, ohead, objID, -1, env, "Debug"); // wird zur weiteren Verarbeitung aufgerufen

			break;
			
			
		case Remote.EOC: // Wird DOCH NICHT beim Beenden der Verbindung empfangen. EOC = End of connection
			// Wird nur empfangen wenn der bisher noch nicht unterstützte Daemon-Mode genutzt wird 
//			System.out.println("MirrorConnectHandler - Reception of EOC messages is only supported in Daemon-mode");
			Env.log.msg(Logger.ERR, "EOC is only supported in Daemon-mode.");
			break;

			
		case Remote.CTRL:
			Env.log.msg(Logger.ERR, "CTRL is not yet supported.");
			System.out.println("MirrorConnectHandler - Reception of CTRL messages is not yet supported");
			System.exit(5);
			break;

			
		case Remote.UNDEF:
			Env.log.msg(Logger.ERR, "UNDEF is not yet supported.");
			System.out.println("MirrorConnectHandler - Reception of UNDEF messages is not yet supported");
			System.exit(5);
			break;	
			

		case Remote.DBG: // Wird nach dem Handshake empfangen und die Infos werden nachgetragen
			host = ohead.pid();				
			debugger=in.i.readString();
			pid = in.i.readInt();
			planguage = in.i.readInt();		
			EnvDebug.mirror.updateDBG(host,debugger,pid,planguage);
			
			
			break;
			



		case Remote.SSH: // Timeout fuer den handshake wird gestartet			
						 // die Zeit ist auf 30 Sekunden gesetzt und kann in 
						 // der Klasse Timeout geaendert werden
			
			Timeout t = new Timeout();
			// Wird keine continuous nachricht ben??tigt !
			t.start();
						
			break;
			
			

		default: // wird bei einer empfangenen RMI-Answer und unbekannten Nachrichten aufgerufen
			objID = in.i.readLong();
		
		if (ohead.tag()>255)System.out.println("RMI Answer empfangen");
		
	
		
			Env.log.msg(Logger.INFO, ".:. Received RMI answer .:.");
			Env.log.msg(Logger.INFO, "\tHeader: "+ohead);
			Env.log.msg(Logger.INFO, "\tObjectID: "+objID);

//		Liest das Objekt mit der empfangenen ObjID aus der map aus.
			Object obj = Env.map.getObj((int)objID); 
			if (obj==null)
			{
				Env.log.msg(Logger.ERR, "Object "+objID+" not found.");
				System.exit(10);
			}
	
//			Sicherungsblock, soll drinbleiben .
			if (!obj.getClass().getName().equals("CTL.rResult"))
			{	
				System.out.println("MirrorConnectHandler - Safetyblock in Default sector reached");
				procRMI(in.i, ohead, objID, 1, env, "Debug");
				return;
			}
			
// 			Liest eventuelle Exceptions aus der Nachricht
			Except ex = in.i.serialRead(Except.class);
			if (ex.name().length()>0)
				throw new CTLException("Class not found!");
			
//			Liest das rResult aus
			rResult res = (rResult)obj;	
			for (int i=0;i<res.results();i++)
			{
				res.recv(in.i, i);
			}
			
			
			
//			Pruefen ob zu aktueller PeerID und messagetag ein constrcall vorliegt
			if (EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag())!=null)
			{
				//ein neuer rPointer wird erzeugt und ausgelesen.
				rPointer foo = (rPointer)res.ObjectValue(res.results()-1);
				long ObjectID = foo.objID();
				
//				Wenn ja werden ObjID und PeerID in der IDList gespeichert
				EnvDebug.mirror.setIDList(ohead.pid(),ObjectID,EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag()));
//				Und es wird ein neues Objekt im Visualisierer erzeugt
//				System.out.println("Objekt soll erzeugt werden :"+ohead.pid().toString()+" "+EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag())+" "+ObjectID);
//				if (!EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag()).equals("javaSys.CallByReferenceRI"))
				EnvDebug.vis.addObject(ohead.pid().toString(),EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag()),ObjectID);
//				und es wird ein neues Objekt in die Env.map eingetragen, das auf die DebugKlasse verweist.	
				String eins = EnvDebug.mirror.getConstrName(ohead.pid(),ohead.tag());
				String zwei = eins.substring(0,eins.length()-2); //schneidet die letzten beiden Buchstaben des Klassennamen ab
				zwei = zwei.concat("Debug");					 //und ersetzt sie durch "Debug"			
				Class foo2 = Class.forName(zwei);				 //Erzeugt eine neue Klasse mit dem Namen der Debug Komponente
				Object bar = foo2.newInstance();				 //instanziert ein Objekt der Klasse
				Env.map.regObj(bar,(int)ObjectID);				 //registriert das Objekt in der ObjectMap der CTL
				
			
				
			}
			
				
			
	
	
//			Nachricht wird im Visualisierer erstellt und der dazugehörige Call wird gelöscht.
			
			
//			System.out.println("addMessage :"+ohead.pid().toString()+" "+null+" "+0+" "+ohead.tag()+" "+"answer"+" "+res.toString()+" "+null+" "+EnvDebug.mirror.getPartner(ohead.pid()).toString()+" "+null+" "+objID);
			EnvDebug.vis.addMessage(ohead.pid().toString(),null, 0, ""+ohead.tag(), "answer", res.toString(),
			        null, EnvDebug.mirror.getPartner(ohead.pid()).toString(), null,
			        0 );
		
			
//			Gibt bei aktiviertem More Modus die Rueckgabewerte aus
			if (EnvDebug.mirror.more)
				System.out.println("result :"+res.toString());
			
			
			//Prueft ob automatisch geantwortet werden soll, oder ob der Absender in die Continuous Liste eingefuegt wird
			if (EnvDebug.mirror.getRunmode())
				EnvDebug.mirror.sendContinuous(ohead.pid());
			else
				EnvDebug.mirror.setContinuousList(ohead.pid(),dummy,"answer",false); // Dummy is used to mark is as an answer
			
			
			
			
				

 
			break;
			
	}
	}
	
	
	
}
