/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Debugger;

import java.awt.event.*;
import java.util.LinkedList;
import CTL.Types.IPaddr;
import CTL.Types.PeerID;
import Debug.EnvDebug;
import Debug.PDBGContainer;

/**
 * Processes the user-actions performed by Prompt
 * 
 * @author Stephan Kluge
 *
 */
public class PromptEvent extends WindowAdapter implements ActionListener, KeyListener {
    
    private Prompt prompt;					//Referenz der klasse Prompt
    private Thread run,
    			   search_breakpoints,
    			   next_timer;
    private CommandBreakOn breakpoint;
    private String command,					//Befehl des Benutzers
    			   component = null;		//Komponente, die der Benutzer mit einem Befehl eingegeben hat
    private short breakpoint_started = 0,	//Hilfsvariable, damit berprft werden kann, ob der Befehl "break on" gerade ausgefhrt wird
                  saved_commands_count = -1,
                  mode = 0;
    private boolean	command_run_started,	//Hilfsvariable, damit berprft werden kann, ob schon ein Programm luft
    				constructor = true;
    private LinkedList<String> saved_commands;
    
    
    /**
     * Constructs the reference to the classes Prompt and Mirror
     * @param prompt A reference to the class Prompt
     * @param mirror A reference to the class Mirror
     */
    public PromptEvent(Prompt prompt) {
        this.prompt = prompt;
        breakpoint = new CommandBreakOn();
        this.saved_commands = new LinkedList();
    }
    
    /**
     * Creates a window, who asks the user if the program really
     * shall be close
     */
    public void windowClosing(WindowEvent we) {
        CloseConfirmWindow confirm_window = new CloseConfirmWindow();
        confirm_window.setVisible(true);
    }
    
    /**
     * Responds to user-actions performed by Prompt
     * @param ae A reference to the class ActionEvent to know which action
     * became executed
     */
    public void actionPerformed(ActionEvent ae) {
        if(ae.getSource() == prompt.textfield) {
            command = prompt.textfield.getText();
            saved_commands_count = -1;
            saved_commands.addFirst(command);
            prompt.textfield.setText("");
            prompt.textfieldGray();
        	prompt.textfield.setEditable(false);
            
        	if(breakpoint_started == 0) {
            	compareCommand();
            }
            else if(breakpoint_started == 1) {
                choseMethod();
            }
        }
    }
    
    /**
     * Responds to keyboard-actions, if a key is pressed
     * @param ke A reference to the class ActionEvent to know which key
     * is pressed
     */
    @SuppressWarnings("deprecation")
	public void keyPressed(KeyEvent ke) {
        if(breakpoint_started == 2) {
            if(ke.getKeyCode() == ke.VK_CONTROL) {
                prompt.textarea.append("\n\ncanceled by user");
                search_breakpoints.stop();
                prompt.textfieldWhite();
                breakpoint_started = 0;
                prompt.textfield.setEditable(true);
            }
        }
        
        if(breakpoint_started == 3) {
            if(ke.getKeyCode() == ke.VK_CONTROL) {
                prompt.textarea.append("\n\ncanceled by user");
                next_timer.stop();
                prompt.textfieldWhite();
                breakpoint_started = 0;
                prompt.textfield.setEditable(true);
            }
        }
        
        if(ke.getKeyCode() == ke.VK_UP) {
        	if(saved_commands.size() != 0) {
        		try {
        			saved_commands_count++;
        			prompt.textfield.setText(saved_commands.get(saved_commands_count));
        		}
        		catch(IndexOutOfBoundsException ioobe) {
        			saved_commands_count--;
        			prompt.textfield.setText(saved_commands.get(saved_commands_count));
        		}
        	}
        }
        
        if(ke.getKeyCode() == ke.VK_DOWN) {
        	if(saved_commands.size() != 0) {
        		try {
        			saved_commands_count--;
        			prompt.textfield.setText(saved_commands.get(saved_commands_count));
        		}
        		catch(IndexOutOfBoundsException ioobe) {
        			saved_commands_count++;
        			prompt.textfield.setText(saved_commands.get(saved_commands_count));
        		}
        	}
        }
    }
    
    /**
     * Compares the user-command with the commands in this method and
     * executes the command if it is equal
     */
    public void compareCommand() {
        if(command.equals("list -breakpoints")) {
        	prompt.textarea.append(breakpoint.getBreakpointList());
        	prompt.textfield.setEditable(true);
        	prompt.textfieldWhite();
        }
        else if(command.equals("next"))
            next();
        else if(command.equals("more"))
        	more();
        else if(command.equals("list"))
            list();
        else if(command.equals("debug"))
            debug();
        else if(command.equals("help"))
            help();
        else if(command.equals("exit"))
        	System.exit(0);
        else {
            component = SplitString.lastExpression(command, ' ');
            command = SplitString.firstExpressions(command, ' ');
            
            
            if(command.equals("run") && component != null) {
                if(!command_run_started)
                    runprog();
                else {
                	prompt.textarea.append("\n\nanother program is currently running");
                	prompt.textfield.setEditable(true);
                    prompt.textfieldWhite();
                }
            }
            else if(command.equals("break on") && component != null)
                breakOn();
            else if(command.equals("next") && component != null)
            	nextArg();
            else if(command.equals("setmode") && component != null)
            	setmode();
            else {
                prompt.textarea.append("\n\ncommand unknown\ntype help for help");
                prompt.textfield.setEditable(true);
                prompt.textfieldWhite();
            }
        }
    }
    
    
    
    /**
     * Starts the searching of a breakpoint, after the user has chosen
     * between multiple methods with the same name
     */
    public void choseMethod() {
        try {
            int tmp = Integer.parseInt(command);
            if(tmp >= 1 && tmp <= breakpoint.index.size()) {
            	if(constructor)
            		breakpoint.setBreakpoint((short)(tmp-1), true);
            	else
            		breakpoint.setBreakpoint((short)(tmp-1), false);
            	prompt.textarea.append("\n\nBreakpoint added");
            }	
            else {
                prompt.textarea.append("\n\nillegal command!");
            }
        }
        catch(NumberFormatException nfe) {
            prompt.textarea.append("\n\nillegal command!");
        }
        breakpoint_started = 0;
        prompt.textfield.setEditable(true);
        prompt.textfieldWhite();
    }
    
    
    
    /**
     * Is executed if the command "run" is given
     */
    public void runprog() {
        run = new CommandRun(this, prompt, component);
        command_run_started = true;
        run.start();
        if(mode != 0) {
        	prompt.textfield.setEditable(true);
        	prompt.textfieldWhite();
        }
    }
    
    
    
    /**
     * Is executed if the command "next" is given
     * Creates a Thread and start it
     */
    public void next() {
        if(mode == 1) {
        	if(breakpoint.breakpointlist.size() == 0) {
        		prompt.textarea.append("\n\nlist contains no breakpoints \ncommand canceled!");
        		prompt.textfield.setEditable(true);
        		prompt.textfieldWhite();
        	}
        	else {
        		breakpoint_started = 2;
        		search_breakpoints = new SearchBreakpoints(this, prompt, breakpoint);
        		search_breakpoints.start();
        	}
        }
        else if(mode == -1 || mode == 0) {
        	EnvDebug.mirror.sendContinuous();
        	prompt.textfield.setEditable(true);
        	prompt.textfieldWhite();
        }
    }
    
    
    
    /**
     * Is executed if the command "break on «package».«class».«method name»" is given
     * «method name» is optional, because of constructors
     * It returns a message to the user per prompt if the method or class was not
     * found, or the method is multiple. Otherwise the searching of the breakpoint starts
     */
    public void addBreakOnMethod() {
    	String method_name = "";
    	
    	method_name = SplitString.lastExpression(component, '.');
    	component = SplitString.firstExpressions(component, '.');
    	
    		short tmp = breakpoint.searchMultipleMethods(component, method_name);

    		if(tmp == 0) {
    			prompt.textarea.append("\n\nobject unknown");
    		}
    		else if(tmp == 1) {
    			breakpoint.setBreakpoint((short)0, false);
    			prompt.textarea.append("\n\nBreakpoint \""
    								   +breakpoint.breakpointlist.getLast().getClasspath()
    								   + "."
    								   +breakpoint.breakpointlist.getLast().getMethod()
    								   + "\" added");
    		}
    		else if(tmp > 1) {
    			prompt.textarea.append("\n\nmultitple methods with this name\n");
    			prompt.textarea.append("Enter the corresponding number " +
    									"to select the wanted method: \n\n");
    			prompt.textarea.append(breakpoint.getMethodsAsString());
    			constructor = false;
    			breakpoint_started = 1;
    		}
        prompt.textfieldWhite();
    	prompt.textfield.setEditable(true);
    }
    
    
    /**
     * Is executed if the command "break on «package».«class» is given
     * It returns a message to the user per prompt if the constructor was not
     * found, or there are multiple constructors . Otherwise the searching of the breakpoint starts
     */
    public void breakOn() {
    	try {
    		short tmp = breakpoint.searchClass(component);
    		if(tmp == 0) {
    			prompt.textarea.append("\n\nobject unknown!!!");
    		}
    		else if(tmp == 1) {
    			breakpoint.setBreakpoint((short)0, true);
    			prompt.textarea.append("\n\nBreakpoint \""
    								   +breakpoint.breakpointlist.getLast().getClasspath()
    								   + "."
    								   +breakpoint.breakpointlist.getLast().getMethod()
    								   + "\" added");
    		}
    		else if(tmp > 1) {
    			prompt.textarea.append("\n\nmultitple constructors with this name\n");
    			prompt.textarea.append("Enter the corresponding number " +
    									"to select the wanted constructor:\n\n");
    			prompt.textarea.append(breakpoint.getConstructorsAsString());
    			breakpoint_started = 1;
    			constructor = true;
    		}
    		prompt.textfieldWhite();
    		prompt.textfield.setEditable(true);
    	}
    	catch(ClassNotFoundException cnfe) {
    		addBreakOnMethod();
    	}
    }
    
    
    
    /**
     * lists the components currently stored in the continuous list
     */
    public void list() {
    	String list = "\n\n\nContinuous-List:\n\n";
    	for(int i = 0; i < EnvDebug.mirror.continuous_list.size(); i++) {
    		list = list+ "\n" +EnvDebug.mirror.continuous_list.get(i).getPeerID();
    	}
    	prompt.textarea.append(list);
        prompt.textfield.setEditable(true);
        prompt.textfieldWhite();
    }
    
    
    
    /**
     * attaches a monolithic debugger to the method at the found breakpoint
     */
    public void debug() {
    	PeerID peerid = EnvDebug.mirror.getPartner(EnvDebug.mirror.continuous_list.getFirst().getPeerID());
    	IPaddr host = peerid.host();
    	int pid = ((PDBGContainer)EnvDebug.mirror.getDBGInfo(peerid)).getPid();
    	String dbg = ((PDBGContainer)EnvDebug.mirror.getDBGInfo(peerid)).getDebug();
    	String cmd = "";
    	if(pid == -1)
    		prompt.textarea.append("\n\n\nDebugging not possible!");
    	else {
    		if(dbg.equals("jdb"))
    			cmd = "xterm -hold -e ssh " +host+ " jdb -attach localhost:" +pid;
    		else
    			cmd = "xterm -hold -e ssh " +host+ " " +dbg+ " -p " +pid;
    	
    		System.out.println(cmd);
    		try {
    			System.out.println(NCLib.Process.exec(cmd));
    		}
    		catch (Exception e) {
    			System.out.println("Could not start process:");
    			e.printStackTrace();
    			System.exit(1);
    		}
    	}
        prompt.textfield.setEditable(true);
        prompt.textfieldWhite();
    }
    
    
    /**
     * Is executed if the command "more" is given
     */
    public void more() {
    	if(EnvDebug.mirror.more) {
    		EnvDebug.mirror.more = false;
    		prompt.textarea.append("\n\nmore off");
    	}
    	else {
    		EnvDebug.mirror.more = true;
    	prompt.textarea.append("\n\nmore on");
    	}
        prompt.textfield.setEditable(true);
        prompt.textfieldWhite();
    }
    
    
    
    public void nextArg() {
    	if(mode == -1) {
    		if(command_run_started == true) {
    			try {
    				int time = Integer.parseInt(component);
    				next_timer = new CommandNextTimer(time, prompt);
    				next_timer.start();
    				breakpoint_started = 3;
    				prompt.textfield.setEditable(false);
    				prompt.textfieldGray();
    			}
    			catch(NumberFormatException nfe) {
    				prompt.textarea.append("\n\nunknown command");
    				prompt.textfield.setEditable(true);
    		        prompt.textfieldWhite();
    			}
    		}
    		else {
    			prompt.textarea.append("\n\nprogram not started");
    			prompt.textfield.setEditable(true);
    	        prompt.textfieldWhite();
    		}
    	}
    	else {
    		prompt.textarea.append("\n\nmode is not \"next\"");
    		prompt.textfield.setEditable(true);
            prompt.textfieldWhite();
    	}
    }
    
    
    
    /**
     * gives a help for the debugger in the textarea of the prompt
     *
     */
    public void help() {
        prompt.textarea.append("\n\ncommands:" +
                               "\n\nrun <program>          starts the given program. \"run Example.Client3\"" +
                               "\nbreak on <breakpoint>    sets a breakpoint. <breakpoint> must be a method or a constructor " +
                                                           "\"break on Impl.CTest\" or \"break on Impl.CTest.add\"" +
                               "\nnext                     continuoues the send of messages in the distributed system" +
                               "\nnext <time>              sends messages automatically in an intervall. <time> is intervall in seconds" +
                               "\nmore                     sets more \"on\" and \"off\". If is it \"on\", it gives information about" +
                                                           "the parameters who are delivered by RMI-Call" +
                               "\nlist                     list the continuous-list" +
                               "\nlist -breakpoints        list the breakpoints" +
                               "\ndebug                    starts a monolithic debugger who debug at a breaked method" +
                               "\nsetmode <mode>           changes the runmode of the debugger. <mode> can be \"next\", \"run\", and \"break on\"" +
                               "\n exit                    closes the debugger" +
                               "\nhelp                     starts this help");
    }
    
    
    /**
     * changes the run mode to next,break or run
     *
     */
    public void setmode() {
    	if(component.equals("next")) {
    		mode = -1;
    		EnvDebug.mirror.setRunmode(false);
    		prompt.textarea.append("\n\nmodus changed: next");
    	}
    	else if(component.equals("break")) {
    		mode = 1;
    		EnvDebug.mirror.setRunmode(false);
    		prompt.textarea.append("\n\nmodus changed: break");
    	}
    	else if(component.equals("run")) {
    		mode = 0;
    		EnvDebug.mirror.setRunmode(true);
    		prompt.textarea.append("\n\nmodus changed: run");
    	}
    	else {
    		prompt.textarea.append("\n\ncommand unknown");
    	}
    	prompt.textfield.setEditable(true);
        prompt.textfieldWhite();
    }
    
    
    
    /**
     * 
     * @return Returns the String component
     */
    public String getComponent() {
        return component;
    }
    
    /**
     * 
     * @return true if constructor
     */
    public boolean getConstructor() {
    	return constructor;
    }
    
    
    
    /**
     * Sets the boolean breakpoint_started
     * @param breakpoint_started Is true if a thread is execute
     */
    public void setBreakpointStarted(short breakpoint_started) {
        this.breakpoint_started = breakpoint_started;
    }
    
    /**
     * Signals that a program is currently running
     * @param command_run_started
     */
    public void setCommandRunStarted(boolean command_run_started) {
    	this.command_run_started = command_run_started;
    }
    
    
    public void keyTyped(KeyEvent ke) {}
    public void keyReleased(KeyEvent ke) {}
}
