/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug;
import java.util.*;
import java.io.OutputStream;
import java.lang.reflect.*;
import java.net.Socket;

import CTL.Remote;
import CTL.Streams.OOStream;
import CTL.Types.*;


/**
 * Mirror manages the communication between the Debugger and the rest of 
 * the distributed system. It also contains several lists expacially the continuous list, where the 
 * components, waiting for a continuous message, are saved.
 * 
 * @author Bjoern Gebhardt, Johannes Baumgarten, Jurij Merker, Stephan Kluge, Svetlana Li
 */
public class Mirror {
	
	/**	temporary variables for MirrorConnectHandler */
	public PeerID[] tmp = new PeerID[2];
	public int counter = 0;
	public boolean handshakerunning = false; //prüft ob Kommunikation ein handshake ist, oder nicht
	public boolean runmode_enabled = true; // wenn true werden nachrichten automatisch versandt
	public boolean more = false; //false >> keine weiteren Informationen zur Kommunikation
	
	
	/**	create some linked lists, contains exspecially types of objects*/
	public LinkedList<CListContainer> continuous_list;	
	private LinkedList<PMContainer> constr_list;				   
	private LinkedList<OPIDContainer> id_list;
	private LinkedList<PDBGContainer> debug_list;
	private LinkedList<LinkedList<PeerID>> peerid_list;
	private LinkedList<PSContainer> ps_list;
	private LinkedList<OPIDContainer> eoc_list;
	private LinkedList<MTagContainer> mtag_list;
	private LinkedList<CompoContainer> compo_list;
	private LinkedList<IPaddr> host_list;
	
	/** 
	 * create a reference to linked lists
	 */
	public Mirror() {
		this.continuous_list = new LinkedList<CListContainer>();
		this.debug_list = new LinkedList<PDBGContainer>();
		this.peerid_list = new LinkedList<LinkedList<PeerID>>();
		this.id_list = new LinkedList<OPIDContainer>();
		this.constr_list = new LinkedList<PMContainer>();
		this.ps_list = new LinkedList<PSContainer>();
		this.eoc_list = new LinkedList<OPIDContainer>();
		this.mtag_list = new LinkedList<MTagContainer>();
		this.compo_list = new LinkedList<CompoContainer>();
	}
	
	/**
	 * send continuous messages to all components in the continuous list
	 */
	/*
	 * Zuerst wird die aktuelle Anzahl an Listenelementen gespeichert.
	 * Ist wichtig, da waehrend dem Abarbeiten neue Elemente in die Liste kommen
	 * koennten und sich die Anzahl Schleifendurchlaeufe dadurch aendert, wenn man
	 * im Schleifenkopf "continuous_list.size()" statt "size" angibt. Es wuerden
	 * dann auch die Elemente abgearbeitet, die zum Zeitpunkt des Befehls noch
	 * nicht in der Liste waren
	 * Die Schleife wird so oft durchlaufen wie die Anzahl der Elemente in der
	 * Liste zum Zeitpunkt des Befehls war.
	 */
	synchronized public void sendContinuous() {
		int size = continuous_list.size();
		//System.out.println("size: "+size);
		//System.out.println("cl.size: "+continuous_list.size());
		//System.out.println(continuous_list.get(0).getPeerID());
		//System.out.println(continuous_list.get(1).getPeerID());
		
		for(int i=0; i<size; i++) {
		    //System.out.println("Continuous wird an: "+continuous_list.get(i).getMethodName()+" geschickt ...");
		    sendContinuous(continuous_list.getFirst().getPeerID());
		    continuous_list.removeFirst();
		}
	}
	
	/**
	 * send a continuous message to the frist element in the continuouslist and remove it after sending
	 */
	synchronized public void sendContinuousToFirstElement() {
		if(continuous_list.size() > 0) {
		    sendContinuous(continuous_list.getFirst().getPeerID());
		    continuous_list.removeFirst();
		}
	}
	
	/**
	 * send a continuous message to the component with the handed over 
	 * @param peerid - PeerID
	 */
	/*
	 * Versendet eine Continuous-Nachricht an das erste Listenelement
	 * und loescht dieses anschlieend aus der Liste
	 */
	synchronized public void sendContinuous(PeerID peerid) {
	  	//Versenden der Continuous-Nachricht an das entsprechende Listenelement mit dieser PeerID
//		System.out.println("Continuous Nachricht wird versandt");
		try {
			
			if (getisVoid(peerid)&& !runmode_enabled)// L�scht die Nachricht, wenn es ein void-call ist
			{
				System.out.println("Void Message wird geloescht");
				EnvDebug.vis.removeMessage(peerid.toString(),null,0,""+getMTag(peerid),getPartner(peerid).toString(),null,0);
				removePeerIDElement(peerid);//loescht den Eintrag aus der MTagListe
			}
			
			//Loescht die Answer und den dazugehoerigen Call aus dem Visualisierer im next oder run modus
			if (continuous_list.size()>0 && getFID(peerid).name().equals("answer"))
			
			{
//				System.out.println("Answer und call werden geloescht");
				int mtag = getMTag(getPartner(peerid));
//				System.out.println("removeMessage :"+getPartner(peerid).toString()+" "+null+0+" "+mtag+" "+peerid.toString()+null+0);
				EnvDebug.vis.removeMessage(getPartner(peerid).toString(),null,0,""+mtag,peerid.toString(),null,0); //call
//				System.out.println("removeMessage :"+peerid.toString()+" "+null+0+" "+mtag+" "+getPartner(peerid).toString()+null+0);
				EnvDebug.vis.removeMessage(peerid.toString(),null,0,""+mtag,getPartner(peerid).toString(),null,0); //answer
				removePeerIDElement(getPartner(peerid)); //loescht den Eintrag aus der MTagListe
			}
			
			PeerID tmp = new PeerID(getSocket(peerid).getLocalAddress().toString().substring(1), getSocket(peerid).getPort());
			Header head = new Header(0, Remote.CTRL, tmp, -1);
			OutputStream out0 = getSocket(peerid).getOutputStream();
			OOStream out1 = new OOStream(out0);
			out1.serialWrite(head);
			out1.flush();
	
			// wenn es sich um einen EOC call handelt werden beide hosts gel�scht und die isten von den PeerIds bereinigt
			
			if (getEOC(peerid))
			{
//				System.out.println("EOC gefunden");
				EnvDebug.vis.removeHost(getPartner(peerid).toString());
				EnvDebug.vis.removeHost(peerid.toString());
				PeerID temp2 = getPartner(peerid);
//				System.out.println("Service :"+temp2.toString());
				removeEOC(temp2);
				removePeerID(temp2);
				removePeerID(peerid);
			
			}
		} 
		catch (Exception e) {
			
			System.out.println("Versenden fehlgeschlagen!");
			
		}
		
	}
	
	/**
	 * remove element from mtag_list, which belongs to the given PeerID
	 * @param peerid - PeerID
	 */
	
	synchronized public void removePeerIDElement(PeerID peerid) {
		
		try {
			
			for (int i=0; i<mtag_list.size(); i++) {
				if (mtag_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					mtag_list.remove(i);
					break;
				}
			}
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Löschen einer PeerID: "+peerid.toString()+" Mirror.removePeerIDElement()");
			
		}
	}
	
	/**
	 * remove all elements in lists, where the handed over PeerID exists
	 * @param peerid - handed over PeerID
	 */
	synchronized public void removePeerID (PeerID peerid) {
	
		try {
			for (int i=0; i<peerid_list.size(); i++) {
				for (int j=0; j<peerid_list.get(i).size(); j++) {
					if (peerid_list.get(i).get(j).toString().equals(peerid.toString())) {
						peerid_list.remove(i);
						break;
					}
				}			
			}
			
			for (int i=0; i<id_list.size(); i++) {
				if (id_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					id_list.remove(i);
//					i=-1;
					i--;
				}
			}
			
			for (int i=0; i<constr_list.size(); i++) {
				if (constr_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					constr_list.remove(i);
					//i=-1;
					i--;
				}
			}
		
			for (int i=0; i<debug_list.size(); i++) {
				if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					debug_list.remove(i);
//					i=-1;
					i--;
				}
			}
			
			for (int i=0; i<ps_list.size(); i++) {
				if (ps_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					ps_list.remove(i);
//					i=-1;
					i--;
				}
			}
			
			for (int i=0; i<compo_list.size(); i++) {
				if (compo_list.get(i).getHost().equals(peerid.host())) {
					compo_list.remove(i);
//					i=-1;
					i--;
				}
			}
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Loeschen einer PeerID: "+peerid.toString()+" Mirror.removePeerID()");
			
		}
			
	}
	
	/**
	 * remove the elements in eoc_list, which belongs to the given PeerID
	 * @param peerid - handed over PeerID
	 */	
	synchronized public void removeEOC (PeerID peerid) {
		
		try {
			for (int i=0; i<eoc_list.size(); i++) {
				if (eoc_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					eoc_list.remove(i);
					break;
				}
			}
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Loeschen eines EOC-Elements: "+peerid.toString()+" Mirror.removeEOC()");
			
		}
			
	}
	
	/**
	 * call the the updateDBG-method in a reference of type PDBGcomponent, 
	 * if the handed over PeerID equals with the instanced PeerID of a component
	 * @param peerid - PeerID
	 * @param debugger - debugger instruction
	 * @param pid - ProcessID
	 * @param plang - component language of programming
	 */
	synchronized public void updateDBG(PeerID peerid, String debugger, int pid, int plang) {
		for (int i=0; i<debug_list.size(); i++) {
			if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
				debug_list.get(i).updateDBG(debugger, pid, plang);
			}
		}
	}
	
	/**
	 * add a pair of communicating PeerIDs in a list
	 * @param peerid1 - PeerID 1
	 * @param peerid2 - PeerID 2
	 */
	synchronized public void addPair(PeerID peerid1, PeerID peerid2) {
		
		LinkedList<PeerID> addfolder = new LinkedList<PeerID>();
		addfolder.addLast(peerid1);
		addfolder.addLast(peerid2);
		
		peerid_list.addLast(addfolder);
		
	}
	
	/**
	 * @param tmp - PeerID
	 */
	public void setPeerID(PeerID pid, int count) {
		
		this.tmp[count] = pid;
	
	}
	
	/**
	 * @param counter - temporary counter for the array
	 */
	public void setCounter(int counter) {
		
		this.counter = counter;
	
	}
	
	/**
	 * @param hrun - timeout in handshake
	 */
	public void setHandshakeRunning(boolean hrun) {
		
		this.handshakerunning = hrun;
	
	}
	
	/**
	 * @param runmode of the bebugger (true = normal running)
	 */
	public void setRunmode(boolean runmode) {
		
		this.runmode_enabled = runmode;
		
	}
	
	/**
	 * set list, which contains the most important informations for debugging a component 
	 * @param peerid - PeerID
	 * @param debugger - debugger instruction 
	 * @param pid - ProcessID
	 * @param plang - component language of programming
	 */
	synchronized public void setDBG(PeerID peerid, String debugger, int pid, int plang) {
	
		debug_list.addLast(new PDBGContainer(peerid, debugger, pid, plang));
	
	}
	
	/**
	 * add some parameters to a listelement in mtag_list
	 * @param peerid - PeerID
	 * @param mtag - message tag
	 * @param isvoid - if method is void or not
	 */
	synchronized public void setMTagList(PeerID peerid, int mtag, boolean isvoid) {
		
		mtag_list.addLast(new MTagContainer(peerid, mtag, isvoid));
	
	}
	
	/**
	 * add a PeerID and its associated Socket to a list
	 * @param peerid - PeerID
	 * @param socket - Socket
	 */
	synchronized public void setPSList (PeerID peerid, Socket socket) {
	
		ps_list.addLast(new PSContainer(peerid, socket));
		
	}
	
	/**
	 * add a PeerID and its associated terminating ObjectID to a list
	 * @param peerid - PeerID
	 * @param objid - ObjectID
	 */
	synchronized public void setEOCList (PeerID peerid, long objid,String name) {
		
		eoc_list.addLast(new OPIDContainer(peerid, objid,name));
		
	}
	
	/**
	 * add a handed over PeerID and ObjID to a list
	 * @param peerid - PerrID
	 * @param objid - ObjectID
	 */
	synchronized public void setIDList(PeerID peerid, long objid, String name) {
		
		id_list.addLast(new OPIDContainer(peerid, objid, name));
		
	}
	
	/**
	 * add a ObjID, PeerID and a classname to the constructor list
	 * @param mtag - message tag
	 * @param peerid - PeerID
	 * @param objectname - name of object, as given by fid._class()
	 */
	synchronized public void setConstrList(PeerID peerid, int mtag, String name) {
		
		constr_list.addLast(new PMContainer(peerid, mtag, name));
		
	}
	
	/**
	 * add a clearly characteristic from components, waiting for a continuous message, to a list
	 * @param peerid - PeerID
	 * @param fid - FunctionID
	 * @param eoc - end of connection (true or false)
	 */
	synchronized public void setContinuousList(PeerID peerid, FID fid, String method_name, boolean eoc) {
	
		continuous_list.addLast(new CListContainer(peerid, fid, method_name, eoc));
	
	}
	
	synchronized public void setCompoList(IPaddr host, String classname) {
		
		compo_list.addLast(new CompoContainer(host, classname));
		
	}
	
	synchronized public void setHostList(IPaddr host) {
		
		host_list.addLast(host);
		
	}
		
	/**
	 * search the obejct of type PDBGContainer with the assciated PeerID 
	 * @param peerid - PeerID
	 * @return object of type PDBGContainer
	 */
	synchronized public PDBGContainer getDBGInfo(PeerID peerid) {
		
		try {
				
			for (int i=0; i<debug_list.size(); i++) {
				if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return debug_list.get(i);
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Debuginfos - Mirror.getDBGInfo()");
			return null;
		}
		
	}
	
	/**
	 * search the information of the needed debugger with the assciated PeerID 
	 * @param peerid - PeerID
	 * @return string which debugger have to be use
	 */
	synchronized public String getDBGInfoDBG(PeerID peerid) {
		
		try {
					
			for (int i=0; i<debug_list.size(); i++) {
				if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return debug_list.get(i).getDebug();
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Debuginfos - Mirror.getDBGInfoDBG()");
			return null;
		}
		
	}

	/**
	 * search the ProcessID with the assciated PeerID in dubug_list
	 * @param peerid - PeerID
	 * @return ProcessID
	 */
	synchronized public int getDBGInfoPID(PeerID peerid) {
		
		try {
					
			for (int i=0; i<debug_list.size(); i++) {
				if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return debug_list.get(i).getPid();
				}
			}
			
			return -2;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Debuginfos - Mirror.getDBGInfoPID()");
			return -2;
		}
		
	}
	
	/**
	 * search the programming language in form of an integer with assciated PeerID 
	 * @param peerid - PeerID
	 * @return language of programming
	 */
	synchronized public int getDBGInfoPLANG(PeerID peerid) {
		
		try {
				
			for (int i=0; i<debug_list.size(); i++) {
				if (debug_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return debug_list.get(i).getLanguage();
				}
			}
			
			return -1;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Debuginfos - Mirror.getDBGInfoPLANG()");
			return -1;
		}
		
	}
	
	/**
	 * search the message tag, which belongs to the given PeerID
	 * @param peerid -PeerID
	 * @return message tag
	 */
	synchronized public int getMTag(PeerID peerid) {
		
		try {
			
			for (int i=0; i<mtag_list.size(); i++) {
				if (mtag_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return mtag_list.get(i).getMTag();
				}
			}
			
			return 0;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen des messagetags - Mirror.getMTag()");
			return 0;
		}	
	
	}
	
	/**
	 * return true, if the method is void, wich belongs to the given PeerID 
	 * @param peerid - PeerID
	 * @return message tag
	 */
	synchronized public boolean getisVoid(PeerID peerid) {
		
		try {
			
			for (int i=0; i<mtag_list.size(); i++) {
				if (mtag_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return mtag_list.get(i).getisVoid();
				}
			}
			
			return false;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen des void-Status - Mirror.getisVoid()");
			return false;
		}	
	
	}
	
	/**
	 * search with the PeerID associated terminating ObjectID  
	 * @param peerid - PeerID
	 * @return ObejctID
	 */
	synchronized public long getEOCObjID(PeerID peerid) {
		try {
			
			for (int i=0; i<eoc_list.size(); i++) {
				if (eoc_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return eoc_list.get(i).getObjID();
				}
			}
			
			return 0;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der ObjID - Mirror.getEOCObjID()");
			return 0;
		}	
	}
	
	/**
	 * get the socket for a given PeerID
	 * @param peerid - PeerID
	 * @return socket - socket
	 */
	synchronized public Socket getSocket(PeerID peerid) {
		try{
				
			for (int i=0; i<ps_list.size(); i++) {
				if (ps_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return ps_list.get(i).getSocket();
				}
			}
		
			return null;
		}
		catch (Exception e){
			System.out.println("Fehler beim auslesen des Sockets Mirror.getSocket(PeerID peerid)");
			return null;
		}
	}
		
	/**
	 * return the partner-PeerID of the given PeerID
	 * @param peerid
	 * @return the other PeerID
	 */
	synchronized public PeerID getPartner(PeerID peerid) {
		
		try {
				
			for (int i=0; i<peerid_list.size(); i++) {
				for (int j=0; j<peerid_list.get(i).size(); j++) {
					if (peerid_list.get(i).get(j).toString().equals(peerid.toString())) {
						return peerid_list.get(i).get((j+1)%2);
					}
				}			
			}
		
			return null;
		}
		catch (Exception e){
			System.out.println("Fehler beim Auslesen der PeerID des Partners - Mirror.getPartner()");
			return null;
		}
	}

	/**
	 * search with the ObjectID associated PeerID  
	 * @param objid - ObjectID
	 * @return PeerID
	 */
	synchronized public PeerID getPeerID(long objid) {
	
		try {
		
			for (int i=0; i<id_list.size(); i++) {
				if (id_list.get(i).getObjID() == objid) {
					return id_list.get(i).getPeerID();
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der PeerID - Mirror.getPeerID()");
			return null;
		}	
	}
	
	/**
	 * search with the ObjectID associated classname  
	 * @param objid - ObjectID
	 * @return name of class
	 */
	synchronized public String getClassName(long objid) {
	
		try {
					
			for (int i=0; i<id_list.size(); i++) {
				if (id_list.get(i).getObjID() == objid) {
					return id_list.get(i).getClassName();
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Klassennamens - Mirror.getClassName()");
			return null;
		}	
	}
	
	/**
	 * search with the PeerID associated classname  
	 * @param peerid - PeerID
	 * @return name of class
	 */
	synchronized public String getClassName(PeerID peerid) {
		
		try {
					
			for (int i=0; i<id_list.size(); i++) {
				if (id_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return id_list.get(i).getClassName();
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der Klassennamens - Mirror.getClassName()");
			return null;
		}	
	}
	
	/**
	 * search the name of object, which belongs to the given PeerID
	 * @param peerid - PeerID
	 * @param mtag - messagetag
	 * @return name of obejct as given by fid._class()
	 */
	synchronized public String getConstrName (PeerID peerid, int mtag)
	{
		try {
			
			for (int i=0; i<constr_list.size(); i++) {
				if (constr_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					if (constr_list.get(i).getMessageTag() == mtag) {
						return constr_list.get(i).getName();
					}
				}
			}	
		
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen des Namens - Mirror.getConstrName()");
			return null;
		}		
	}
	
	/**
	 * search the FunctionID, which belongs to the given PeerID saved in the continuouslist
	 * @param peerid
	 * @return FunctionID
	 */
	synchronized public FID getFID (PeerID peerid) {
		
		try {
					
			for (int i=0; i<continuous_list.size(); i++) {
				if (continuous_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return continuous_list.get(i).getFunctionID();
				}
			}
			
			return null;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen der FunctionID - Mirror.getFID()");
			return null;
		}	
	}
	
	/**
	 * @param peerid - PeerID
	 * @return boolean parameter (if the call is an end of connection call or not)
	 */
	synchronized public boolean getEOC (PeerID peerid) {
		
		try {
					
			for (int i=0; i<continuous_list.size(); i++) {
				if (continuous_list.get(i).getPeerID().toString().equals(peerid.toString())) {
					return continuous_list.get(i).isEOC();
				}
			}
			
			return false;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen des EOC - Mirror.getEOC()");
			return false;
		}	
	}
	
	/**
	 * 
	 * @param host
	 * @param classname
	 * @return
	 */
	synchronized public boolean getComponent(IPaddr host, String classname) {
		
		try {
//			System.out.println("!!!");
//			System.out.println(compo_list.size());
			
			for (int i=0; i<compo_list.size(); i++) {
				if (compo_list.get(i).getClassName().equals(classname) && compo_list.get(i).getHost().equals(host)) {
					return true;
				}
			}
			
			return false;
		}
		catch (Exception e) { 
			System.out.println("Fehler beim Auslesen des EOC - Mirror.getComponent()");
			return false;
		}
		
	}
	
	/**
	 * @return PeerID array
	 */
	public PeerID getPeerID(int count) {
		
		return tmp[count];
	
	}
	
	/**
	 * @return counter for tmp (array of PeerIDs)
	 */
	public int getCounter() {
		
		return counter;
	
	}
	
	/**
	 * @return timeout in handshake
	 */
	public boolean getHandshakeRunning() {
		
		return handshakerunning;
	
	}
	
	/**
	 * @return runmode of the debugger in boolean form (true = normal running)
	 */
	public boolean getRunmode() {
		
		return runmode_enabled;
		
	}
	
	/**
	 * return the list of PeerID and ObjID
	 * @return id_list
	 */
	synchronized public LinkedList getIDList() {
				
		return id_list;
		
	}
	
	/**
	 * return the list of PeerID, messagetag and objectname
	 * @return constr_list
	 */
	synchronized public LinkedList getConstrList() {
		
		return constr_list;
		
	}

	/**
	 * return the list of PeerID and Socket
	 * @return ps_list
	 */
	synchronized public LinkedList getPSList() {
		
		return ps_list;
		
	}

	/**
	 * return the list of PeerIDs
	 * @return peerid_list
	 */
	synchronized public LinkedList<LinkedList<PeerID>> getPeerIDList() {
		
		return peerid_list;
		
	}
	
	/**
	 * return the list of termination ObjectIDs and their PeerIDs
	 * @return eoc_list
	 */
	synchronized public LinkedList getEOCList() {
		
		return eoc_list;
		
	}
	
	/**
	 * return the list of termination ObjectIDs and their PeerIDs
	 * @return eoc_list
	 */
	synchronized public LinkedList getDBGList() {
		
		return debug_list;
		
	}
	
	synchronized public LinkedList getCompoList() {
		
		return compo_list;
		
	}

	synchronized public LinkedList getHostList() {
	
		return host_list;
	
	}
	
}
