/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug;
/**
 * @author Johannes Baumgarten
 * @version 1.0
 */

import CTL.Types.*;
/**
 * Contains all informatio needed from an component
 * for storage in the mirror 
 */ 
public class PDBGContainer{
	
	
	
	/** PeerID object of the component*/
	private PeerID peerid;
	/** processID -needed for attaching monolithic debugger*/
	private int processid;
	/** defines the programming language -needed for attaching monolithic debugger
	 * for the exact languages see CTL.Remote*/
	private int planguage;
	/** the monolithic debugger which has to be called to debug this component*/
	private String debug;
	
	
	/**
	 * assign given information to the variables
	 * @param peerid PeerID which is recieved during the handshake
	 * @param debugger Debugger used to debug this component
	 * @param pid ProcessID
	 * @param plang Defines the programming language of the component
	 */
	public PDBGContainer (PeerID peerid , String debugger, int pid , int plang){
		this.peerid=peerid;
		this.processid=pid;
		this.planguage = plang;
		this.debug = debugger;	
	}
	
	synchronized public void updateDBG (String debugger, int pid, int plang){
		this.processid=pid;
		this.planguage = plang;
		this.debug = debugger;	
	}
	/**
	 * Retrieve the ProcessID
	 * @return ProcessID
	 */
	synchronized public int getPid(){
		return processid;
	}
	
	
	/**
	 * retrieve the programming language
	 * @return int representing programing language
	 */
	synchronized public int getLanguage (){
		return planguage;
	}
	
	/**
	 * retrieve the debugger
	 * @return debugger
	 */
	synchronized public String getDebug (){
		return debug;
	}
	
	/**
	 * retrieve the PeerID
	 * @return PeerID
	 */
	synchronized public PeerID getPeerID (){
		return peerid;
	}
	/**
	 * override the to.String()-method
	 */
	synchronized public String toString () {
		return this.peerid+"\n"+this.debug+"\n"+this.planguage+"\n"+this.processid;
	}
	
	
}