/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.util.ArrayList;
import java.util.List;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.*;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import Debug.Visualizer.editpolicies.MoveableObjectEditPolicy;

import Debug.Visualizer.figure.ComponentFigure;
import Debug.Visualizer.figure.HostFigure;
import Debug.Visualizer.gui.VisualizerColors;

import Debug.Visualizer.model.ComponentModel;
import Debug.Visualizer.model.ObjectModel;
import Debug.Visualizer.model.commands.MoveComponentCommand;

/**
 * Controls the behavior of the ComponentFigure and the stored Data in the
 * ComponentModel
 * 
 * @author Daniel Meyer and Ling Wu
 */
public class ComponentEditPart extends AbstractGraphicalEditPart implements
        PropertyChangeListener {

	/**
	 * childish change event for scale
	 */
    public static final String PROPERTY_CHILDISH_CHANGE = "some changes";
    
	@SuppressWarnings("unused")
    private List<Point> pointlist = new ArrayList<Point>();

    /**
     * Creates a org.eclipse.draw2d.Figure
     * 
     * @return Figure
     */
    protected IFigure createFigure() {

        ComponentFigure figure1 = new ComponentFigure(); //$NON-NLS-1$

        // Set Colors
        figure1.setForegroundColor(VisualizerColors.HEAD_1);
        figure1.setBackgroundColor(VisualizerColors.BODY_2);
        
        // set the Name to the Label
        figure1.setLabel(((ComponentModel) getModel()).getlogID()); //$NON-NLS-1$

        // get The String extends
        Dimension d = FigureUtilities.getStringExtents(
                ((ComponentModel) getModel()).getlogID(), new Font(null,
                        new FontData("Arial", 10, SWT.BOLD))); //$NON-NLS-1$ 

        // get the Insets of the Parents Border
        Rectangle in = ((HostFigure) ((HostEditPart) getParent()).getFigure()).getClientArea();

        // get the Location
        Point p = ((ComponentModel) getModel()).getLocation();

        // Set the Components Size
        figure1.setBounds(new Rectangle(p.x + in.x, 
        								p.y + in.y,
        								(d.width + 60),
        								(d.height + 60)));

        // Update the Model
        ((ComponentModel) getModel()).setConstraint(new Rectangle(p,d.expand(60,60)));
//        ((ComponentModel) getModel()).setConstraint(figure1.getBounds().getCopy());
        
        // Add a layoutManager
        figure1.setLayoutManager(new XYLayout());

        return figure1;
    }

    /**
     * Returns the corresponding Models list of Children
       * 
     * @return list List of children
     */
    protected List<ObjectModel> getModelChildren() {
        return ((ComponentModel) getModel()).getChildren();
    }

    /**
     * Activates this EditPart, used to register Listeners
     */
    @Override
    public void activate() {
        ((ComponentModel) getModel()).addPropertyChangeListener(this);
        super.activate();
    }

    /**
     * Deactivates the EditPart, used to unregister Listeners
     */
    @Override
    public void deactivate() {
        ((ComponentModel) getModel()).removePropertyChangeListener(this);
        super.deactivate();
    }

    /**
     * Installs EditPolicies to react on User-Interactions
     */
    public @Override void createEditPolicies() {
        // Dummy Implementation
        installEditPolicy(EditPolicy.LAYOUT_ROLE,
                new MoveableObjectEditPolicy());
    }

    /**
     * Reacts on Changes made to the corresponding model
     * 
     * @param event
     *            PropertyChangeEvent fired by an HostModel
     * @see model.HostModel.firePropertyChange()
     */
    @SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent evt) {

//    	System.out.println(evt.getPropertyName()+" "+evt.getSource());
    	
        if (evt.getPropertyName().equals("SCALE")){
    	
	    		if (evt.getSource() instanceof MoveComponentCommand){
	    		
	    			ArrayList<Float> a = ((DiagramEditPart)((HostEditPart)getParent()).getParent()).
	    				 getMultiplier((Rectangle)evt.getOldValue(),(Rectangle)evt.getNewValue());
	            
	    			for (ObjectModel om : getModelChildren()){
	                om.firePropertyChange("SCALE",null,a); //$NON-NLS-1$
	    			}
	    		}
	    		else{
	    			ArrayList<Float> a = (ArrayList<Float>) evt.getNewValue();

	    			Rectangle old = ((ComponentModel)getModel()).getConstraint();
	    			Rectangle newr = ((ComponentModel)getModel()).getConstraint().scale(a.get(0),a.get(1));
	    		
	    			newr.setLocation(old.getLocation());
	    			((ComponentModel)getModel()).setConstraint(newr);
		        
	    			for (ObjectModel om : getModelChildren()){
		            om.firePropertyChange("SCALE",null,a); //$NON-NLS-1$
		           }
	    	   }
    	   }
        
        if (evt.getPropertyName().equals("OUTPUT")) //$NON-NLS-1$
           {
            refreshSourceConnections();
            refreshTargetConnections();
           }
        
        //  New Connection as Target
        else if (evt.getPropertyName().equals("INPUT")) //$NON-NLS-1$
           {
            refreshSourceConnections();
            refreshTargetConnections();
           }
        
        if (evt.getPropertyName().equals(
                ComponentModel.PROPERTY_CONSTRAINT_CHANGED)) {
        	
        	Point p = getFigure().getParent().getClientArea().getLocation();
       	 	
//        	System.out.println(p.toString());
       	 	
        	Rectangle r = ((ComponentModel)getModel()).getConstraint();
        	
//        	System.out.println(r.toString());
        	
        	Point p1 = new Point(r.x+p.x,r.y+p.y);
        	
//        	System.out.println(r.toString());
        	
        	r.setLocation(p1);
        	
       	 	getFigure().setBounds(r);
       	 	
       	 	//       	 	getFigure().setBounds(((ComponentModel)getModel()).getConstraint());
//       	 getFigure().setLocation(p);
        		
            ((HostEditPart)getParent()).propertyChange(new PropertyChangeEvent(
            		this,PROPERTY_CHILDISH_CHANGE,null,null));
           }

        if (evt.getPropertyName().equals(ComponentModel._NEWOBJECT)) {
            if (evt.getNewValue() instanceof ObjectEditPart){
                removeChild((EditPart) evt.getNewValue());
            }
            
            	 calculateSize(getModelChildren());
            	 
//            	 getFigure().setBounds(((ComponentModel)getModel()).getConstraint());
            	             	 
//            	 Point p = getFigure().getParent().getClientArea().getLocation();
//            	 getFigure().setLocation(p);
//            	 
            	 Point p = getFigure().getParent().getClientArea().getLocation();
            	 	
//             	System.out.println(p.toString());
            	 	
             	Rectangle r = ((ComponentModel)getModel()).getConstraint();
             	
//             	System.out.println(r.toString());
             	
             	Point p1 = new Point(r.x+p.x,r.y+p.y);
             	
//             	System.out.println(r.toString());
             	
             	r.setLocation(p1);
            	 
             	getFigure().setBounds(r);
             	
            	 refreshChildren();
            	 refreshVisuals();
            	 
            	 
            	 ((HostEditPart)getParent()).propertyChange(new PropertyChangeEvent(
            			 							this,PROPERTY_CHILDISH_CHANGE,null,null));
            
           }
        
                
        if (evt.getPropertyName().equals(ObjectEditPart.OBJECT_CHILDISH_CHANGE)){
        		
        		
        			calculateSize(getModelChildren());
        		
        		Point p = getFigure().getParent().getClientArea().getLocation();
           	 
           	 	
//            	System.out.println(p.toString());
           	 	
            	Rectangle r = ((ComponentModel)getModel()).getConstraint();
            	
//            	System.out.println(r.toString());
            	
            	Point p1 = new Point(r.x+p.x,r.y+p.y);
            	
//            	System.out.println(r.toString());
            	
            	r.setLocation(p1);
        		
           	 	getFigure().setBounds(r);
//           	 getFigure().setBounds(((ComponentModel)getModel()).getConstraint());
//           	 getFigure().setLocation(p);
           	 	refreshVisuals();
           	 	refreshChildren();
           	 ((HostEditPart)getParent()).propertyChange(new PropertyChangeEvent(
						this,PROPERTY_CHILDISH_CHANGE,null,null));        		
        }

        refreshChildren();

        refreshVisuals();

        // Causes Graph to re-layout
        ((GraphicalEditPart) (getViewer().getContents())).getFigure()
                .revalidate();
    }

    /**
     * Calculates the Models Size by adding up the Childrens size
     * 
     * @param modelChildren
     *            List<ObjectModel>
     * @return The new Bounds
     */
    private Rectangle calculateSize(List<ObjectModel> modelChildren) {

        // Init Rects
        Rectangle bounds;
        Rectangle save;        
                
        // save old value
        save = bounds = new Rectangle(((ComponentModel)getModel()).getConstraint().getLocation(),new Dimension(0,0));
        
//        Point p = bounds.getLocation();
        
        if (modelChildren.size() ==0)
        	return null;
//        		bounds = new Rectangle(getFigure().getBounds().getLocation(),
//        						new Dimension(getFigure().getMinimumSize()));
                
        // add up children Size
        for (ObjectModel om : modelChildren) 
        	
//            if ((om.getLocation().x + om.getSize().width) >= (bounds.x + bounds.width))
//                bounds = bounds.getUnion(om.getConstraint());
//            
//            if ((om.getLocation().y + om.getSize().height) >= (bounds.y + bounds.height))
//                bounds = bounds.getUnion(om.getConstraint());

            if (((ObjectModel)om).getEditPart() != null){
            
//            ObjectEditPart oep = (ObjectEditPart) ((ObjectModel)om).getEditPart();
//
//        		IFigure f = oep.getFigure();
        		
        		bounds.union(om.getConstraint());
        		bounds.width += 10;
            }
            else{
            	return null;
            	
            }
//        		bounds.setLocation(p);
        bounds.width += 60;
		bounds.height += 60;

        
        		

        // update the Model
        ((ComponentModel) getModel()).setSize(bounds.getSize());

        // update the Parent
        ((HostEditPart) getParent())
                .propertyChange(new PropertyChangeEvent(this,
                        ComponentEditPart.PROPERTY_CHILDISH_CHANGE, save,
                        bounds));

        return bounds;
    }

    
    /**
     * searches on the view for an other ObjectModel at the point temp
     * 
     * @param temp, the point that is possiply occured
     * @return Model found at point temp or null
     */
    public ObjectModel findModelAt(Point temp) {
        for (int i = 0; i <= ((ComponentModel) getModel()).getChildren().size() - 1; i++) {
            if (temp.x >= ((ComponentModel) getModel()).getChildren().get(i)
                    .getLocation().x
                    && temp.x <= ((ComponentModel) getModel()).getChildren()
                            .get(i).getSize().width
                            + ((ComponentModel) getModel()).getChildren()
                                    .get(i).getLocation().x
                    && temp.y >= ((ComponentModel) getModel()).getChildren()
                            .get(i).getLocation().y
                    && temp.y <= ((ComponentModel) getModel()).getChildren()
                            .get(i).getSize().height
                            + ((ComponentModel) getModel()).getChildren()
                                    .get(i).getLocation().y)
                return ((ComponentModel) getModel()).getChildren().get(i);
        }
        return null;
    }
    /**
     * starts the method ObjectModel findModelAt(Point temp) with the overgiven values
     * 
     * @param x position of point on x - axle
     * @param y position of point on y - axle
     * @return findModelAt
     */  
    public ObjectModel findModelAt(int x, int y) {
        return findModelAt(new Point(x, y));
    }

    public Point getAutomaticLocation(ObjectModel om) {
    	
    	int minDist = 50;
    	
    	Point temp = new Point (0,0);
    	
    	int i =getModelChildren().size()+1;
    	
    	if (i==1) return  temp;
    	
    	int z = (int)Math.ceil(Math.sqrt(i));
    	
    	int maxElements = z*z;
    	
    	if (i==maxElements){
    		//Rechteck komplett
    		Rectangle r = getModelChildren().get(i-2).getConstraint();
//    		Rectangle rabove = getModelChildren().get((z-1)*(z-1)-1).getConstraint();
    
//			if (r.y < (rabove.y+rabove.height)){
//				
//				for (int j=maxElements-z+1;maxElements>j;j++){
//					getModelChildren().get(j).setLocation(new Point(r.x,rabove.y+rabove.height+50));
//				}
//				r.y = rabove.y+rabove.height+minDist;
//			}
//			System.out.println("Unten Rechts: "+i+" "+z+" "+maxElements);
			return new Point(r.x+r.width+minDist,r.y);
    	}
    	else if (i <= maxElements-z){
    		if (i==((z-1)*(z-1)+1)){
    			//rechts oben
    			
    			Rectangle r = getModelChildren().get((z-2)*(z-2)).getConstraint();
    			temp.x = r.x+r.width+minDist;
//    			System.out.println("Rechts Oben: "+i+" "+z+" "+maxElements);
    			return temp;
    		}
    		else{
    			Rectangle r = getModelChildren().get(i-2).getConstraint();
    			temp.x = r.x;
    			temp.y = r.y+r.height+minDist;
//    			System.out.println("Rechts: "+i+" "+z+" "+maxElements);
    			return temp;
    		//nimm vorgänger und setz drunter
    		}
    	}
    	else if (i > maxElements-z+1){
    		//nimm vorgänger und setz daneben
    		Rectangle r = getModelChildren().get(i-2).getConstraint();
			temp.x = r.x+r.width+minDist;
			temp.y = r.y;
//			System.out.println("Unten: "+i+" "+z+" "+maxElements);
			return temp;
    	}
    	else if (i==maxElements-z+1){
    		//setze unten links unter (z-1)^2-z+1
    		Rectangle r = getModelChildren().get((z-2)*(z-2)+(z-2)).getConstraint();
			temp.x = r.x;
			temp.y = r.y+r.height+minDist;
//			System.out.println("Unten Links: "+i+" "+z+" "+maxElements);
			return temp;
    	}
//    	System.out.println("None: "+i+" "+z+" "+maxElements);
    	return null;
    }
    
    /**
     * Updates the Pointlist (Currently unused)
     * 
     * @param model
     * @param object
     * @param object2
     */
    @SuppressWarnings("unused")
    private void updateLayout(@SuppressWarnings("unused")
    ObjectModel model, @SuppressWarnings("unused")
    Object object, @SuppressWarnings("unused")
    Object object2) {
       
        // Update the Pointlist...

        // Update the View
        ((GraphicalEditPart) (getViewer().getContents())).getFigure()
                .revalidate();
    }
    
    /**************************Connections*********************/

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
     */
    @Override
    protected List getModelSourceConnections()
     {
        return getComponent().getStartConnections();
     }
    
     /**
     * @return ComponentModel
     */
    private ComponentModel getComponent() {
        return (ComponentModel)getModel();
     }

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
     */
    @Override
    protected List getModelTargetConnections()
     {
        return getComponent().getEndConnections();
     }

	public boolean checkCollision(ObjectModel aa, ObjectModel oo) {
		// TODO Auto-generated method stub
		return false;
	}
}
