/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import Debug.Visualizer.model.*;

import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.SimpleRaisedBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import Debug.Visualizer.figure.InfoscreenFigure;

/**
 * Controls the Behavior of an InfoscreenDataModel and an InfoscreenDataFigure
 * @author Daniel Meyer
 *
 */
public class InfoscreenDataEditPart extends AbstractGraphicalEditPart 
		implements PropertyChangeListener{

	//Maximum Label - Sizes
	public static int txtmaxwidth;
	public static int submaxwidth;
	public static int maxwidth;
	public static int maxheight;
	
	private static int fontsize=0;
	
	
	@Override
    /**
     * @see org.eclipse.draw2d.IFigure.createFigure()
     */
	protected IFigure createFigure() {
		
		if (InfoscreenDataEditPart.fontsize == 0)
			setFontSize(8);
        
        //Create Font
		FontData fd = new FontData("Arial", InfoscreenDataEditPart.fontsize, SWT.BOLD); //$NON-NLS-1$
		Font f = new Font(null, fd);
		
		//Get Data to Display
		String sub = ((InfoscreenDataModel)getModel()).getSubject();
		String txt = ((InfoscreenDataModel)getModel()).getData();
		
        //Get Text Size
		Dimension dsub = FigureUtilities.getStringExtents(sub,f);
		Dimension dtxt = FigureUtilities.getStringExtents(txt,f);
		
		//Get Layout of parent Figure
		InfoscreenFigure fig2 = (InfoscreenFigure) ((InfoscreenEditPart)getParent()).getFigure();
		Insets in = fig2.getBorder().getInsets(fig2);
		
		Point p = ((InfoscreenEditPart)getParent()).getAutomaticLocation();
		
		//Get needed Bounds
		int bordersize = 1;
		Rectangle rsub = new Rectangle(p.x+in.left,p.y+in.top,dsub.width+2*bordersize,dsub.height+2*bordersize);
		Rectangle rtxt = new Rectangle(rsub.x+bordersize+rsub.width,rsub.y,dtxt.width+2*bordersize,rsub.height);
		
		Label fig = new Label();
		
		Label parent = new Label();
		
        
        //Get the common Subject - Labelsize
		if (rsub.width > InfoscreenDataEditPart.submaxwidth){
			InfoscreenDataEditPart.submaxwidth = rsub.width;
		}
		else rsub.width = InfoscreenDataEditPart.submaxwidth;
		
        //Get the Common Text _ LabelSize
		if (rtxt.width > InfoscreenDataEditPart.txtmaxwidth){
			InfoscreenDataEditPart.txtmaxwidth = rtxt.width;
		}
		else rtxt.width = InfoscreenDataEditPart.txtmaxwidth;
                
        Rectangle rboth = rsub.getUnion(rtxt);
        
    if (rboth.width >  InfoscreenDataEditPart.maxwidth){
        
        InfoscreenDataEditPart.maxwidth = rboth.width;
        
        InfoscreenModel ifm = (InfoscreenModel)((InfoscreenEditPart)getParent()).getModel();
        
        //8 = InfoscreenBorderSize
        ifm.setSize(new Dimension(rboth.width+in.getWidth(),ifm.getSize().height));
    }
    else rboth.width = InfoscreenDataEditPart.maxwidth;
       
    	if (maxheight > rboth.height)		rboth.height = maxheight;    		
    
    	else 		maxheight = rboth.height;
    	    
		parent.setBounds(rboth);
		
		((InfoscreenDataModel)getModel()).setConstraint(rboth);
		
		//create parent figure
		parent.setFont(f);
		
		SimpleRaisedBorder border = new SimpleRaisedBorder(1);
		
		parent.setBorder(border);
		
		parent.setText(sub);
		
		parent.setTextAlignment(PositionConstants.LEFT);
		
		parent.setTextPlacement(PositionConstants.WEST);
		
		parent.setLabelAlignment(PositionConstants.LEFT);
				
		//Create second Label
		fig.setBounds(rtxt);
		
		fig.setFont(f);
		
		fig.setBorder(border);
		
		fig.setText(txt);
		
		fig.setTextAlignment(PositionConstants.CENTER);
		
		fig.setTextPlacement(PositionConstants.CENTER);
		
		fig.setLabelAlignment(PositionConstants.CENTER);
				
		parent.add(fig);
		
		return parent;
	}

	
    /**
     * Returns the List of the Models Children (an Empty List)
     */
    @Override
	protected List getModelChildren(){
		return new ArrayList();
	}
	
    /**
     * Install EditPolicies (NONE)
     */
    @Override
	protected void createEditPolicies() {
		// Dummy Implementation
	}

    /**
     * React on Changes Made in the model
     */
	@SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent evt) {
		
        if (evt.getPropertyName().equals("SCALE")){ //$NON-NLS-1$
        	
        		ArrayList<Float> a = (ArrayList<Float>) evt.getNewValue();
        	
        		Rectangle old = getFigure().getBounds();
        	
        		Rectangle newr = getFigure().getBounds().getCopy().scale(a.get(0),a.get(1));
    		
        		newr.setLocation(old.getLocation());
        	
            ((InfoscreenDataModel)getModel()).setConstraint(newr);
            
            scaleChildren(a.get(0),a.get(1));
            
            refreshVisuals();
            
            ((InfoscreenEditPart)getParent()).refreshChildrenSize();

           }
    
        //Model Resized
        else if (evt.getPropertyName().equals(InfoscreenDataModel.INFOSRCEENDATA_CONSTRAINT_CHANGED)){
		    
        		getFigure().setBounds(((InfoscreenDataModel)getModel()).getConstraint());
        	
        		refreshVisuals();
        }	
	}
	
	/**
	 * scale the child size
	 * @param x scale value of x-coordinate
	 * @param y scale value of y-coordinate
	 */
	public void scaleChildren(Float x, Float y) {
			
		Label parent = (Label) getFigure();
	   Label fig = (Label) parent.getChildren().get(0); 
		
		Dimension dsub = new Dimension(submaxwidth,maxheight);
		Dimension dtxt = new Dimension(txtmaxwidth,maxheight);
		
		Dimension dnewsub = new Dimension(0,0),
				  	 dnewtxt = new Dimension(0,0);
		
		if (x>=1 && y>=1 ){
				while(dsub.contains(dnewsub) && dtxt.contains(dnewtxt)){
					dnewsub = FigureUtilities.getStringExtents(parent.getText(), new Font(null,
		                        new FontData("Arial", InfoscreenDataEditPart.fontsize++, SWT.BOLD)));
					dnewtxt = FigureUtilities.getStringExtents(fig.getText(), new Font(null,
	                        new FontData("Arial", InfoscreenDataEditPart.fontsize, SWT.BOLD)));
				}
				fontsize--;
		}
		else{ //x<1 || y < 1
			dnewsub.setSize(new Dimension(999,999));
			dnewtxt.setSize(new Dimension(999,999));
			
			while(!dsub.contains(dnewsub) && !dtxt.contains(dnewtxt) && fontsize >= 2){
				
				dnewsub = FigureUtilities.getStringExtents(parent.getText(), new Font(null,
	                        new FontData("Arial", InfoscreenDataEditPart.fontsize--, SWT.BOLD)));
				dnewtxt = FigureUtilities.getStringExtents(fig.getText(), new Font(null,
                        new FontData("Arial", InfoscreenDataEditPart.fontsize, SWT.BOLD)));
			}
		}
		
		parent.setFont(new Font(null,
                        new FontData("Arial",InfoscreenDataEditPart.fontsize, SWT.BOLD)));
		
		fig.setFont(parent.getFont());
		
		//Resize Figures
		parent.setSize(InfoscreenDataEditPart.maxwidth,maxheight);
		fig.setSize(InfoscreenDataEditPart.txtmaxwidth,maxheight);
		
	}
	
	/**
	 * Set the font size
	 * @param i
	 */
	public void setFontSize(int i) {
		InfoscreenDataEditPart.fontsize=i;
		
	}

    /**
     * Activates this EditPart, used to register Listeners
     */
    @Override
	public void activate(){
		((InfoscreenDataModel)getModel()).addPropertyChangeListener(this);
		super.activate();
	}

    /**
     * Deactivates the EditPart, used to unregister Listeners
     */
	@Override
	public void deactivate(){
		((InfoscreenDataModel)getModel()).removePropertyChangeListener(this);
		super.deactivate();
	}
    
    /**
     * Repaints the InfoscreenDataFigure and his Children
     */
    @Override
    public void refreshVisuals(){
                
    	  int bordersize = 1;
    	
        //Get needed Bounds
        Rectangle rsub = getFigure().getBounds();
        
        //Set width to MaxLabel - Size
        rsub.width=InfoscreenDataEditPart.submaxwidth;
        rsub.height = maxheight;
        
        Rectangle rtxt = new Rectangle(rsub.x+bordersize+rsub.width,
        		rsub.y,InfoscreenDataEditPart.txtmaxwidth+2*bordersize,maxheight);
        
        rtxt.width = InfoscreenDataEditPart.maxwidth - InfoscreenDataEditPart.submaxwidth;
        
        //Get Figures
        Label parent = (Label) getFigure();
        Label fig = (Label) parent.getChildren().get(0);
        
        //get Bounds
        Rectangle rboth = rsub.getUnion(rtxt);
        
        //Resize parent
        parent.setBounds(rboth);
        
        //Add Text
        parent.setText(((InfoscreenDataModel)getModel()).getSubject());
        parent.setTextAlignment(PositionConstants.LEFT);
        parent.setTextPlacement(PositionConstants.WEST);
        parent.setLabelAlignment(PositionConstants.LEFT);
        
        
        //Create second Label
        fig.setBounds(rtxt);
        
        fig.setText(((InfoscreenDataModel)getModel()).getData());
        fig.setTextAlignment(PositionConstants.CENTER);
        fig.setTextPlacement(PositionConstants.CENTER);
        fig.setLabelAlignment(PositionConstants.CENTER);
        
        parent.add(fig);
        
        parent.revalidate();
        parent.repaint();      
    }
}
