/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import Debug.Visualizer.model.DiagramElementModel;
import Debug.Visualizer.model.MessageModel;

/**
 * 
 * @author Andre Schucht
 *
 */
public class MessageEditPart extends AbstractConnectionEditPart
			implements PropertyChangeListener{

	/**
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	@Override
	public void activate() {
		((MessageModel)getModel()).addPropertyChangeListener(this);
		super.activate();
	}
		
	/**
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	@Override
	public void deactivate() {
		((MessageModel)getModel()).removePropertyChangeListener(this);
		super.deactivate();
	}
		
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	public void createEditPolicies(){
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE, new ConnectionEndpointEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractConnectionEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure(){
		
		PolylineConnection conn = new PolylineConnection();
		PolygonDecoration dec = new PolygonDecoration();
		dec.setScale(10,5);
		conn.setTargetDecoration(dec);
		ConnectionEndpointLocator loc = new ConnectionEndpointLocator(conn, true);

		Label label = new Label();
		label.setFont(new Font(null,new FontData("Arial",8,SWT.NONE)));
		label.setBackgroundColor(ColorConstants.lightGray);
		label.setOpaque(true);
		
		conn.add(label,  loc);
		
		FanRouter fr = new FanRouter();
		fr.setNextRouter(new ShortestPathConnectionRouter(((DiagramEditPart)getViewer().getContents()).getFigure()));
		conn.setConnectionRouter(fr);
		
		return conn;
	}

    /**
     * Reacts on Changes made in the Model
     */
	public void propertyChange(PropertyChangeEvent evt) {

		String property = evt.getPropertyName();

        //New Connection
		if (DiagramElementModel.INPUT.equals(property))
			refreshSourceConnections();
		else if (DiagramElementModel.OUTPUT.equals(property))
			refreshTargetConnections();

		((GraphicalEditPart) (getViewer().getContents())).getFigure().revalidate();
		
	}
	
}
