/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editparts.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import Debug.Visualizer.figure.ComponentFigure;
import Debug.Visualizer.gui.VisualizerColors;

import Debug.Visualizer.model.ObjectModel;
import Debug.Visualizer.model.commands.MoveObjectCommand;

/**
 * Controls the behavior of the ObjectFigure and the stored Data in 
 * the ObjectModel
 * @author Daniel Meyer and Ling Wu
 */
public class ObjectEditPart extends AbstractGraphicalEditPart implements
		PropertyChangeListener{	
	
	//fontsize of objectID
	private int fontsize = 0;

	//maxwidth of object Rectangle
	private int maxwidth;
	
	//maxheight of object Rectangle
	private int maxheight;
		
	//object childish change event
	public static final String OBJECT_CHILDISH_CHANGE = "object childish change";
	
	/**
	 * set the fontsize
	 * @param i size to be set
	 */
	public void setFontSize(int i) {
		fontsize=i;		
	}
	
	/**
	 * Creates a org.eclipse.draw2d.Figure
	 * @return Figure  
	 */
	@Override
	protected IFigure createFigure(){
	              
        if (fontsize == 0)	setFontSize(8);
        
        FontData fd = new FontData("Arial", fontsize, SWT.NONE); //$NON-NLS-1$
        
        Font f = new Font(null, fd);
        
		  String st = "object"; //$NON-NLS-1$
        
        ObjectModel om = (ObjectModel)getModel();
        
        long x = om.getObjectID();
           
        	//set the objectID as Label
        Label lobj = new Label();

        //set the object as label
        Label figure1 = new Label();
                
        //set backgroudcolor
        figure1.setBackgroundColor(VisualizerColors.BODY_3);
        
        //set the figure to be opaque
        figure1.setOpaque(true);
        
        GroupBoxBorder border = new GroupBoxBorder();
        
        border.setFont(new Font(null, new FontData("Arial", 8, SWT.NONE)));
          
        String st1 =  "ID: "+ x;
	     
        //get the dimension of the objectID text
        Dimension dobj = FigureUtilities.getStringExtents(st1,f);
          
        //set the label as st
		  border.setLabel(st);
		  		  
		  figure1.setBorder(border);
		  		
		  //get insets of parent figure
		  Rectangle in = ((ComponentFigure)((ComponentEditPart)getParent())
				        .getFigure()).getClientArea();
		
		  //get the location of parent figure
		  Point p = ((ObjectModel)getModel()).getLocation();
		          
//		  Point p = ((ComponentEditPart)getParent()).getAutomaticLocation(null);
		  //set bounds of object
		  figure1.setBounds(new Rectangle(p.x+in.x,
					 p.y+in.y,
					 (dobj.width+border.getInsets(figure1).getWidth()+20),
					 (dobj.height+border.getInsets(figure1).getHeight()+20)));

		  //get location of object figure
		  Point p2 = figure1.getLocation();
		  
		  //set rectangle of objectID
		  Rectangle robj = new Rectangle(p2.x+border.getInsets(figure1).left,
				  									p2.y+border.getInsets(figure1).top,
				  									dobj.width,
				  									dobj.height);
		 		  
		  if (robj.width > maxwidth){
		  		maxwidth = robj.width;
		  		}
		  else {
			  robj.width = maxwidth;
			  }
		  
	     if (maxheight > robj.height) robj.height = maxheight;
	     else maxheight = robj.height;
	    		    	        
	     //set bounds of objectID label
        lobj.setBounds(robj);
        
        //set foreground color of objectID label
        lobj.setForegroundColor(ColorConstants.black);
            
        //set font of objectID label
        lobj.setFont(f);
              
        //set the text of label
        lobj.setText(st1);
        
        //set text alignment
        lobj.setTextAlignment(PositionConstants.CENTER);
        
        //set text placement
        lobj.setTextPlacement(PositionConstants.CENTER);
        
        //set label alignment
        lobj.setLabelAlignment(PositionConstants.LEFT);
                
        //add objectID label as child of object label
        figure1.add(lobj);
        
        //give the new value of objectModel constraint
        ((ObjectModel) getModel()).setConstraint(new Rectangle(p,dobj.getExpanded(20+border.getInsets(figure1).getWidth(),20+border.getInsets(figure1).getHeight())));
//        ((ObjectModel) getModel()).setConstraint(figure1.getBounds().getCopy());
        
        //set layout of object figure as XYLayout
        figure1.setLayoutManager(new XYLayout());
        
		  return figure1;
	  }
	
	/**
	 * Returns the corresponding Models list of Children
	 * @return list List of children 
	 */
	@Override
	protected List getModelChildren() {
		return new ArrayList();
	}
	
	/**
	 * Activates this EditPart, used to register Listeners
	 */
	@Override
	public void activate() {
		((ObjectModel) getModel()).addPropertyChangeListener(this);
//		System.out.println("listener added");
		super.activate();
	}
	
	/**
	 * Deactivates the EditPart, used to unregister Listeners
	 */
	@Override
	public void deactivate(){
		((ObjectModel) getModel()).removePropertyChangeListener(this);
		super.deactivate();
	}
	
    /**
     * @return The EditParts ObjectModel
     */
	public ObjectModel getObject(){
		return (ObjectModel) getModel();
	}

	/**
	 * Installs EditPolicies to react on User-Interactions
	 */
	@Override
	protected void createEditPolicies() {
		// Dummy Implementation
	}

    /**
     * Reacts on Changes made in the Model
     * @param evt changeevent to be fired by ObjectModel
     */
	@SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent evt) {
     if (evt.getPropertyName().equals("SCALE")){    	 
        	if (evt.getSource() instanceof MoveObjectCommand){
        		ArrayList<Float> a = ((DiagramEditPart)((HostEditPart)((ComponentEditPart)getParent())
        				.getParent()).getParent()).getMultiplier((Rectangle)evt.getOldValue(),
        						(Rectangle)evt.getNewValue());
		        
		        //RESIZE LABEL
        		ObjectModel om = (ObjectModel)getModel();
        		
        		maxwidth = om.getConstraint().width;
        		maxheight = om.getConstraint().height;
        		
        		scaleChildren(a.get(0),a.get(1));
        		
        		refreshVisuals();
        		//set the event to ComponentEditPart
        		((ComponentEditPart)getParent()).propertyChange(new PropertyChangeEvent(
                		this,OBJECT_CHILDISH_CHANGE,(Rectangle)evt.getOldValue(),
                		(Rectangle)evt.getNewValue()));
        		
        	}
        	else{
        		ArrayList<Float> a = (ArrayList<Float>) evt.getNewValue();
        		
        		Rectangle old = ((ObjectModel) getModel()).getConstraint();
        		
        		//        		Rectangle old = getFigure().getBounds();
	    		Rectangle newr = old.scale(a.get(0),a.get(1));
	    		
	    		newr.setLocation(old.getLocation());
	    		
	    		((ObjectModel)getModel()).setConstraint(newr);
	    		        		
	    		//RESIZE LABEL 
        		maxwidth = (int)Math.ceil(maxwidth*a.get(0));
        		maxheight = (int)Math.ceil(maxheight*a.get(1));
        		
        		scaleChildren(a.get(0),a.get(1));
        	   
        		getFigure().setSize(((ObjectModel)getModel()).getSize());
        		
        		refreshVisuals();
        		
        		((ComponentEditPart)getParent()).propertyChange(new PropertyChangeEvent(
                		this,OBJECT_CHILDISH_CHANGE,(Rectangle)old,
                		(Rectangle)newr));
        	}
       }
        
     //Object Renamed/ Resized
     if (evt.getPropertyName().equals(ObjectModel.OBJECT_CONSTRAINT_CHANGE)){
    	 
//    	   Rectangle r = ((ObjectModel)getModel()).getConstraint();
//    	  	
//    	   Point p = getFigure().getParent().getClientArea().getLocation();
//    	   
//    	   r.setLocation(r.getLocation().getTranslated(p));
    	 
    	 Point p = getFigure().getParent().getClientArea().getLocation();
 	 	
//      	System.out.println(p.toString());
     	 	
      	Rectangle r = ((ObjectModel)getModel()).getConstraint();
      	
//      	System.out.println(r.toString());
      	
      	Point p1 = new Point(r.x+p.x,r.y+p.y);
      	
//      	System.out.println(r.toString());
      	
      	r.setLocation(p1);
     	 
      	getFigure().setBounds(r);
    	        	
        	maxwidth = r.width ;
        	        	
        	((ComponentEditPart)getParent()).propertyChange(new PropertyChangeEvent(
            		this,ObjectEditPart.OBJECT_CHILDISH_CHANGE,(Rectangle)evt.getOldValue(),
            		(Rectangle)evt.getNewValue()));
        	
        	refreshVisuals();
        	refreshChildren();
         }
        
        // New Connection as Source
		else if (evt.getPropertyName().equals("OUTPUT")) //$NON-NLS-1$
		{
            refreshSourceConnections();
            refreshTargetConnections();
		}
        //  New Connection as Target
		else if (evt.getPropertyName().equals("INPUT")) //$NON-NLS-1$
		{
            refreshSourceConnections();
            refreshTargetConnections();
		}
        
		refreshVisuals();
        ((GraphicalEditPart) (getViewer().getContents())).getFigure()
        .revalidate();
	}

	/**
	 * scale the child size
	 * @param x scale value of x-coordinate
	 * @param y scale value of y-coordinate
	 */
	public void scaleChildren(Float x, Float y) {
		
		//object label
		Label parent = (Label) getFigure();
		
		Point p = parent.getLocation();
		
		//objectID label
		Label txt = (Label) parent.getChildren().get(0); 
						
		Dimension dtxt = new Dimension(maxwidth,maxheight);
		
		Insets in = getFigure().getBorder().getInsets(getFigure());
		
//		dtxt.width-=in.getWidth();
//		dtxt.height-=in.getHeight();
		
		Dimension dnewtxt = new Dimension(0,0);
		
		if (x>=1 && y>=1 ){
				while(dtxt.contains(dnewtxt) && fontsize >= 1){
					dnewtxt = FigureUtilities.getStringExtents(txt.getText(), new Font(null,
	                        new FontData("Arial", fontsize++, SWT.NONE)));
				}
				fontsize--;
		}
		//		x<1 || y < 1
		else { 
			dnewtxt.setSize(new Dimension(999,999));
			
			while(!dtxt.contains(dnewtxt) && fontsize >= 2){
				dnewtxt = FigureUtilities.getStringExtents(txt.getText(), new Font(null,
                        new FontData("Arial", fontsize--, SWT.NONE)));
			}
		}
				
		txt.setFont(new Font(null,new FontData("Arial",fontsize, SWT.NONE)));
			
		//Resize Figures	
		
		Dimension dtxt1 = FigureUtilities.getStringExtents(txt.getText(),txt.getFont());
		
		//set the new size of objectID label
		txt.setSize(dtxt1.width,dtxt1.height);
		
		txt.setLocation(new Point(p.x+in.left,p.y+in.top));
	
		
		//set the new size of object label
		parent.setSize(maxwidth + 3*in.left, maxheight+in.top);
		parent.setLocation(p);
		
						
	}


	
/**************************Connections*********************/

    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
     */
    @Override
    protected List getModelSourceConnections(){
    	return getObject().getStartConnections();
    }
    
    /**
     * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
     */
    @Override
    protected List getModelTargetConnections(){
    	return getObject().getEndConnections();
    }
}