/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editparts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import Debug.Visualizer.model.*;


/**
 * Called by GEF when a new Model has to get its own EditPart, which
 * is created here.
 * @author Daniel Meyer
 *
 */
public class VisualizerEditPartFactory implements EditPartFactory {

	/**
	 * Creates an EditPart for the Corresponding <em>model</em>
	 * @param context EditPart which called the EditPartFactory (not used)
	 * @param model Object which should be an Instance of a Model
	 * @return part Editpart for the Model
	 */
	public EditPart createEditPart(@SuppressWarnings("unused") EditPart context, Object model) {
		
		EditPart part = null;
		
		//get the Model 
		if(model instanceof DiagramModel){
						
			part = new DiagramEditPart();
			
			((DiagramModel) model).setEditPart((DiagramEditPart) part);
		
		}
		else if (model instanceof HostModel){
						
			part = new HostEditPart();
            
			((HostModel)model).setEditPart((HostEditPart) part);
		}
		
		else if (model instanceof InfoscreenModel){
			
			part = new InfoscreenEditPart();
			
			((InfoscreenModel)model).setEditPart((InfoscreenEditPart) part);
		}
		
		else if (model instanceof ComponentModel){
						
			part = new ComponentEditPart();
			
			((ComponentModel)model).setEditPart((ComponentEditPart) part);
		}
		
		else if (model instanceof ObjectModel){
			
			part = new ObjectEditPart();
						
			((ObjectModel)model).setEditPart(part);            

		}
		
		else if (model instanceof InfoscreenDataModel){
			
			part = new InfoscreenDataEditPart();
			
			
			((InfoscreenDataModel)model).setEditPart(part);
		}
        
		else if (model instanceof MessageModel){
						
			part = new MessageEditPart();
			
			((MessageModel)model).setEditPart((MessageEditPart) part);
		}
		
//		else System.out.println("ERROR IN THE FACTORY"); //$NON-NLS-1$
		
		//add the Model to the EditPart
		part.setModel(model);
		
		return part;
	}

}
