/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.editpolicies;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.*;

import Debug.Visualizer.model.ComponentModel;
import Debug.Visualizer.model.commands.MoveComponentCommand;
import Debug.Visualizer.editparts.ComponentEditPart;


/**
 * This class restricts the Ability of Moving the Model.
 * Actually, it is only available for XYLayoutManagers,
 * not for automatic layout.
 * @author Andre Schucht
 * @version 1.2
 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy
 */

public class MoveableComponentEditPolicy extends XYLayoutEditPolicy {
    
    /*DUMMY Method*/
    @Override
    protected Command createAddCommand(@SuppressWarnings("unused") EditPart child, @SuppressWarnings("unused") Object constraint)
    {
        return null;
    }

    /**
     * Creates command to move or resize a model.
     * @return HostEditPart
     * @return HostFigure
     */
    @Override
    protected Command createChangeConstraintCommand(EditPart child, Object constraint)
    {

        if (!(constraint instanceof Rectangle))
            return null;
      
            EditPart part = (ComponentEditPart) child;
            ComponentModel model = (ComponentModel)part.getModel();
//            IFigure figure = (ComponentFigure) ((GraphicalEditPart) part).getFigure();
            Rectangle oldBounds = model.getConstraint();
            Rectangle newBounds = (Rectangle) constraint;
            
//            Rectangle b = figure.getParent().getClientArea();
            
//        	newBounds.x += b.x;
//            newBounds.y += b.y;
            
            
            if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
            	//Move...
//            	System.out.println("Move Component: "+newBounds.getLocation());
            }
            else{
            	//resize
//            	System.out.println("Resize Component: "+newBounds.getSize());
            	newBounds.x = oldBounds.x;
            	newBounds.y = oldBounds.y;
            }
            

        MoveComponentCommand command = new MoveComponentCommand(model, oldBounds.getCopy(), newBounds.getCopy());
        return command;
    }


    /*DUMMY Methods*/   
    
    @Override
    protected Command getCreateCommand(@SuppressWarnings("unused") CreateRequest request)
    {
        return null;
    }

    @Override
    protected Command getDeleteDependantCommand(@SuppressWarnings("unused") Request request)
    {
        return null;
    }

}
