/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.editpolicies;

import Debug.Visualizer.model.ObjectModel;

import Debug.Visualizer.model.commands.MoveObjectCommand;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;

import Debug.Visualizer.editparts.ObjectEditPart;

/**
 * This class restricts the Ability of Moving and Resizing
 * ObjectModel.
 * Actually, it is only available for XYLayoutManagers,
 * not for automatic layout.
 * @author Andre Schucht
 * @version 1.2
 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy
 */
public class MoveableObjectEditPolicy extends XYLayoutEditPolicy{
    
    /*DUMMY Method*/
    @Override
    protected Command createAddCommand(@SuppressWarnings("unused") EditPart child, @SuppressWarnings("unused") Object constraint)
    {
        return null;
    }

    /**
     * Creates command to move or resize a model.
     * @return HostEditPart
     * @return HostFigure
     */
    @Override
    protected Command createChangeConstraintCommand(EditPart child, Object constraint)
    {

        if (!(constraint instanceof Rectangle))
            return null;
      
            ObjectEditPart part = (ObjectEditPart) child;
            ObjectModel model = (ObjectModel)part.getModel();
           
            Rectangle oldBounds = model.getConstraint();
//            System.out.println(oldBounds.toString());
            
            Rectangle newBounds = (Rectangle) constraint;
//            System.out.println(newBounds.toString());
            
            
            if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
            	//Move...
//            	System.out.println("Move Object: "+newBounds.getLocation());
            }
            else{
//            	System.out.println("Resize Object: "+newBounds.getSize());
            	//resize
            	newBounds.x = oldBounds.x;
            	newBounds.y = oldBounds.y;
            }
            
        /**
         * Denied change Rectangles size
         */
//      if (oldBounds.width != newBounds.width && newBounds.width != -1)
//          return null;
//      if (oldBounds.height != newBounds.height && newBounds.height != -1)
//          return null;

        MoveObjectCommand command = new MoveObjectCommand(model, oldBounds.getCopy(), newBounds.getCopy());
        return command;
    }

      /**
       * Returns the current bounds as the constraint if none can be found in the
       * figures Constraint object
       */
      @Override
      public Rectangle getCurrentConstraintFor(GraphicalEditPart child)
      {
          IFigure fig = child.getFigure();
          Rectangle rect = (Rectangle) fig.getParent().getLayoutManager().getConstraint(fig);
          if (rect == null)
          {
              rect = fig.getBounds();
          }
          return rect;
      }

    /*DUMMY Methods*/   
    
    @Override
    protected Command getCreateCommand(@SuppressWarnings("unused") CreateRequest request)
    {
        return null;
    }

    @Override
    protected Command getDeleteDependantCommand(@SuppressWarnings("unused") Request request)
    {
        return null;
    }

}
