/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.figure;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.swt.graphics.Color;
import org.eclipse.draw2d.geometry.*;

import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.SWT;

import Debug.Visualizer.gui.VisualizerColors;

/**
 * 
 * This class sets the border of hostfigure
 *
 */
public class HostFigureBorder extends AbstractLabeledBorder{
    
    //init color
	private Color color;
    
    //init border size
	private static final int BORDER_SIZE = 2;
    
    //init line width
	private static final int LINE_WIDTH = 2;
    
    //init title border size
	private static final int TITLE_BORDER_SIZE = 2;
    
    //init font
	private Font font;
	
	
	/**
     * Constructor of HostFigureBorder
     * Sets the font
	 */
	public HostFigureBorder(){
		
		FontData fd = new FontData("Arial", 10, SWT.BOLD); //$NON-NLS-1$
		this.font = new Font(null, fd);
	
	}
	
	/**
     * Paint the Host
	 */
	public void paint(IFigure f, Graphics g, Insets insetsFigure){
		
		//HOST-Farbe... mal sehen...
		if(this.color != null)
			
		g.setForegroundColor(this.color);
		
		//Linienbreite
		g.setLineWidth(LINE_WIDTH);
		  
		/*paint rectangle using the childs of Host as size
		 * paremerer (Insets)
		 */ 
		Rectangle area = f.getBounds().getCropped(insetsFigure);
		
		//set Painting area to size "area" (no painting outside)
		g.setClip(area);
		
		//get the Dimension of the LabelText (Should be similar with each IP)
		Dimension textSize = FigureUtilities.getTextExtents(" "+getLabel()+" ", getFont(f));
		
		//get leftmost point of Rectangle
		Point left = area.getLocation().translate(0,(getInsets(f).top-BORDER_SIZE));
		
		//get rightmost point of Rectangle
		Point right = left.getCopy().translate(area.width,0);
		
		//Draw Line below Label
		g.drawLine(left, right);
		
		//set background to red
		g.setBackgroundColor(VisualizerColors.BORDER_1);
		
		//Fill the rectangle where the IP will stand
		g.fillRectangle(area.getLocation().x,
				area.getLocation().y,
				area.width,
				getInsets(f).top - LINE_WIDTH - 1);
		
		g.setFont(this.font);
		
		//get the center position for IP
		Point centeredLabel = area.getLocation().translate((area.getSize().width - textSize.width )/ 2,TITLE_BORDER_SIZE);
		
		g.drawText(getLabel(), centeredLabel);
	}
	
	@Override
    /**
     * Calculate the Diminsion of a Text
     */
	public Insets calculateInsets(IFigure f){
		Dimension textSize = FigureUtilities.getTextExtents(getLabel(), getFont(f));
		int top = textSize.height + 2*BORDER_SIZE + 2 * TITLE_BORDER_SIZE;
		return new Insets(top, BORDER_SIZE, BORDER_SIZE, BORDER_SIZE);
	}
	
	/**
     * Set the color of the foreground
     * @param c color to be set
	 */
	public void setColor (Color c){
		this.color = c;
	}
		
	@Override
	public void finalize(){
		this.font.dispose();
	}
}