/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.gui;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Stack;

/**
 * 
 * @author Daniel Meyer
 *
 */
public class VisiRun extends Thread{
    
   private PropertyChangeSupport listener;
    
	private Stack stack;
	
	/**
	 * Constructor of VisiRun
	 * register the listener and init stack
	 *
	 */
	public VisiRun(){
        this.listener=new PropertyChangeSupport(this);
		  this.stack = new Stack<ArrayList<String>>();
        
        Run run = new Run(this);
        run.start();
        
        while (true){
            if (this.listener.hasListeners("STACK")) //$NON-NLS-1$
                break;
        }
        
	}
	
	@SuppressWarnings({"unchecked","nls"})
	/**
	 * Add the host
	 * @param s name of the host to be added
	 */
    public void addHost(String s){
		ArrayList<String> a = new ArrayList<String>();

		a.add(0,"addHost");
		a.add(1,s);
        this.stack.add(this.stack.size(),a);
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
		
	}
	
	@SuppressWarnings({"unchecked","nls"})
	/**
	 * Remove the host
	 * @param s name of the host to be removed
	 */
    public void removeHost(String s){
		
	    ArrayList<String> a = new ArrayList<String>();
		a.add(0,"removeHost");
		a.add(1,s);
        this.stack.add(this.stack.size(),a);
        //wait for listener
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
    /**
     * Add a component
     * @param host name of the host, where the component to be added
     * @param comp name of the component to be added
     */
	public void addComponent(String host, String comp){

		ArrayList<String> a = new ArrayList<String>();
		a.add(0,"addComponent");
		a.add(1,host);
		a.add(2,comp);
		this.stack.add(this.stack.size(),a);
      this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
    /**
     * Remove the component
     * @param host name of the host, from where the component to be removed
     * @param comp name of the component to be removed
     */
	public void removeComponent(String host, String comp) {
		ArrayList<String> a = new ArrayList<String>();
		a.add(0,"removeComponent");
		a.add(1,host);
		a.add(2,comp);
      this.stack.add(this.stack.size(),a);
      while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
    /**
     * Add an object
     * @param host name of the host, where the object is
     * @param name of the component, where the object is
     * @param objectID id of object, which is to be added
     */
	public void addObject(String host, String name, long objectID){
		ArrayList<String> a = new ArrayList<String>();

      a.add(0,"addObject");
		a.add(1,host);
		a.add(2,name);
		a.add(3,objectID+"");
        this.stack.add(this.stack.size(),a);
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
        /**
     * Remove an object
     * @param host name of the host, where the object is
     * @param name of the component, where the object is
     * @param objectID id of object, which is to be removed
     */
	public void removeObject(String host, String name, long objectID){
		ArrayList<String> a = new ArrayList<String>();
		a.add(0,"removeObject");
		a.add(1,host);
		a.add(2,name);
		a.add(3,objectID+"");
        this.stack.add(this.stack.size(),a);
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
    /**
     * Add a Message
     * @param starthost hostname from where the message start
     * @param startcomp componentname from where the message start
     * @param oidstart object id, from where the message start
     * @param mtag information of the message
     * @param function functino of the message
     * @param rValues rValues of the message
     * @param args arguments of the message
     * @param endhost hostname to where the message end
     * @param endcomp componentname to where the message end
     * @param oidend to where the message end
     */
	public void addMessage (String starthost, String startcomp, long oidstart, 
			String mtag, String function, String rValues, String args,
            String endhost, String endcomp, long oidend){

		ArrayList<String> a = new ArrayList<String>();
		a.add(0,"addMessage");
		a.add(1,starthost);
		a.add(2,startcomp);
		a.add(3,oidstart+"");
		a.add(4,mtag);
		a.add(5,function);
		a.add(6,rValues);
		a.add(7,args);
		a.add(8,endhost);
		a.add(9,endcomp);
		a.add(10,oidend+"");
        this.stack.add(this.stack.size(),a);
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
	
    @SuppressWarnings({"unchecked","nls"})
    /**
     * Remove a Message
     * @param starthost hostname from where the message start
     * @param startcomp componentname from where the message start
     * @param oidstart object id, from where the message start
     * @param mtag information of the message
     * @param function functino of the message
     * @param rValues rValues of the message
     * @param args arguments of the message
     * @param endhost hostname to where the message end
     * @param endcomp componentname to where the message end
     * @param oidend to where the message end
     */
    public void removeMessage (String starthost, String startcomp, long oidstart, 
    		String mtag, String endhost, String endcomp, long oidend){
		
		ArrayList<String> a = new ArrayList<String>();
		a.add(0,"removeMessage");
		a.add(1,starthost);
		a.add(2,startcomp);
		a.add(3,oidstart+"");
		a.add(4,mtag);
		a.add(5,endhost);
		a.add(6,endcomp);
		a.add(7,oidend+"");
        this.stack.add(this.stack.size(),a);
        while (!this.listener.hasListeners("STACK")){
            //
        };
        this.listener.firePropertyChange("STACK",null,this.stack);
	}
    
    /**
     * Add a propertychange listener
     * @param p listener
     */
    public void addPropertyChangeListener(PropertyChangeListener p){
        this.listener.addPropertyChangeListener(p);
//        System.out.println("LISTENER ADDED"); //$NON-NLS-1$
    }
    
    @SuppressWarnings("deprecation")
    /**
     * Remove a perpertychange listener
     */
    public void removePropertyChangeListener(PropertyChangeListener p){
        this.listener.removePropertyChangeListener(p);
//        System.out.println("VISIRUN STOP"); //$NON-NLS-1$
    }
    
    /**
     * Send a close event
     *
     */
    public void close(){
//    	System.out.println("Sending Close Event");
        this.listener.firePropertyChange("CLOSE",null,null); //$NON-NLS-1$     
    }

    /**
     * Remove a message
     * @param host1 one of hostname 
     * @param mtag information of message
     * @param host2 the other host's name
     */
	public void removeMessage(String host1, String mtag, String host2) {
		removeMessage(host1,null,0,mtag,host2,null,0);
		
	}
    
}
