/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.ApplicationWindow;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import Debug.Visualizer.editparts.ComponentEditPart;
import Debug.Visualizer.editparts.DiagramEditPart;
import Debug.Visualizer.editparts.HostEditPart;
import Debug.Visualizer.model.ComponentModel;
import Debug.Visualizer.model.DiagramElementModel;
import Debug.Visualizer.model.HostModel;
import Debug.Visualizer.model.ObjectModel;

import Debug.Visualizer.editparts.*;

import Debug.Visualizer.exceptions.*;
import Debug.Visualizer.figure.InfoscreenFigure;

import Debug.Visualizer.model.*;

/**
 * VisualizerWindow is a JFace ApplicationWindow extended by GEF Content
 * 
 * @author Daniel Meyer
 */
public class VisualizerWindow extends ApplicationWindow
		implements ISelectionChangedListener, PropertyChangeListener{
	 
	//The GEF Graphical Viewer
	public GraphicalViewer viewer;
	
	//The Parent Model
	private DiagramModel model;

    private boolean block=false;

//	private VisiRun run;
    
    private static Display display;
    
	class ChangeColor extends Action {
		
		
		public ChangeColor() {
			setToolTipText("Change Color");
			setImageDescriptor(
				ImageDescriptor.createFromFile(VisualizerWindow.class, 
					System.getenv("PWD")+"/data/img/color.gif"));

		}
		public void run() {
			
			VisualizerColors.setSchema();
			display.getActiveShell().setText("Visualizer|Style:"+VisualizerColors.getStyle());
			
			for(DiagramElementModel mod:model.getChildren()){
				
				//HOSTS
				if(mod instanceof HostModel){
					((HostEditPart)((HostModel)mod).getEditPart()).getFigure()
						.setBackgroundColor(VisualizerColors.BODY_1);
					((HostEditPart)((HostModel)mod).getEditPart()).getFigure()
					.setForegroundColor(VisualizerColors.HEAD_1);
					((HostEditPart)((HostModel)mod).getEditPart()).refresh();
					
					//COMPONENTS
					for (ComponentModel mod1: ((HostModel)mod).getChildren()){
						((ComponentEditPart)((ComponentModel)mod1).getEditPart()).getFigure()
							.setBackgroundColor(VisualizerColors.BODY_2);
						((ComponentEditPart)((ComponentModel)mod1).getEditPart()).getFigure()
						.setForegroundColor(VisualizerColors.HEAD_1);
						((ComponentEditPart)((ComponentModel)mod1).getEditPart()).refresh();
						
						//OBJECTS
						for (ObjectModel mod2: ((ComponentModel)mod1).getChildren()){
							((ObjectEditPart)((ObjectModel)mod2).getEditPart()).getFigure()
							.setBackgroundColor(VisualizerColors.BODY_3);
							((ObjectEditPart)((ObjectModel)mod2).getEditPart()).getFigure()
							.setForegroundColor(VisualizerColors.HEAD_1);
						((ObjectEditPart)((ObjectModel)mod2).getEditPart()).refresh();
							
						}
					}
				
				
				}
				
				//INFOSCREEN
				if(mod instanceof InfoscreenModel){
					((InfoscreenEditPart)((InfoscreenModel)mod).getEditPart()).getFigure()
						.setBackgroundColor(VisualizerColors.BODY_4);
					((InfoscreenEditPart)((InfoscreenModel)mod).getEditPart()).getFigure()
					.setForegroundColor(VisualizerColors.HEAD_2);
					((InfoscreenFigure)((InfoscreenEditPart)((InfoscreenModel)mod).getEditPart()).getFigure())
					.labeledBorder.setColor(VisualizerColors.LABELBORDER_1);
					((InfoscreenEditPart)((InfoscreenModel)mod).getEditPart()).refresh();
					}
			}

		}
	}
	
//	class ChangeLayout extends Action {
//			
//			
//			public ChangeLayout() {
//				setToolTipText("Change Layout");
//				setImageDescriptor(
//					ImageDescriptor.createFromFile(VisualizerWindow.class,
//						System.getenv("PWD")+"/data/img/layout.gif"));
//	
//			}
//			public void run() {
//				
//				VisualizerLayout.setLayout(model);
//				display.getActiveShell().setText("Visualizer|Style:"+VisualizerColors.getStyle()+"|Layout:"+VisualizerLayout.getLayout());
//					
//			}//run
//	}//ChangeLayout
				

    
	/**
	 * Constructor to create and open a VisualizerWindow
	 * and to set its main Content and size.
	 * @param run1 
	 */
	 public VisualizerWindow(VisiRun run1,Display display1, Shell shell) {
		
	    //call the ApplicationWindow Constructor
	    super(shell);
        
        display = display1;
        
//        this.run = run1;
        
        run1.addPropertyChangeListener(this);
        
        this.model = new DiagramModel();
        
        //add the Infoscreen
        this.model.addChild(new InfoscreenModel(350,150));
        
        //add a Toolbar
        addToolBar(SWT.FLAT);
        
        //create this Window
        create();
        
        //Set the Windows minimum Size
        getShell().setMinimumSize(400,520);
        
        getShell().setSize(400,520);
        
        getShell().setLocation(0,0);
        
        getShell().setText("Visualizer|Style:"+VisualizerColors.getStyle());
        
        
        ImageDescriptor imageDescriptor = ImageDescriptor.createFromFile(VisualizerWindow.class,
			System.getenv("PWD")+"/data/img/shell_icon.gif");

        Image image = imageDescriptor.createImage();
        
        getShell().setImage(image);
        //open the Window
        
        open();
        
        shell.setActive();
	}

	/**
	 * createContents is called by JFace upon creation of the Window
	 * @param parent A Composite holding the Windows Controls
	 * 
	 * @return The updated Composite <em>composite</em>
	 */
	 @Override
	protected Control createContents(Composite parent) {
		
		//Create a new Composite
		Composite composite = new Composite(parent, SWT.BORDER);
		
		//set Composite Layout
		composite.setLayout(new FillLayout());
		
		//Instantiate the viewer
		this.viewer = new ScrollingGraphicalViewer();
		
		//add a GEF EditDomain
		this.viewer.setEditDomain(new EditDomain());
        
		//Create viewer Control
		this.viewer.createControl(composite);
		
		//Set EditPartFactory to generate EditParts
		this.viewer.setEditPartFactory(new VisualizerEditPartFactory());
		
		//add the parent Model to the Viewer
		this.viewer.setContents(this.model);
		
		this.viewer.addSelectionChangedListener(this);
		
		
		        
		return composite;
	}
	 
	/**
	 * Creates a Toolbarmanager for the ApplicationWindow
	 * (Currently not used) called upon Window creation
	 * @param style style to be created
	 */
	@Override
	protected ToolBarManager createToolBarManager(int style) {
		ToolBarManager manager = new ToolBarManager(style);
		
		manager.add(new ChangeColor());
//		manager.add(new ChangeLayout());

		return manager;
	}
	 
	/**
	 * addHost adds a new HostModel to the Parent Model
	 * and updates the Display
	 * @param rect an org.eclipse.draw2d.geometry.Rectangle
	 * @param newHost String holding the Hosts Address
	 * @return <em>true</em> if Host is added, otherwise <em>false</em>
	 * @throws DoubleHostException 
	 */
	public synchronized void addHost(String newHost) throws DoubleHostException{
		
        //Synchronize
        while (!setBlock()){
            //wait for true
        }
        
        int x = newHost.indexOf(":");
        String host = newHost.substring(0,x);
        Integer port = Integer.parseInt(newHost.substring(x+1));
        
//        System.out.println("AddHost: started");
        


        HostModel hm1 = getHost(host,port);

        
		if (hm1 != null){
		    if (!hm1.getPorts().contains(port)){
		    		System.out.println("Port not found, set new:"+port);
		    		hm1.addPort(port);
		    		this.block=false;
		    		updateDisplay();

			return;
		    }

		    this.block=false;
		return;
		}
		    
//		    	System.out.println("AddHost: Host already present");
//		        throw new DoubleHostException("There where equal Hosts added: "+hm.toString()+" and "+newHost);   //$NON-NLS-1$ //$NON-NLS-2$
		    
//        }
		
		//add new Host to the Parent Model
		HostModel hm = new HostModel(host,port);
		
		Point p = this.model.getEditPart().getAutomaticLocation(null);
		
		hm.setLocation(p);
		
		FontData fd = new FontData("Arial", 10, SWT.BOLD);  //$NON-NLS-1$
		
		Dimension d = FigureUtilities.getStringExtents(host,new Font(null,fd));
		
		d.height +=20;
		d.width +=20;
		
		hm.setSize(d);
		
		System.out.println("AddHost: Host added");
        
		this.model.addChild(hm);
        
//		if ((VisualizerLayout.getLayoutInt() == 1) || (VisualizerLayout.getLayoutInt() == 2)){
//			update( hm,  null,  null);
//		}
	
		updateDisplay();
        
        while (hm.getEditPart() == null){
            //wait for creation
  
        }
        
        this.block=false;

//        System.out.println("AddHost: Clean Termination");
	}

    
    /**
     * Synchronize Method
     * @return
     */
    private synchronized boolean setBlock() {
        if (this.block) return false;
        this.block = true;
        return true;
    }

    /**
	 * removes a HostModel from the Parent Model
	 * and updates the Display
	 * @param host String holding the Hosts Address
	 * @return <em>true</em> if Host is removed, otherwise <em>false</em>
	 */
	public synchronized void removeHost(String newHost){
		
        //Synchronize
        while (!setBlock()){
            //wait for true
        }
        
        int x = newHost.indexOf(":");
        String host = newHost.substring(0,x);
        Integer port = Integer.parseInt(newHost.substring(x+1));

        
               
        //System.out.println("Remove Host: started");
		HostModel hm = getHost(host,port);
		
		if (hm == null){
			//System.out.println("Remove Host: Host not Found");
			throw new NullPointerException("Unknowwn Host: "+host);  //$NON-NLS-1$
		}
		else {
			if (hm.getPorts().size() != 0){
				
				if (hm.getPorts().contains(port)){
					hm.removePort(port);
					if (hm.getPorts().size() != 0){
						this.block=false;
						return;
					}
				}
			}
		}
		
		//System.out.println("Remove Host: Host Found");
		
        if (this.viewer.getSelectedEditParts().contains(hm.getEditPart())){
        	//System.out.println("RemoveHost: EditPart selected... trying to remove");
            
        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
        
	        while (this.viewer.getSelectedEditParts().contains(hm.getEditPart())){
	        	//Wait for removal
//	        	System.out.print(".");
	        }

	        //System.out.println("RemoveHost: EditPart deselected");
        }
        else{
        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
        	updateDisplay();
        	for (ComponentModel cm : hm.getChildren()){
        		for (ObjectModel om : cm.getChildren()){
        			if (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
        				//System.out.println("RemoveHost: ChildEditPart selected... trying to remove");
        	            
        	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
        	        	
        	        	while (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
        		        	//Wait for removal
//        		        	System.out.print(".");
        		        }

        		        //System.out.println("RemoveHost: ChildEditPart deselected");
        			}
        		}
    			if (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
    				//System.out.println("RemoveHost: ChildEditPart selected... trying to remove");
    	            
    	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
    	        	
    	        	while (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
    		        	//Wait for removal
//    		        	System.out.print(".");
    		        }

    		        //System.out.println("RemoveHost: ChildEditPart deselected");
    			}
        	}
        }

		this.model.removeChild(hm);
		
//		if ((VisualizerLayout.getLayoutInt() == 1)){
//			update( hm,  null,  null);
//		}
		
		updateDisplay();
		
		this.block=false;
		
		//System.out.println("RemoveHost: Clean Termination");
	}
	
	/**
	 * Adds a Component to a Host
	 * @param Host String holding the Hosts address 
	 * @param logID String path to the Component  
	 * @return boolean <em>true</em> if Component is added, otherwise <em>false</em>
	 * @throws DoubleComponentException 
	 */
	 public synchronized void addComponent (String newHost, String name) 
             throws DoubleComponentException{
		 
            while (!setBlock()){
                //wait for true
            }
         
         //System.out.println("AddComponent: started");
              
            int x = newHost.indexOf(":");
            String host = newHost.substring(0,x);
            Integer port = Integer.parseInt(newHost.substring(x+1));

            
            
		 HostModel hm = getHost(host,port);
		 
		 if (hm == null){
			 //System.out.println("AddComponent: Host not Found");
			 throw new HostNullPointerException("Unknowwn Host: "+host); //$NON-NLS-1$
		 }
		 
		 //System.out.println("AddComponent: Host Found");
		 //getAutomaticPosition
		 Point p = ((HostEditPart)hm.getEditPart()).getAutomaticLocation(null);
		 
		 //Add the Component
		 ComponentModel cm = new ComponentModel(name, port);
		 
		 cm.setLocation(p);
		 
			FontData fd = new FontData("Arial", 10, SWT.BOLD);  //$NON-NLS-1$
			
			Dimension d = FigureUtilities.getStringExtents(name,new Font(null,fd));
			
			d.height +=20;
			d.width +=20;
			
			cm.setSize(d);

         ComponentModel cm2 = getComponent(hm,name,port);
         
         if (cm2 != null){
            	 //System.out.println("AddComponent: Component already present");
                 throw new DoubleComponentException("There where equal Components added: "+cm2.toString()); //$NON-NLS-1$
         }
         //System.out.println("AddComponent: Model added");
         hm.addChild(cm);
		 	 
//         updateDisplay( hm,  cm,  null);
         
// 		if ((VisualizerLayout.getLayoutInt() == 1) || (VisualizerLayout.getLayoutInt() == 2)){
//			update( hm,  cm,  null);
//		}
//         
         updateDisplay();
		 
		 while(cm.getEditPart()==null){
			 //wait for creation of ComponentEditPart
		 }
		 //System.out.println("AddComponent: EditPart created");
		 
		 this.block=false;
		 
		 //System.out.println("AddComponent: Clean Termination");
	 }

	/**
	 * Removes a Component from a Host
	 * @param Host String holding the Hosts address 
	 * @param logID String path to the Component  
    * @return boolean <em>true</em> if Component is added, otherwise <em>false</em>
	 */
	 public synchronized void removeComponent (String newHost,String name){
		  
         //Synchronize
         while (!setBlock()){
             //wait for true
         }
         
         //System.out.println("removeComponent: started");
         
         int x = newHost.indexOf(":");
         String host = newHost.substring(0,x);
         Integer port = Integer.parseInt(newHost.substring(x+1));

         
	     HostModel hm = getHost(host,port);
	      
	     if (hm == null) throw new NullPointerException("Unknowwn Host: "+host);  //$NON-NLS-1$
	      
	     //System.out.println("removeComponent: Host found");
	      
	      
	      ComponentModel cm = getComponent(hm,name,port);
	      
	      //Dummy ???
	      if (cm == null){
              throw new NullPointerException("Unknowwn Component: "+name);  //$NON-NLS-1$
          }
	      
	      //System.out.println("removeComponent: Component found");
	      
	      if (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
	        	//System.out.println("RemoveHost: EditPart selected... trying to remove");
	            
	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
	        
		        while (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
		        	//Wait for removal
//		        	System.out.print(".");
		        }

		        //System.out.println("RemoveHost: EditPart deselected");
	        }
	        else{
	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
	        
	        	updateDisplay();
	        	
        		for (ObjectModel om : cm.getChildren()){
        			if (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
        				//System.out.println("RemoveHost: ChildEditPart selected... trying to remove");
        	            
        	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
        	        	
        	        	while (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
        		        	//Wait for removal
//        		        	System.out.print(".");
        		        }

        		        //System.out.println("RemoveHost: ChildEditPart deselected");
        			}
        		}
    			if (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
    				//System.out.println("RemoveHost: ChildEditPart selected... trying to remove");
    	            
    	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
    	        	
    	        	while (this.viewer.getSelectedEditParts().contains(cm.getEditPart())){
    		        	//Wait for removal
//	    		        	System.out.print(".");
    		        }

    		        //System.out.println("RemoveHost: ChildEditPart deselected");
    			}
        	}
	      
	      
	      //remove Component from Host
		  hm.removeChild(cm);
		  
		  //System.out.println("removeComponent: Model removed");
//			if ((VisualizerLayout.getLayoutInt() == 1)){
//				update( hm,  cm,  null);
//			}
		  
		  
		  updateDisplay();
		  
		  while (hm.getChildren().contains(cm)){
			  //wait for deletion
		  }
		  
		  this.block=false;
		  
		  //System.out.println("removeComponent: Clean Termination");
	}

	/**
	 * adds an "remote Object" to a ComponentModel
	 * @param host String: the Host address
	 * @param logID String: Path to Component
	 * @param objectID long: Hash-Code of Object
	 * @throws DoubleObjectException 
	 */
	public synchronized void addObject(String newHost, String name,
            long objectID) throws DoubleObjectException {

        while (!setBlock()){
            //wait for true
        }
        
        //System.out.println("addOBJECT"); //$NON-NLS-1$
        
        int x = newHost.indexOf(":");
        String host = newHost.substring(0,x);
        Integer port = Integer.parseInt(newHost.substring(x+1));

        
        
		//get the component
		ComponentModel cm = getComponent(getHost(host,port),name,port);
		
		//Dummy?
		if (cm != null){
			
			ObjectModel om = new ObjectModel(objectID);
            
			om.setLocation(((ComponentEditPart)cm.getEditPart()).getAutomaticLocation(null));
			
            
            if (getObject(cm,objectID) == null){
                cm.addChild(om);
            }
            else throw new DoubleObjectException(
                    "There where equal Objects added: "+objectID); //$NON-NLS-1$

           
//    		if ((VisualizerLayout.getLayoutInt() == 1) || (VisualizerLayout.getLayoutInt() == 2)){
//    			update( getHost(host,port),  cm,  om);
//    		}
//            
//            updateDisplay( getHost(host),  cm,  om);
            updateDisplay();
            
            while(om.getEditPart()==null){
            	//wait for creation of ObjectEditPart
            }
            
            ((ComponentEditPart)cm.getEditPart()).propertyChange(new PropertyChangeEvent(this,ObjectEditPart.OBJECT_CHILDISH_CHANGE,null,null));
            
            this.block=false;
            
		}
		else{
			throw new NullPointerException("Unknown Component: "+name);  //$NON-NLS-1$
		}
	}


	/**
	 * removes an "remote Object" from a ComponentModel
	 * @param host String: the Host address
	 * @param logID String: Path to Component
	 * @param objectID long: Hash-Code of Object
	 */
	public synchronized boolean removeObject(String newHost, String name, 
			long objectID) {

        //Synchronize
        while (!setBlock()){
            //wait for true
        }
        
        //System.out.println("removeObject: started");
        
        int x = newHost.indexOf(":");
        String host = newHost.substring(0,x);
        Integer port = Integer.parseInt(newHost.substring(x+1));
        
		//get the component
		ComponentModel cm = getComponent(getHost(host,port),name,port);
		
		//Dummy?
		if (cm != null){
			
			ObjectModel om = getObject(cm,objectID);
			
			if(om == null){
				throw new NullPointerException("Object not Found"+objectID);
			}
			
			if (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
	        	//System.out.println("RemoveHost: EditPart selected... trying to remove");
	            
	        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
	        
		        while (this.viewer.getSelectedEditParts().contains(om.getEditPart())){
		        	//Wait for removal
//		        	System.out.print(".");
		        }

		        //System.out.println("RemoveHost: EditPart deselected");
	        }
	        
			cm.removeChild(om);
			
//			if ((VisualizerLayout.getLayoutInt() == 1)){
//				update( getHost(host,port),  cm,  om);
//			}
			
			updateDisplay();
			
			this.block=false;
			
			return true;
		}
		//System.out.println("removeObject: Parent Component does not exits");
		throw new NullPointerException("Component not Found");  //$NON-NLS-1$
	}
	
	/**
	 * Adds a Messages between two HostFigures
	 * @param starthost String, the Host the Message starts from
     * @param startcomp String, the Component where the Message starts or null
     * @param path String, the path to the Component
     * @param start long, the ObjectID, from where the Message started, or null
     * @param endhost String, the Host where the Message ends
     * @param endcomp String, the Component where the Message ends or null
     * @param end long, the ObjectID, from where the Message ends, or null
	  * @throws DoubleMessageException 
     */
	public synchronized void addMessage (String starthost, String startcomp, 
            long oidstart, String mtag, String function, String rValues, 
            String args, String endhost, String endcomp,long oidend) throws DoubleMessageException{
		
        while (!setBlock()){
            //wait for true
        }

        
//        

        int x = starthost.indexOf(":");
        String shost = starthost.substring(0,x);
        Integer sport = Integer.parseInt(starthost.substring(x+1));


        x = endhost.indexOf(":");
        String ehost = endhost.substring(0,x);
        Integer eport = Integer.parseInt(endhost.substring(x+1));

        
//        System.out.println("Host1 "+getHost(shost,sport)+"Component1 "+getComponent(getHost(shost,sport),startcomp,sport)+" Object "+getObject(getComponent(getHost(shost,sport),startcomp,sport),oidstart)+
//        		"Host2 "+getHost(ehost,eport)+" Component2 "+getComponent(getHost(ehost,eport),endcomp,eport)+" Object "+getObject(getComponent(getHost(ehost,eport),endcomp,eport),oidend));
        
        //System.out.println("AddMessage: started");

	    Object start,
               end;
		 
        //get Source and Target Model
        HostModel shm = getHost(shost,sport);
        HostModel ehm = getHost(ehost,eport);
        
        //test if models exist
        if (shm == null){
        		//System.out.println("AddMessage: Source Host not Found");
            	throw new NullPointerException("Source Host doesn't exist"); //$NON-NLS-1$
        }
        
        if (ehm == null){
        	//System.out.println("AddMessage: Target Host not Found");
        	throw new NullPointerException("Target Host doesn't exist"); //$NON-NLS-1$
		}
        
        if (startcomp == null){
            start = shm;
            //System.out.println("AddMessage: Source is Host");
        }
        else {
            if (oidstart == 0){
                start = getComponent(shm,startcomp,sport);
                //System.out.println("AddMessage: Source is Component");
                if (start == null){
                	//System.out.println("AddMessage: Source not Found");
                	throw new NullPointerException("AddMessage: Source not Found");
                }
            }
            else{
                
                start = getObject(getComponent(shm,startcomp,sport),oidstart);
                //System.out.println("AddMessage: Source is Object");
                if (start == null){
                	//System.out.println("AddMessage: Source not Found");
                	throw new NullPointerException("AddMessage: Source not Found");
                }
            }
        }
        
        if (endcomp == null){
            end = ehm;
    		//System.out.println("AddMessage: Target is Host");
        }
        else {
            if (oidend == 0){
                end = getComponent(ehm,endcomp,eport);
                //System.out.println("AddMessage: Target is Component");
                if (end == null){
                	//System.out.println("AddMessage: Target not Found");
                	
                	throw new NullPointerException("AddMessage: Target not Found");
                }
            }
            else
                end = getObject(getComponent(ehm,endcomp,eport),oidend);
	            //System.out.println("AddMessage: Target is Object");
	            if (end == null){
	            	//System.out.println("AddMessage: Target not Found");
	            	throw new NullPointerException("AddMessage: Target not Found");
	            }
        }
		 

        //update the Display
		
		MessageModel mm1 = getMessage(start,mtag,function,rValues,args,end);
						 
		if (mm1 != null){
			throw new DoubleMessageException("There were equal Messages added: "+mm1.getName());
		}
		
        //create Message
		MessageModel mm = new MessageModel(start,mtag,function,rValues,args,end);
		
		updateDisplay();
		//System.out.println("AddMessage: MessageModel created");
		
		while(mm.getEditPart()==null){
			//wait for EditPart Creation
		}

		//Well you cant connect from MessageEditPart to its model, therefore this is here
		
		if(mm.getName().equals("answer")) {
			
			((MessageEditPart)mm.getEditPart()).getFigure().setForegroundColor(ColorConstants.red);
			
		}

		((Label)((MessageEditPart)mm.getEditPart()).getFigure().getChildren().get(1)).setText(mm.getName()+"("+mm.getMtag()+")");

		((MessageEditPart)mm.getEditPart()).refresh();
		
		//System.out.println("AddMessage: EditPart created");
		
		this.block=false;
		
		//System.out.println("AddMessage: Clean Termination");
	 }
	
	/**
	 * Get the MessageModel
	 * @param start the object, from which the message start
	 * @param mtag information of the message
	 * @param function function of the message
	 * @param values value of the message
	 * @param args argument of the message
	 * @param end the object, from which the message ended
	 * @return the MessageModel
	 */
    private MessageModel getMessage(Object start, String mtag, String function, String values, String args, Object end) {
    	
    	List<MessageModel> ls = new ArrayList<MessageModel>();
    	List<MessageModel> le = new ArrayList<MessageModel>();
    	
    	if (start instanceof HostModel){
    		ls = ((HostModel)start).getStartConnections();
    	}
    	else if (start instanceof ComponentModel){
    		ls = ((ComponentModel)start).getStartConnections();
    	}
    	else if (start instanceof ObjectModel){
    		ls = ((ObjectModel)start).getStartConnections();
    	}
    	
    	for (MessageModel mm : ls){
    		if (mm.getMtag().equals(mtag)){
    			if (mm.getName().equals(function)){
    				return mm;
    			}
    		}
    	}
    	
    	if (start instanceof HostModel){
    		le = ((HostModel)start).getEndConnections();
    	}
    	else if (start instanceof ComponentModel){
    		le = ((ComponentModel)start).getEndConnections();
    	}
    	else if (start instanceof ObjectModel){
    		le = ((ObjectModel)start).getEndConnections();
    	}
    	
    	for (MessageModel mm : le){
    		if (mm.getMtag().equals(mtag)){
    			if (mm.getName().equals(function)){
    				return mm;
    			}
    		}
    	}
		return null;

	}

	/**
     * Removes a Messages between two HostFigures
     * @param starthost String, the Host the Message starts from
     * @param startcomp String, the Component where the Message starts or null
     * @param path String, the path to the Component
     * @param start long, the ObjectID, from where the Message started, or null
     * @param endhost String, the Host where the Message ends
     * @param endcomp String, the Component where the Message ends or null
     * @param end long, the ObjectID, from where the Message ends, or null
     */
	@SuppressWarnings("unused")
	public synchronized void removeMessage (String starthost,String startcomp,
            long oidstart, String mtag, String endhost, String endcomp, long oidend){
		
        while (!setBlock()){
            //wait for true
        }
        
        int x = starthost.indexOf(":");
        String shost = starthost.substring(0,x);
        Integer sport = Integer.parseInt(starthost.substring(x+1));

        x = endhost.indexOf(":");
        String ehost = endhost.substring(0,x);
        Integer eport = Integer.parseInt(endhost.substring(x+1));

        
        //System.out.println("RemoveMessage: started");
        
        //get Source and Target Model
        HostModel shm = getHost(shost,sport);
		HostModel ehm = getHost(ehost,eport);
		
        //test if models exist
		if ((shm == null) || (ehm == null))
            throw new NullPointerException("Target or Source Host doesn't exist."+" Source: "+starthost+" Target: "+endhost); //$NON-NLS-1$
		
		//System.out.println("RemoveMessage: Corresponding Hosts found");

        MessageModel mm = getDeleteMessage(shm,ehm,mtag);
		
        if (mm == null){
        	throw new NullPointerException("Message not Found");  //$NON-NLS-1$
        }
        
        //System.out.println("RemoveMessage: MessageModel found");
        
        if (this.viewer.getSelectedEditParts().contains(mm.getEditPart())){
        	//System.out.println("RemoveMessage: EditPart selected... trying to remove");
        	
        	this.viewer.select(((InfoscreenModel)this.model.getChildren().get(0)).getEditPart());
        
	        while (this.viewer.getSelectedEditParts().contains(mm.getEditPart())){
	        	//Wait for removal
//	        	System.out.print(".");
	        }
        
	        //System.out.println("RemoveMessage: EditPart deselected");
        }
        

        //Add Source to Infoscreen
        if (mm.getSource() instanceof ObjectModel){
            ((ObjectModel)mm.getSource()).removeStartConnection(mm);
            //System.out.println("RemoveMessage: MessageSource Found (Object)");
        }
        else if (mm.getSource() instanceof ComponentModel){
            ((ComponentModel)mm.getSource()).removeStartConnection(mm);
            //System.out.println("RemoveMessage: MessageSource Found (Component)");
        }
        else if (mm.getSource() instanceof HostModel){
        	//System.out.println("RemoveMessage: MessageSource Found (Host)");
            ((HostModel)mm.getSource()).removeStartConnection(mm);
        }
        else{
        	//System.out.println("RemoveMessage: MessageSource not Found");
        	return;
        }
        
        //Add Target to Infoscreen
        if (mm.getTarget() instanceof ObjectModel){
            ((ObjectModel)mm.getTarget()).removeEndConnection(mm);
            //System.out.println("RemoveMessage: MessageTarget Found (Object)");
        }
        else if (mm.getTarget() instanceof ComponentModel){
            ((ComponentModel)mm.getTarget()).removeEndConnection(mm);
            //System.out.println("RemoveMessage: MessageTarget Found (Component)");
        }
        else if (mm.getTarget() instanceof HostModel){
            ((HostModel)mm.getTarget()).removeEndConnection(mm);
            //System.out.println("RemoveMessage: MessageTarget Found (Host)");
        }
        else{
        	//System.out.println("RemoveMessage: MessageTarget not Found");
        	return;
        }

        //update the Display
		updateDisplay();
		
		this.block=false;
		//System.out.println("RemoveMessage: Clean Termination");
	 }
	
	/**
	 * Searches for a Message Model between Host hm and ehm
	 * with the MessageTag mtag
	 * @param hm one HostModel
	 * @param ehm the other HostModel
	 * @param mtag information of the message
	 * @return the MessageModel
	 */
    private MessageModel getDeleteMessage(HostModel hm, HostModel ehm, String mtag) {
		
    	for(MessageModel mm : hm.getStartConnections()){
			if (mm.getMtag().equals(mtag)){
				return mm;
			}
		}
    	//System.out.println("removeMessage: Source is no Host");
    	for (ComponentModel cm : hm.getChildren()){
    		for (ObjectModel om : cm.getChildren()){
    			for(MessageModel mm : om.getStartConnections()){
    				if (mm.getMtag().equals(mtag)){
    					return mm;
    				}
    			}
    		}
    		for(MessageModel mm : cm.getStartConnections()){
				if (mm.getMtag().equals(mtag)){
					return mm;
				}
			}
			
		}
    	
    	//System.out.println("RemoveMessage: Source not Found");
    	
    	for(MessageModel mm : ehm.getEndConnections()){
			if (mm.getMtag().equals(mtag)){
				return mm;
			}
		}
    	//System.out.println("removeMessage: Target is no Host");
    	for (ComponentModel cm : ehm.getChildren()){
    		for (ObjectModel om : cm.getChildren()){
    			for(MessageModel mm : om.getEndConnections()){
    				if (mm.getMtag().equals(mtag)){
    					return mm;
    				}
    			}
    		}
    		for(MessageModel mm : cm.getEndConnections()){
				if (mm.getMtag().equals(mtag)){
					return mm;
				}
			}
			
		}
    	//System.out.println("RemoveMessage: Target not Found");

		return null;
	}

	/**
	 * Searches for a Host
	 * @param host String: address of the Host
	 * @param port 
	 * @return the searched ComponentModel
	 */
	private HostModel getHost(String host, Integer port) {
		List models = this.model.getChildren();
        
        if (models.size() <= 1)
            return null;
        
        HostModel hm = null;
        
		for(int i=1;i<=models.size()-1;i++){
			if ( ((HostModel) models.get(i)).getName().equals(host)){
				if ( !((HostModel) models.get(i)).getPorts().contains(port) ){
					hm = ((HostModel) models.get(i));
				}
				else{
					return ((HostModel) models.get(i));
				}
			}
		}
		return hm;
	}
	
	/**
	 * Searches for a Component using a known Host
	 * @param port 
	 * @param hm HostModel: The Host where the Component is located
	 * @param logID String: path to the Component
	 * @return the searched ComponentModel
	 */
	private ComponentModel getComponent(HostModel host, String name, Integer port) {
		
		ComponentModel cm = null;
		
		//is it a Dummy
		if (host != null) {
		
			for(int j=0;j<=host.getChildren().size()-1;j++){
				if ( host.getChildren().get(j).getlogID().equals(name)){
				
					if (host.getChildren().get(j).getPort().compareTo(port) == 0){
						return host.getChildren().get(j);
					}
//					else{
//						cm = host.getChildren().get(j);
//					}
				}
			}
		}
		return cm;
	}
	
	/**
	 * Searches for an ObjectModel
	 * @param cm ComponentModel: The Parent Model of the ObjectModel
	 * @param objectID long: The Hash-Code of the Object
	 * @return the searched ObjectModel
	 */
	private ObjectModel getObject(ComponentModel cm, long objectID) {
		
        if (cm == null) return null;
        
		List<ObjectModel> models = cm.getChildren();
		
		for(int i=0;i<=models.size()-1;i++){
			if (models.get(i).getObjectID() == objectID){
				return models.get(i);
			}
		}
		return null;
	}
	
	/**
	 * Updates the viewer and the Display
	 *
	 */
	private boolean updateDisplay() {
		
		//force viewer to update
		this.viewer.flush();
        
        return true;
	}

	/**
     * Called by GEF after a Mouse Click. This Method displays the Data of the clicked Model
     * to the Infoscreen
     * @param evt the SelectionChangedEvent
	 */
    @SuppressWarnings("nls")
    public void selectionChanged(SelectionChangedEvent evt) {
            
        //init String
		String x=""; 
		
        //get the clicked EditPart
		EditPart part = (EditPart) (((StructuredSelection)evt.
                getSelection()).getFirstElement());
		
		if (part instanceof InfoscreenEditPart || part instanceof InfoscreenDataEditPart) {
        	return;
		}

        //get the InfoscreenModel
		InfoscreenModel ism = (InfoscreenModel)this.model.
                getChildren().get(0);
		
        //remove former Childs
		ism.removeChildren();      
		

        while (!setBlock()){
            //wait for true
        }
        
        //update the Display
		updateDisplay();
		
		this.block=false;

        while (!setBlock()){
            //wait for true
        }
        
		//Add the Children to the Infoscreen
		if (part instanceof HostEditPart){
            
            ism.setName(" [Host]"); 
			
            //get the corresponding Model
			HostModel hm = (HostModel)part.getModel();
			
            //Add Field Host
			ism.addChild("Host",hm.getName()); 
			
            //Add field Port
			if (hm.getPorts() != null)
				ism.addChild("Ports",hm.getPorts().toString()); 
			
            //Add Fild #of Components
			ism.addChild("Components",x+(hm.getChildren().size())); 
			
			//Add number of current Messages
			if (hm.getStartConnections().size()!=0)
				ism.addChild("Current Calls",x+hm.getStartConnections().size());
			
			//Add number of currently received answers
			if (hm.getEndConnections().size()!=0)
				ism.addChild("Current answers",x+hm.getEndConnections().size());
		}
		else if (part instanceof DiagramEditPart){
            
            ism.setName(" [ComponentSystem]"); 
			
            //get the Corresponding Model
			DiagramModel dm = (DiagramModel)part.getModel();
						
            //Add Field #of Hosts
			ism.addChild("Hosts",x+(dm.getChildren().size()-1)); 		
		}
		else if (part instanceof ComponentEditPart){

            ism.setName(" [Component]"); 

            // get the Corresponding Model
            ComponentModel cm = (ComponentModel) part.getModel();

            //add the Components Host
            ism.addChild("Host",((HostModel)part.getParent().getModel()).getName());

            if (cm.getPort() != null)
                // Add Field Role of Component
                ism.addChild("Port", x+cm.getPort());
            
            // Add Field logID of Component
            ism.addChild("logID", cm.getlogID()); 
            
		}
        else if (part instanceof ObjectEditPart) {

            ObjectModel om = (ObjectModel)part.getModel();
            
                // Add Field Role of Component
            ism.addChild("ObjectID", x+om.getObjectID()); 
            
            ism.setName(" [Object]"); 
            
        }
        else if (part instanceof MessageEditPart) {

            ism.setName(" [Message]"); 
            
            MessageModel mm = (MessageModel)part.getModel();
            
            if (mm.getName() != null)
                //Add Function Name to Infoscreen
                ism.addChild("Method Name", mm.getName()); 
            
            //Add Source to Infoscreen
            if (mm.getSource() instanceof ObjectModel)
                ism.addChild("Source", "Object "+mm.getSource().toString());  
            
            else if (mm.getSource() instanceof ComponentModel){
                ism.addChild("Source", "Component "+mm.getSource().toString());  
            }
            else if (mm.getSource() instanceof HostModel){
                ism.addChild("Source", "Host "+mm.getSource().toString());  
            }
            
            //Add Target to Infoscreen
            if (mm.getTarget() instanceof ObjectModel)
                ism.addChild("Target", "Object "+mm.getTarget().toString());  
            
            else if (mm.getTarget() instanceof ComponentModel){
                ism.addChild("Target", "Component "+mm.getTarget().toString());  
            }
            else if (mm.getTarget() instanceof HostModel){
                ism.addChild("Target", "Host "+mm.getTarget().toString());  
            }
            
            if (mm.getMtag() != null)
                //Add Message Tag to the Infoscreen
                ism.addChild("Message Tag", mm.getMtag()); 
            
            if (mm.getArgs() != null)
                //Add Arguments to The Infoscreen
                ism.addChild("Args",mm.getArgs()); 
            
            if (mm.getRvalues() != null)
                //Add Remote Values to the Infoscreen
                ism.addChild("Remote Values",mm.getRvalues()); 
        }
        else{
        	//System.out.println("ERROR IN SELECTION "+part.toString());
        	return;
        }
		
        updateDisplay();
        
        while(ism.getChildren().get(ism.getChildren().size()-1).getEditPart() == null){
        	//wait for creation
        }
        
        this.block=false;
    }

    /** 
     * Handle the events
     * @param evt the change event
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent evt){
    	
        if (evt.getPropertyName().equals("CLOSE")){ //$NON-NLS-1$
            ((VisiRun)evt.getSource()).removePropertyChangeListener(this);
            
            display.asyncExec(new Runnable(){
                public void run(){
                    try {
                    	if (!display.isDisposed()){
                    		if (display.getActiveShell() != null){
                    			display.getActiveShell().close();
                    		}
                    		else
                    			if (getShell()!=null)
                    				getShell().close();
                    	}
                    } catch (Exception e) {

                        e.printStackTrace();
                    }
                }
            });
            
        }
        
        if (evt.getPropertyName().equals("STACK")){ //$NON-NLS-1$
            final Stack stack = (Stack) evt.getNewValue();
            
            while (display == null){
                //wait for initialisation
            }
                display.syncExec(new Runnable(){
                        public void run(){
                            try {
                            updateWindow(stack);
                            } catch (Exception e) {

                                e.printStackTrace();
                            }
                        }
                });
                
            
        }
        
    }

    /**
     * update the window
     * @param stack
     * @throws Exceptions 
     */
    @SuppressWarnings({"nls","unchecked"}) 
    void updateWindow(Stack stack) throws Exception {
       
        while (this.model.getEditPart() == null){
            //WAIT FOR INITIALIZaTIion
        }
        while (((InfoscreenModel) this.model.getChildren().get(0)).getEditPart() == null){
            //wait for infoscreen
        }

        if (stack.isEmpty()) return;
        
        ArrayList<String> b = (ArrayList<String>) stack.pop();
   
//        //System.out.println(b.toString());
        
        
        
        if (b.get(0).equals("addHost"))
            addHost(b.get(1));
        else if (b.get(0).equals("removeHost"))
                removeHost(b.get(1));
        else if (b.get(0).equals("addComponent"))
                addComponent(b.get(1),b.get(2));
        else if (b.get(0).equals("removeComponent"))
                removeComponent(b.get(1),b.get(2));
        else if (b.get(0).equals("addObject"))
                addObject(b.get(1),b.get(2),Long.parseLong(b.get(3)));
        else if (b.get(0).equals("removeObject"))
                removeObject(b.get(1),b.get(2),Long.parseLong(b.get(3)));
        else if (b.get(0).equals("addMessage"))
                addMessage(b.get(1),b.get(2),Long.parseLong(b.get(3)),b.get(4),
                    b.get(5),b.get(6),b.get(7),b.get(8),b.get(9),
                    Long.parseLong(b.get(10)));
        else if (b.get(0).equals("removeMessage"))
                removeMessage(b.get(1),b.get(2),Long.parseLong(b.get(3)),b.get(4)
                		,b.get(5),b.get(6),Long.parseLong(b.get(7)));
    } 
    
    

    @Override
    protected void handleShellCloseEvent(){
        
//        this.run.close();
//        this.run.interrupt();
//        
////        //System.out.println("SHELL CLOSED"); //$NON-NLS-1$
//        
//        super.handleShellCloseEvent();
    }
    
//************************ANORDNUNG**************************************
    
    
//    /**
//     * searches in the children-list of the DiagramModel for Hosts to repositionate
//     *     
//     * @param HostModel hm
//     */
//    public void updateDisplay(HostModel hm) {
//    	boolean abc=false;
//
//        int size =this.model.getChildren().size();
//    	if (abc){
//        if (size>2) {
//        	//CHECK  Infoscreen
//        	if (this.model.getEditPart().infoscreenBlocked((HostModel)hm)) hm.setLocation( ((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation(hm));   		
//        	for (int i=0;i< size; i++){
//    			for (int j=0;j< size; j++){
//    				if (i!=j && ((DiagramEditPart) this.model.getEditPart()).checkCollision(this.model.getChildren().get(i), this.model.getChildren().get(j))) {
//    			
//    			if (this.model.getChildren().get(i)== hm) hm.setLocation(((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation(hm));
//    			else if (this.model.getChildren().get(j) ==hm)	hm.setLocation(((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation(hm));
//    			//avoid relocading the Infoscreen (index j=0)
//    			else if (i!=0)this.model.getChildren().get(i).setLocation(((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation((HostModel)this.model.getChildren().get(i)));		
//    			else this.model.getChildren().get(j).setLocation(((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation((HostModel)this.model.getChildren().get(j)));			
//    				}}}   	
//         	//Last  - possibliy unnessecary		
//        	for (DiagramElementModel dem: this.model.getChildren()){
//        	if (this.model.getChildren().get(size-1)!=dem && ((DiagramEditPart) this.model.getEditPart()).checkCollision( (HostModel)this.model.getChildren().get(size-1), dem))
//        		((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation((HostModel) this.model.getChildren().get(size-1));
//      //  		, this.model.getChildren().get(j)))
//       				}
//        		/*TODO wieder rein damit	((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation((HostModel) this.model.getChildren().get(size-1));
//    		//	checkCollision((HostModel) this.model.getChildren().get(i)) != null) getAutomaticLocation(this.model.getChildren().get(i));       	
//        	//Then others
//   /*     	for (int i =2; i<this.model.getChildren().size();i++){       		
//        		HostModel dem = (HostModel)this.model.getChildren().get(i);
//        		HostModel dum = (HostModel)this.model.getChildren().get(i-1);
//        		if (dem != hm && dem != this.model.getChildren().get(0))
//					if (((DiagramEditPart) this.model.getEditPart()).checkCollision(dem, dum)) {
//				    	 dem.setLocation(((DiagramEditPart) this.model.getEditPart()).getAutomaticLocation(dem));
//					}      
//        }*/
//         //force viewer to update
//    	}
//    	
//    	}
//    	else {
//    		if (size>2) { 			
//    			for (int j= 1; j< this.model.getChildren().size();j++) {
//					DiagramElementModel forward = this.model.getChildren().get(j);
//    				for (int i= this.model.getChildren().size()-1; i>2 ;i--){
//    					DiagramElementModel back = this.model.getChildren().get(i);
//    					if (back != forward){
//    						//Checking the Infoscreen
//    					if (((DiagramEditPart) this.model.getEditPart()).checkCollision(back, this.model.getChildren().get(0))){
//    						((DiagramEditPart) this.model.getEditPart()).insert((HostModel)back, back.getConstraint());}
//    				
//    					if (((DiagramEditPart) this.model.getEditPart()).checkCollision(back,forward)){
//    						((DiagramEditPart) this.model.getEditPart()).insert((HostModel)forward, forward.getConstraint());}
//    				
//    					}
//    					
//    					}
//    				((HostEditPart)((HostModel)this.model.getChildren().get(j)).getEditPart()).refresh();
//    			}
//    			
//    		}   
//    		}
//    	this.viewer.flush(); 	
//    }
//    
//    /**
//     * searches in the children-list of the HostModel for Components to repositionate
//     * 
//     * @param HostModel hm - the Parent of ComponentModelcm
//     * @param ComponentModel cm
//     */
//    public void updateDisplay(HostModel hm, ComponentModel cm) {
//    	if (hm.getChildren().size()>1) {
//    		for (ComponentModel aa: hm.getChildren()) {
//    			for (ComponentModel bb: hm.getChildren()){
//    				if (aa != bb && ((HostEditPart) hm.getEditPart()).checkCollision(aa,bb))
//    					bb.setLocation(((HostEditPart) hm.getEditPart()).getAutomaticLocation(bb)); 				
//    			}  			
//    		}
//   /* 		for (int i=0;i< hm.getChildren().size(); i++){
//    			Rectangle aa= hm.getChildren().get(i).getConstraint();
//    			for (int j=0;j< hm.getChildren().size(); j++){
//    				Rectangle oo= hm.getChildren().get(j).getConstraint();
//    				if (i!=j && hm.getChildren().get(i) == hm.getChildren().get(j))
//    					hm.getChildren().get(j).setLocation(((HostEditPart) hm.getEditPart()).getAutomaticLocation(hm.getChildren().get(j)));		
//    			}	
//    		}*/
//       	((HostEditPart) hm.getEditPart()).calculateSize(hm.getChildren());
//         //force viewer to update
//         this.viewer.flush();}}
//    
//    /**
//     * searches in the children-list of the ComponentModel for Objects to repositionate
//     * 
//     * @param HostModel hm - the Parent of ComponentModelcm
//     * @param ComponentModel cm - the Parent of ObjectModel om
//     * @param ObjectModel om
//     */
//    public void updateDisplay(HostModel hm, ComponentModel cm, ObjectModel om) {      	
//    	if (cm.getChildren().size()>1) {
//       		for (ObjectModel aa: cm.getChildren()) {
//       			for (ObjectModel bb: cm.getChildren()){
//       				if (aa != bb && ((ComponentEditPart) cm.getEditPart()).checkCollision(aa,bb))
//       					bb.setLocation(((ComponentEditPart) cm.getEditPart()).getAutomaticLocation(bb)); 				
//        			}  			
//        		}
//       		/*
//        	for (int i =1; i<cm.getChildren().size();i++){
//        		ObjectModel aa = (ObjectModel)cm.getChildren().get(i);
//        		ObjectModel oo = (ObjectModel)cm.getChildren().get(i-1);
//        		if (aa != om )
//					if (((ComponentEditPart) cm.getEditPart()).checkCollision(aa, oo)) {
//						aa.setLocation(((ComponentEditPart) cm.getEditPart()).getAutomaticLocation(aa));
//					}      
//        }*/
//       	((HostEditPart) hm.getEditPart()).calculateSize(hm.getChildren());
//         //force viewer to update
//         this.viewer.flush();            
//         }}
// 
//    /**
//     * catches the layout - order and decides at which level the refreshing starts 
//     * @param hm
//     * @param cm
//     * @param om
//     */
//    public void update(HostModel hm, ComponentModel cm, ObjectModel om){
//
//	    if (VisualizerLayout.getLayoutInt() ==2){
//		   	if ( om != null) updateDisplay( hm,  cm,  om);
//		   	if ( cm != null) updateDisplay( hm,  cm);
//		   	if ( hm != null) updateDisplay( hm);
//	    }
//	    else if (VisualizerLayout.getLayoutInt() == 1 && this.model.getChildren().size()>0)
//	    	automaticUpdate();
//    }
//
//    /**
//     * gives order to one of possibly a few different autlayout - algorithms
//     *
//     */
//    public void automaticUpdate(){
//    //TODO: Install Switch to different Layouts
//    	//
////    	this.model.getEditPart().autoStarLayout();
//    	
////    	this.model.getEditPart().autoHierarchy("size");
//    	this.model.getEditPart().autoSqrt2();
//    }
}
