/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;


import java.util.*;

import org.eclipse.draw2d.geometry.Rectangle;

import Debug.Visualizer.editparts.ComponentEditPart;

/**
 * This class define the instance of the component, which was saved
 * in the tree-structure.
 * @author ling Wu
 *
 */
public class ComponentModel extends DiagramElementModel{
	
    //Arraylist to hold the connection of target
    private ArrayList<MessageModel> targetConnection = new ArrayList<MessageModel>();
    
    //Arraylsit to hold the connection of source
    private ArrayList<MessageModel> sourceConnection = new ArrayList<MessageModel>();
    
	 //the new object event
	 public static final String _NEWOBJECT="newObject"; //$NON-NLS-1$

	 @SuppressWarnings("hiding")
	 //component constraint change event
	 public static final String PROPERTY_CONSTRAINT_CHANGED = "Component_constraint"; //$NON-NLS-1$

	 //Port of Component
	 private String logID;
	   
    //list to hold children of ComponentModel
	 private List<ObjectModel> children = new ArrayList<ObjectModel>();
	
    //editpart of ComponentModel
	 private ComponentEditPart part;

	 //role of ComponentModel
	 private String role;

	 private Integer port;
	
	/**
	 * Constructor of ComponentModel
	 * @param newlogID logID of component
	 * @param path1 of component
	 */
    public ComponentModel(String newlogID, Integer port1) {
			super();
			this.logID=newlogID;
			this.port=port1;
	}

	/**
	 * Set portnumber of the component
	 * @param id portnumber
	 */
	public void setlogID(String id){

		this.logID=id;
	}
	
	/**
	 * 
	 * @return Port
	 */
	public Integer getPort(){
		return this.port;
	}
    
   /**
    * @return port number of the component
    */
   public String getlogID(){
    	return this.logID;
    }

    /**
	 * 
	 * @return the list of children owned by ComponentModel
	 */
	public List<ObjectModel> getChildren() {
		return this.children;
	}
	
	/**
	 * Adds a child to the children list of the ComponentModel
	 * @param model child to be added
	 */
	public void addChild(ObjectModel model) {
		this.children.add(model);
		firePropertyChange(ComponentModel._NEWOBJECT,null,null);
	}
	
    /**
     * Sets constraint of a model
     * @param rect Rectangle to be set for the ComponentModel
     */
    @Override
    public void setConstraint(Rectangle rect){
        this.size = rect.getSize();
        this.location = rect.getLocation();
        firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, null);
    }
    
	/**
	 * Removes a child from childrenlist 
	 * @param object child to be removed
	 */
	public void removeChild(ObjectModel object) {
		this.children.remove(object);
		firePropertyChange(ComponentModel._NEWOBJECT,null,object.getEditPart());
	}
	
	/**
	 * Set the editpart of ComponentModel
	 * @param p ComponnetEditPart
	 */
	public void setEditPart(ComponentEditPart p) {
		this.part = p;		
	}
	
	/**
	 * 
	 * @return the editpart of ComponentModel
	 */
	public ComponentEditPart getEditPart() {
		return this.part;
	}
    
    @Override
    /**
     * @return logID as String
     */
    public String toString(){
        return this.getlogID();
    }
    
/****************************CONNECTIONS********************************************/
    
    /**
     * adds Connection to the List of Endconnection-Points of the Object
     * @param c Connection which is added to the List
     * 
     */
    public void addEndConnection(MessageModel c)
    {
        this.targetConnection.add(c);
        firePropertyChange(OUTPUT, null, c);
    }

    /**
     * adds Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is added to the List
     * 
     */
    public void addStartConnection(MessageModel c)
    {
        this.sourceConnection.add(c);
        firePropertyChange(INPUT, null, c);
    }

    /**
     * removes Connection from the List of Endconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
    public void removeEndConnection(MessageModel c)
    {
        this.targetConnection.remove(c);
        firePropertyChange(OUTPUT, c, null);
    }

    /**
     * removes Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
    public void removeStartConnection(MessageModel c)
    {
        this.sourceConnection.remove(c);
        firePropertyChange(INPUT, c, null);
    }
    
    /**
     * @return Returns the endConnections
     */
    public List<MessageModel> getEndConnections()
    {
        return this.targetConnection;
    }

    /**
     * @return Returns the startConnections
     */
    public List<MessageModel> getStartConnections()
    {
        return this.sourceConnection;
    }

    /**
     * 
     * @return role as String
     */
	public String getRole() {
		return this.role;
	}
	
	/**
	 * Set the role of component
	 * @param role1 role of component
	 */
	public void setRole(String role1) {
		this.role=role1;
	}

}
