/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

//import java.util.ArrayList;
//import java.util.List;

import org.eclipse.draw2d.geometry.*;
import org.eclipse.gef.EditPart;

/**
 * Abstract class used by HostModel and InfoscreenModel, which
 * holds basic Methods to manipulate a Models Data
 * 
 * @author Daniel Meyer (initial Code), Ling Wu   
 */
public abstract class DiagramElementModel{
	
	//init location
	protected Point location=new Point(0,0);
	
	//init size
	protected Dimension size=new Dimension(0,0);
	
   //event change listener
	protected PropertyChangeSupport listeners;

private EditPart part;
	
	//constraint changed event
	public static final String PROPERTY_CONSTRAINT_CHANGED = "constraint"; //$NON-NLS-1$
	
	//input event
	public static final String INPUT = "INPUT"; //$NON-NLS-1$
	
	//output event
	public static final String OUTPUT = "OUTPUT"; //$NON-NLS-1$
	
	/**
	 * Set the location of model
	 * @param loc top-left point of location
	 */
	public void setLocation(Point loc){
		this.location = loc;
		setConstraint(new Rectangle(getLocation(),getSize()));
	}
	
	/**
	 * 
	 * @return location of model
	 */
	public Point getLocation(){
		return this.location;
	}
	
	/**
	 * Set the size of model
	 * @param size1
	 */
	public void setSize(Dimension size1){
		this.size = size1;
		setConstraint(new Rectangle(getLocation(),getSize()));
	}
	
	/**
	 * 
	 * @return the size of model
	 */
	public Dimension getSize(){
		return this.size;
	}
	
	/**
	 * Sets constraint of a model
	 * @param rect Rectangle of model
	 */
	public void setConstraint(Rectangle rect){
		Rectangle save = new Rectangle(getLocation(), getSize());
		this.size = rect.getSize();
		this.location = rect.getLocation();
		if (!save.equals(rect))
			firePropertyChange(PROPERTY_CONSTRAINT_CHANGED, null, rect);
	}
	
	/**
	 * 
	 * @return the constraint of a model
	 */
	public Rectangle getConstraint(){
		return new Rectangle(getLocation(), getSize());
	}
  
	/**
	 * Constructs the DiagramElementModel
	 */
	public DiagramElementModel() {
		this.listeners = new PropertyChangeSupport(this);
	}

	/**
	 * Removes a Listener from the List of <em>listeners</em>
	 * @param l PropertyChangeListener (here HostEditPart, InfoscreenEditPart)
	 */
	public void removePropertyChangeListener(PropertyChangeListener l) {
		this.listeners.removePropertyChangeListener(l);
	}


	/**
	 * Registers a HostEditPart or InfoscreenEditPart 
	 * as Listener for this Model
	 * @param part HostEditPart, InfoscreenEditPart
	 */
    public void addPropertyChangeListener(PropertyChangeListener l) {
		this.listeners.addPropertyChangeListener(l);
	}
    
    /**
	 * Notifies Listeners in the List <em>listeners</em> of made changes
	 * @param propName A Constant explaining the Change
	 * @param oldValue	The old Value
	 * @param newValue The new Value
	 */
    public void firePropertyChange(
			String propName,
			Object oldValue,
			Object newValue) {
			this.listeners.firePropertyChange(propName, oldValue, newValue);
	}

	public EditPart getEditPart() {
		return this.part;
		
	}
}
