/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;


import Debug.Visualizer.editparts.DiagramEditPart;


/**
 * Represents the Parent Model, which is responsible for the Layout.
 * @author Daniel Meyer (initial Code) , Ling Wu
 */
public class DiagramModel extends DiagramElementModel implements Listener{
	
    //host removed event
	public static final String D_HOST_REMOVED = "Host_removed"; //$NON-NLS-1$

	//diagram constraint change event
   public static final String DIAGRAM_CONSTRAINT_CHANGED = "diagram_constraint_changed"; //$NON-NLS-1$

	//List to hold the DiagramModel's children (DiagramElementModel)
	private List<DiagramElementModel> children = new ArrayList<DiagramElementModel>();
	
    //editpart of DiagramModel
	private DiagramEditPart part;
	
	/**
	 * Constructor of DiagramModel
	 *
	 */
	public DiagramModel(){
		this.listeners = new PropertyChangeSupport(this);
	}
	
	/**
	 * Adds a Child to the DiagramModels List children and
	 * notifies Listeners of the List listeners
	 * @param o DiagramElementModel (i.e. HostModel)
	 */
	public void addChild(DiagramElementModel o){
		
		//add Model to list
		this.children.add(o);
		
		//System.out.println("New Host Child [DM]");
        
		//notify listeners
		this.listeners.firePropertyChange("newElement",null,o); //$NON-NLS-1$
	}
	
	/**
	 * @return The List of <em>children</em> owned by DiagramModel
	 */
	public List<DiagramElementModel> getChildren(){
		return this.children;
	}

	
	/**
	 * Remove the child from children list of the DiagramModel
	 * @param hm child to be removed
	 */
	public void removeChild(HostModel hm) {
		this.children.remove(hm);
		firePropertyChange(D_HOST_REMOVED,hm.getEditPart(),null);
		
	}
	
	/**
	 * Set the editpart of DiagramModel
	 * @param part1
	 */
	public void setEditPart(DiagramEditPart part1) {
		this.part = part1;
	}
	
	/**
	 * 
	 * @return the editpart of DiagramModel
	 */
	public DiagramEditPart getEditPart(){
		return this.part;
	}
    
    /**
     * Sets constraint of a model
     * @param rect Rectangle of DiagramModel
     */
    @Override
    public void setConstraint(Rectangle rect){
    	
        Rectangle save = new Rectangle(getLocation(), getSize());
        this.size = rect.getSize();
        this.location = rect.getLocation();
        
        firePropertyChange(DIAGRAM_CONSTRAINT_CHANGED, save, rect);
    }

    /**
     * Catch Shell Resize Events
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
     */
    public void handleEvent(Event e) {

        //SCALE THE VIEW
        org.eclipse.swt.graphics.Rectangle r1 = ((Shell)e.widget).getBounds();
        Rectangle r = new Rectangle(r1.x,r1.y,r1.width,r1.height);
        firePropertyChange("SCALE",getConstraint(),r); //$NON-NLS-1$
    }
}
