/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.util.List;
import java.util.ArrayList;

import org.eclipse.gef.EditPart;

import Debug.Visualizer.editparts.HostEditPart;

/**
 * Represents a remote Host in the Distributed Component System
 * @author Daniel Meyer (initial Code), Ling Wu
 *
 */
public class HostModel extends DiagramElementModel{

    //Arraylist to hold the connection of target
    private ArrayList<MessageModel> targetConnection = new ArrayList<MessageModel>();
    
    //Arraylsit to hold the connection of source
    private ArrayList<MessageModel> sourceConnection = new ArrayList<MessageModel>();
    
	 //Adress of the Host
	 String name;
	
    //editpart of HostModel
	 private HostEditPart part;
	
	 //List to hold the children of ComponentModel
	 private List<ComponentModel> children = new ArrayList<ComponentModel>();

	 private List<Integer> ports;
	
	 //component added event
	 public static String PROPERTY_NEW_COMPONENT="new Component added"; //$NON-NLS-1$

	/**
	 * Constructor to create a HostModel whith fixed constraints
     * @param newname name of HostModel
	 * @param port 
	 */
	public HostModel(String newname, Integer port) {
		super();
		this.ports = new ArrayList<Integer>();
		this.ports.add(ports.size(),port);
		
		System.out.println(ports.toString());
		
		this.name=newname;
	}
	
	/**
	 * 
	 * @return name of the host
	 */
	public String getName(){
		return this.name;
	}
	
	/**
	 * Adds a child to the children list of ComponentModel
	 * @param model child to be added
	 */
	public void addChild(ComponentModel model) {
		
		this.children.add(model);

		firePropertyChange(PROPERTY_NEW_COMPONENT,model.getEditPart(),null);
		
	}
	
	/**
	 * Remove a child from the children list of the ComponentModel
	 * @param model child to be removed
	 */
	public void removeChild(ComponentModel model){
		this.children.remove(model);
		
		firePropertyChange(PROPERTY_NEW_COMPONENT,null,model.getEditPart());
	}
	
	/**
	 * 
	 * @return children list of the HostModel
	 */
	public List<ComponentModel> getChildren(){
		return this.children;
	}
	
	/**
	 * Set the HostEditpart of HostModel
	 * @param p
	 */
	public void setEditPart(HostEditPart p) {
		this.part = p;
	}
	
	/**
	 * 
	 * @return the HostEditpart of HostModel
	 */
	public EditPart getEditPart() {
		return this.part;
	}
    
    @Override
    /**
     * toString methode return the name as String
     */
    public String toString(){
        return this.name;
    }
    
    /**
     * Compare the name of HostModel
     * @param hm the HostModel to be compared
     * @return true for equals false for not 
     */
    public boolean equals(HostModel hm){
        return (this.name.equals(hm.toString()));
    }
    
    //*************************CONNECTIONS********************************
    
    /**
     * adds Connection to the List of Endconnection-Points of the Object
     * @param c Connection which is added to the List
     * 
     */
    public void addEndConnection(MessageModel c)
    {
        this.targetConnection.add(c);
        firePropertyChange(OUTPUT, null, c);
    }

    /**
     * adds Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is added to the List
     * 
     */
    public void addStartConnection(MessageModel c)
    {
        this.sourceConnection.add(c);
        firePropertyChange(INPUT, null, c);
    }

    /**
     * removes Connection from the List of Endconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
    public void removeEndConnection(MessageModel c)
    {
        this.targetConnection.remove(c);
        firePropertyChange(OUTPUT, c, null);
    }

    /**
     * removes Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
    public void removeStartConnection(MessageModel c)
    {
        this.sourceConnection.remove(c);
        firePropertyChange(INPUT, c, null);
    }
    
    /**
     * @return Returns the endConnections
     */
    public List<MessageModel> getEndConnections()
    {
        return this.targetConnection;
    }

    /**
     * @return Returns the startConnections
     */
    public List<MessageModel> getStartConnections()
    {
        return this.sourceConnection;
    }

	public List<Integer> getPorts() {
		return this.ports;
	}

	public void addPort(Integer p) {
		this.ports.add(ports.size(),p);
		System.out.println(ports.toString());
	}
	
	public void removePort(Integer p){
		this.ports.remove(p);
	}

}
