/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;

import Debug.Visualizer.editparts.InfoscreenDataEditPart;

/**
 * This class represent the information of the choosed component.
 * @author ling Wu
 *
 */
public class InfoscreenDataModel extends DiagramElementModel{
	
    /**
     * Infoscreen data changed event
     */
	public static final String INFOSCREENDATA_DATA_CHANGED = "IDM_Data_changed"; //$NON-NLS-1$

	/**
	 * infoscreenDATA constraint changed event
	 */
	public static final String INFOSRCEENDATA_CONSTRAINT_CHANGED = "infoscreenDATAconstraintchanged";
	
   //Name of the component
	private String subject;
	
	//Information of the component
	private String data;
	
   //editpart of InfoscreenDataModel
	private EditPart part;
	
	/**
	 * Constructor of InfoscreenDataModel
	 * @param title of data
	 * @param content of data
	 */
	public InfoscreenDataModel(String title, String content) {
		this.listeners = new PropertyChangeSupport(this);
		this.subject = title;
		this.data = content;
	}
	
	/**
	 * Set the constraint of infoscreendata
	 * @param r Rectangle of the InfoscreenModel
	 */
	public void setConstraint(Rectangle r){
		Rectangle save = getConstraint();
		this.size = r.getSize();
		this.location = r.getLocation();
		firePropertyChange(INFOSRCEENDATA_CONSTRAINT_CHANGED,save,r);
	}

	/**
	 * Set the name of the component
	 * @param subject1 name of the component
	 */
	public void setSubject(String subject1){
		this.subject = subject1;
	}
	
	/**
	 * 
	 * @return the name of the component
	 */
	public String getSubject(){
		return this.subject;
	}
	
	/**
	 * Set the message-information of the component
	 * @param data1 the massage-information of the component
	 */
	public void setData(String data1){
		
		this.data = data1;
	}
	
	/**
	 * 
	 * @return the message-information of the component
	 */
	public String getData(){
		return this.data;
	}
	
	/**
	 * Set the editpart of InfoscreenDataModel
	 * @param part1 InfoscreenDataEditPart of InfoscreenDataEditPart
	 */
	public void setEditPart(EditPart part1) {
		this.part = part1;;		
	}
	
	/**
	 * 
	 * @return editpart of InfoscreenDataModel
	 */
	public EditPart getEditPart(){
		return this.part;
	}
	
	/**
	 * Adds the event listener
	 * @param l
	 */
	public void addPropertyChangeListener(InfoscreenDataEditPart l) {
		this.listeners.addPropertyChangeListener(l);
	}
	
	/**
	 * Removes the event listener
	 * @param l
	 */
	public void removePropertyChangeListener(InfoscreenDataEditPart l) {
		this.listeners.removePropertyChangeListener(l);
	}
}
