/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.model;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;

import Debug.Visualizer.editparts.InfoscreenEditPart;
import Debug.Visualizer.model.DiagramElementModel;

/**
 * Represents the Model used to store Data of User interactions
 * (i.e. clicked Figures)
 * @author Daniel Meyer (initial Code), Ling Wu
 */
public class InfoscreenModel extends DiagramElementModel {
	
    /**
     * new DataModel event
     */
	public static final String NEWDATA = "newDataModel"; //$NON-NLS-1$
	
    /**
     * new name to display in []
     */
   static public final String INFOSCREEN_NAME_CHANGED = "newNameAddon"; //$NON-NLS-1$
	
   /**
    * Infoscreen constraint changed event
    */
   public static final String INFOSCREEN_CONSTRAINT_CHANGED = "InfoscreenConstraintChanged";
    
    //list to hold InfoscreenDataModel as children of InfoscreenModel
	private List<InfoscreenDataModel> children = new ArrayList<InfoscreenDataModel>();
	
    //editpart of InfoscreenModel
	private InfoscreenEditPart part;
	
	//name of infoscreen to be added
   private String nameaddon=""; //$NON-NLS-1$
	
	/**
	 * Constructor to create a InfoscreenModel
	 * @param w width of Infoscreen
     * @param h height of Infoscreen
	 */
	public InfoscreenModel(int w,int h){

		setConstraint(new Rectangle(0,300,w,h));
		
	}
	
	/**
	 * Adds InfoscreenDataModel as children of InfoscreenModel
	 * @param title
	 * @param content
	 */
	public void addChild(String title, String content) {
        
		this.children.add(new InfoscreenDataModel(title,content));
		
		firePropertyChange(NEWDATA,null,null);
	}
	
	/**
	 * 
	 * @return children of InfoscreenModel as List
	 */
	public List<InfoscreenDataModel> getChildren() {
		return this.children;
	}
	
	/**
	 * Remove a child from children list of the InfoscreenModel
	 * @param model to be removed
	 */
	public void removeChild(InfoscreenDataModel model){
		this.children.remove(model);
		firePropertyChange(NEWDATA,null,null);		
	}
	
	/**
	 * 
	 * @return children list
	 */
	public List getModelchildren(){
		return this.children;
	}
	
	/**
	 * Set the eidtpart of InfoscreenModel
	 * @param part1
	 */
	public void setEditPart(InfoscreenEditPart part1){
		this.part = part1;
	}
	
	/**
	 * 
	 * @return the editpart of InfoscreenModel
	 */
	public EditPart getEditPart(){
		return this.part;
	}

	/**
	 * Remove the children of InfoscreenModel
	 *
	 */
	public void removeChildren() {
		this.children.clear();
	}

    /**
     * Add a name to infoscreen
     * @param string 
     */
    public void setName(String string) {
        this.nameaddon = string;
        firePropertyChange(INFOSCREEN_NAME_CHANGED,null,this.nameaddon);
        
    }

    /**
     * @return The Name Addon
     */
    public String getName() {
        return this.nameaddon;
    }
    
    /**
     * Set the constraint of InfoscreenModel
     * @param r Rectangle of infoscreen
     */
    public void setConstraint(Rectangle r){
    	this.location = r.getLocation();
    	this.size = r.getSize();
    	
    	listeners.firePropertyChange(InfoscreenModel.INFOSCREEN_CONSTRAINT_CHANGED,null,r);
    }
}
