/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.beans.PropertyChangeSupport;

import Debug.Visualizer.editparts.MessageEditPart;

/**
 * This class contains the total Attribute and Mothods to describe the
 * connections between the components.
 * @author Andre Schucht
 *
 */
public class MessageModel extends DiagramElementModel{
	
	//connection informations of the message
	public String mtag;
	
	//connection name of the message
	private String name;
	
	//result of the calls
	private String rvalues;
	
	//argument of the calls
	private String args;
	
	//the source object
	private Object source;
	
	//the target object
	private Object target;
	
   //the editpart of this message
	private MessageEditPart part;
	
	/**
	 * The constraint change event 
	 */
	public static final String PROPERTY_CONSTRAINT_CHANGED = "constraint changed"; //$NON-NLS-1$

	/**
	 * Constructor of MessageModel
	 * @param target targetModel
	 * @param mtag2 connection Information
	 * @param args2 argument of message
	 * @param values values of message
	 * @param function function of message
	 * @param source sourceModel
	 */
	public MessageModel(Object source1, String mtag2, String function, String values, String args2, Object target1)
	{
			listeners = new PropertyChangeSupport(this);
			this.source = source1;
			this.target = target1;
        
			//Add Source to Infoscreen
        if (this.source instanceof ObjectModel)
            ((ObjectModel) this.source).addStartConnection(this);
        
        else if (this.source instanceof ComponentModel){
            ((ComponentModel)this.source).addStartConnection(this);
           }
        
        else if (this.source instanceof HostModel){
            ((HostModel) this.source).addStartConnection(this);
           }
        
        //Add Target to Infoscreen
        if (this.target instanceof ObjectModel)
            ((ObjectModel) this.target).addEndConnection(this);
        
        else if (this.target instanceof ComponentModel){
            ((ComponentModel)this.target).addEndConnection(this);

           }
        else if (this.target instanceof HostModel){
            ((HostModel) this.target).addEndConnection(this);

           }

	  this.mtag = mtag2;
      this.name = function;
      this.rvalues = values;
      this.args = args2;
	}
	
	/**
	 * Set the Editpart of MessageModel
	 * @param p MessageEditPart of MessageModel
	 */
	public void setEditPart(MessageEditPart p) {
		this.part = p;
	}
	
	/**
	 * 
	 * @return the connnectionEditPart
	 */
	public MessageEditPart getEditPart() {
		return this.part;
	}
	
	
	/**
	 * Set the connection information of the message
	 * @param mtag1 the connection information of the message to be set
	 */
	public void setMtag(String mtag1){
		this.mtag=mtag1;
	}
	
	/**
	 * 
	 * @return mtag connection information of the message
	 */
	public String getMtag(){
		return this.mtag;
	}
	
	/**
	 * Set the name of the massage
	 * @param name1 name of the message to be set
	 */
	public void setName(String name1){
		this.name=name1;
	}
	
	/**
	 * 
	 * @return name of the massage
	 */
	public String getName(){
		return this.name;
	}
	
	/**
	 * Set the result of the calls
	 * @param rvalues1 result of the calls to be set
	 */
	public void setRvalues(String rvalues1){
		this.rvalues=rvalues1;
	}
	
	/**
	 * 
	 * @return rvalues result of the calls
	 */
	public String getRvalues(){
		return this.rvalues;
	}
	
	/**
	 * Set the argument of the message
	 * @param args1 argument of the message to be set
	 */
	public void setArgs(String args1){
		this.args=args1;
	}
	
	/**
	 * 
	 * @return the argument of the message
	 */
	public String getArgs(){
		return this.args;
	}
	
	/**
	 * Set a new object as source
	 * @param newSource object as new source
	 */
	public void setSource(ObjectModel newSource){
		this.source=newSource;
	}
	
	/**
	 * 
	 * @return the source object
	 */
	public Object getSource(){
		return this.source;
	}
	
	/**
	 * Set a new object as target
	 * @param newTarget object as new target
	 */
	public void setTarget(Object newTarget){
		this.target=newTarget;
	}
	
	/**
	 * 
	 * @return the target object
	 */
	public Object getTarget(){
		return this.target;
	}
}
