/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model;

import java.util.*;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;

import Debug.Visualizer.model.MessageModel;

/**
 * Represents the object of ComponentModel
 * @author ling Wu
 *
 */
public class ObjectModel extends DiagramElementModel{
	
    //Arraylist to hold the connection of target
	private ArrayList<MessageModel> targetConnection = new ArrayList<MessageModel>();
    
    //Arraylsit to hold the connection of source
	private ArrayList<MessageModel> sourceConnection = new ArrayList<MessageModel>();
	
	//ID-Number of the object
	private long objectID;
	
	//state of the object, IDLE or AKTIV
	private String state;
    
    //name of the object
	private String name;
    
    //role of the object
	private String role;
    	
    //editpart of the ObjectModel
	@SuppressWarnings("unused")
	private EditPart part;
    
    /**
     * Object changed event 
     */
    public static final String OBJECT_CONSTRAINT_CHANGE = "object_constraint_changed"; //$NON-NLS-1$
	
	/**
     * Object changed event 
	 */
	public static final String PROPERTY_OBJECT_CHANGED = "object changed"; //$NON-NLS-1$
	
    /**
     * new object event
     */
	public static final String PROPERTY_NEWOBJECT = "new_Object"; //$NON-NLS-1$
	
	/**
	 * Constructor of ObjectModel
	 * @param OID objectID
	 */
	public ObjectModel(long OID) {
		
		super();
		
		this.objectID = OID;
	}

	
//	/**
//	 * SPECIAL OBJECT
//	 * @param componentname
//	 * @param componentrole
//	 * @param componentstate
//	 */
//	public ObjectModel(String componentname,String componentrole,String componentstate) {
//		super();
//		this.name = componentname;
//		this.role = componentrole;
//		this.state = componentstate;
//		this.objectID = 0;
//	}
//	
	/**
	 * Set the state of component
	 * @param componentstate
	 */
	@SuppressWarnings("unused")
	private void setState(String componentstate) {
		this.state = componentstate;
		firePropertyChange(PROPERTY_OBJECT_CHANGED,null,null);
	}
	
	/**
	 * Set the role of component
	 * @param componentrole
	 */
	@SuppressWarnings("unused")
	private void setRole(String componentrole) {
		this.role = componentrole;
		firePropertyChange(PROPERTY_OBJECT_CHANGED,null,null);		
	}
	
	/**
	 * Set the name of component
	 * @param componentname
	 */
	@SuppressWarnings("unused")
	private void setName(String componentname) {
		this.name = componentname;
		firePropertyChange(PROPERTY_OBJECT_CHANGED,null,null);
	}

	/**
	 * Set the Id-Number of the object
	 * @param o the object Id-number
	 */
	public void setobjectID(long o){
		this.objectID = o;
	}
	
	/**
	 * 
	 * @return the object Id-number
	 */
	public long getObjectID(){
		return this.objectID;
	}
		
	/**
	 * 
	 * @return the state of the object
	 */
	public String getState(){
		return this.state;
	}
	
	/**
	 * 
	 * @return the name of object
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * 
	 * @return role of object
	 */
	public String getRole(){
		return this.role;
	}
	
	/**
	 * Set the editpart of ObjectModel
	 * @param p
	 */
	public void setEditPart(EditPart p) {
		this.part = p;
		
	}

    /**
     * toString method 
     * @return name + role + state as string
     */
	@SuppressWarnings("nls")
	@Override
	public String toString(){
        if (this.objectID == 0)
            return (getName()+" "+getRole()+" "+getState());
        
        return this.objectID+"";
	}
	
/********************Connections*************************/
	
	/**
	 * adds Connection to the List of Endconnection-Points of the Object
     * @param c Connection which is added to the List
	 * 
	 */
	public void addEndConnection(MessageModel c)
	{
		this.targetConnection.add(c);
		firePropertyChange(OUTPUT, null, c);
	}

    /**
     * adds Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is added to the List
     * 
     */
	public void addStartConnection(MessageModel c)
	{
		this.sourceConnection.add(c);
		firePropertyChange(INPUT, null, c);
	}

    /**
     * removes Connection from the List of Endconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
	public void removeEndConnection(MessageModel c)
	{
		this.targetConnection.remove(c);
		firePropertyChange(OUTPUT, c, null);
	}

    /**
     * removes Connection to the List of Startconnection-Points of the Object
     * @param c Connection which is removed to the List
     */
	public void removeStartConnection(MessageModel c)
	{
		this.sourceConnection.remove(c);
		firePropertyChange(INPUT, c, null);
	}
	
	/**
	 * @return Returns the endConnections
	 */
	public List<MessageModel> getEndConnections()
	{
		return this.targetConnection;
	}

	/**
	 * @return Returns the startConnections
	 */
	public List<MessageModel> getStartConnections()
	{
		return this.sourceConnection;
	}


    /**
     * @return EditPart of this model
     */
    public EditPart getEditPart() {
        return this.part;
    }
	
    /**
     * Sets constraint of a object
     * @param rect Rectangle to be set for the ObjectModel
     */
    @Override
    public void setConstraint(Rectangle rect)
    {
//        Rectangle save = new Rectangle(getLocation(), getSize());
        this.size = rect.getSize();
        this.location = rect.getLocation();
        firePropertyChange(OBJECT_CONSTRAINT_CHANGE, null, null);
    }
    
    /**
	 * Set the location of model
	 * @param loc top-left point of location
	 */
	public void setLocation(Point loc){
		this.location = loc;
		setConstraint(new Rectangle(getLocation(),getSize()));
	}
	
	/**
	 * 
	 * @return location of model
	 */
	public Point getLocation(){
		return this.location;
	}
	
	/**
	 * Set the size of model
	 * @param size1
	 */
	public void setSize(Dimension size1){
		this.size = size1;
		setConstraint(new Rectangle(getLocation(),getSize()));
	}
	
	/**
	 * 
	 * @return the size of model
	 */
	public Dimension getSize(){
		return this.size;
	}
}
