/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */

package Debug.Visualizer.model.commands;

import java.beans.PropertyChangeEvent;

import org.eclipse.gef.commands.*;
import org.eclipse.draw2d.geometry.*;

import Debug.Visualizer.editparts.DiagramEditPart;
import Debug.Visualizer.editparts.HostEditPart;
import Debug.Visualizer.editparts.InfoscreenEditPart;

import Debug.Visualizer.model.HostModel;
import Debug.Visualizer.model.InfoscreenModel;

/**
 * This class controls Moving a grafical element on the window
 * The Command uses 'constraints' for getting location and size. Therefor you
 * need the XYLayout-Role for Moving and Resizing the 'constraint'
 * @author Andre Schucht
 * @version 1.2
 * @see org.eclipse.draw2d.XYLayout
 */

public class MoveCommand extends Command {
	
	//old bounds of model
	private Rectangle oldBounds;
	
	//new bounds of model
	private Rectangle newBounds;
	
	//model which will be moved
	private Object model;
	
	/**
	 * The Parameters oldBounds and newBounds were passed, to
	 * localize the former and new Loaction of the Model.
	 * @param model1 the model to be localized
	 * @param oldBounds1 old bounds of the model
	 * @param newBounds1 new bounds of the model
	 */
	public MoveCommand(Object model1, Rectangle oldBounds1, Rectangle newBounds1)
	{
		super();
		this.model = model1;
		this.oldBounds = oldBounds1;
		this.newBounds = newBounds1;
	}

	/**
	 * This Methods execute the Moving of the Model.
	 */
	@Override
	public void execute(){
				
		if (this.model instanceof HostModel){
			
//			this.oldBounds = ((HostModel)this.model).getConstraint();

			//update the  Pointlist
//			((DiagramEditPart)((HostEditPart)((HostModel)this.model).getEditPart())
//					.getParent()).propertyChange(new PropertyChangeEvent(
//							((HostModel)this.model).getEditPart(),
//							HostEditPart.HOST_CONSTRAINT_CHANGED,
//							this.oldBounds, 
//							this.newBounds));
						
			Rectangle parent = ((DiagramEditPart)((HostEditPart)((HostModel)this.model).getEditPart())
					.getParent()).getFigure().getBounds();
			
			//compare the bounds of HostModel and DiagrammModel, decide if removeble
         if (parent.x >= this.newBounds.x)      this.newBounds.x=0;
         if (parent.y >= this.newBounds.y)     	this.newBounds.y=0;
						
			if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
				//MOVE
				((HostModel)this.model).setConstraint(this.newBounds);
			}
			else{
				//RESIZE

				this.newBounds.x = this.oldBounds.x;
				this.newBounds.y = this.oldBounds.y;
				
				((HostModel)this.model).setConstraint(this.newBounds);
				((HostEditPart)((HostModel)this.model).getEditPart())
				.propertyChange(new PropertyChangeEvent(this,"SCALE",this.oldBounds,this.newBounds));	
			}
			
		}
		else if (this.model instanceof InfoscreenModel){
			
			this.oldBounds = ((InfoscreenModel)this.model).getConstraint();

//			//update the  Pointlist
//			((DiagramEditPart)((InfoscreenEditPart)((InfoscreenModel)this.model).getEditPart())
//					.getParent())
//					.propertyChange(new PropertyChangeEvent(
//							((InfoscreenModel)this.model).getEditPart(),
//							InfoscreenEditPart.INFOSCREEN_CONSTRAINT_CHANGED,
//							this.oldBounds, 
//							this.newBounds));
			
			Rectangle parent = ((DiagramEditPart)((InfoscreenEditPart)((InfoscreenModel)this.model).getEditPart())
					.getParent()).getFigure().getBounds();
			
            if (parent.x >= this.newBounds.x)
                this.newBounds.x=parent.x;
            if (parent.y >= this.newBounds.y)
            	this.newBounds.y=parent.y;

			if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
				//MOVE
				((InfoscreenModel)this.model).setConstraint(this.newBounds);
			}
			else{
				//RESIZE

				this.newBounds.x = this.oldBounds.x;
				this.newBounds.y = this.oldBounds.y;
				
				//System.out.println(newBounds.toString());
				
				((InfoscreenModel)this.model).setConstraint(this.newBounds);
				((InfoscreenEditPart)((InfoscreenModel)this.model).getEditPart())
				.propertyChange(new PropertyChangeEvent(this,"SCALE",this.oldBounds,this.newBounds));	
			}
			
		}		
	}

    /**
     * Actually, this Method is only DUMM-Method.
     * In future, you will be able to undo an execute.
     */
	@Override
	public void undo()
	{
		if (this.model instanceof InfoscreenModel)
			((InfoscreenModel)this.model).setConstraint(this.oldBounds);
		else if (this.model instanceof HostModel)
			((HostModel)this.model).setConstraint(this.oldBounds);
	}
}
