/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.model.commands;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;

import Debug.Visualizer.editparts.HostEditPart;

import Debug.Visualizer.editparts.ComponentEditPart;

import Debug.Visualizer.model.ComponentModel;

/**
 * This class controls Moving a grafical element on the window
 * The Command uses 'constraints' for getting location and size. Therefor you
 * need the XYLayout-Role for Moving and Resizing the 'constraint'
 * @author Andre Schucht
 * @version 1.1
 * @see org.eclipse.draw2d.XYLayout
 */
public class MoveComponentCommand extends Command{
    
    private Rectangle oldBounds;
    private Rectangle newBounds;
    private Object model;
    
    /**
     * The Parameters oldBounds and newBounds were passed, to
     * localize the former and new Loaction of the Model.
     * @param model1 the model to be localized
	  * @param oldBounds1 old bounds of the model
	  * @param newBounds1 new bounds of the model
     */
    public MoveComponentCommand(Object model1, Rectangle oldBounds1, Rectangle newBounds1)
    {
        super();
        this.model = model1;
        this.oldBounds = oldBounds1;
        this.newBounds = newBounds1;
    }

    /**
     * This Methods execute the Moving of the Model.
     */
    @Override
    public void execute(){           
//            this.oldBounds = ((ComponentModel)this.model).getConstraint();
            
            Rectangle parent = ((HostEditPart)((ComponentEditPart)
            		((ComponentModel)this.model).getEditPart())
            		.getParent()).getFigure().getBounds();
            
//            Insets in = ((HostEditPart)((ComponentEditPart)
//            		((ComponentModel)this.model).getEditPart()).getParent())
//            		.getFigure().getBorder().getInsets(((HostEditPart)
//            				((ComponentEditPart)((ComponentModel)this.model)
//            						.getEditPart()).getParent()).getFigure());
            
            //compare the bounds of ComponentModel and HostModel, 
            //decide if it is removeble
            if (parent.x >= this.newBounds.x + parent.x)     this.newBounds.x=0;
            if (parent.y >= this.newBounds.y + parent.y)     this.newBounds.y=0;
                  
            ((ComponentModel)this.model).setConstraint(this.newBounds.getCopy());

//            //update the  Pointlist
//            ((HostEditPart)((ComponentEditPart)((ComponentModel)this.model).getEditPart())
//                    .getParent())
//                    .propertyChange(new PropertyChangeEvent(
//                            ((ComponentModel)this.model).getEditPart(),
//                            ComponentEditPart.INFOSCREEN_CONSTRAINT_CHANGED,
//                            this.oldBounds, 
//                            this.newBounds));
            if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
            	//Move...
            	((ComponentModel)this.model).setConstraint(this.newBounds.getCopy());
            }
            else{
            	//resize

            	this.newBounds.x = this.oldBounds.x;
				this.newBounds.y = this.oldBounds.y;
            	
				((ComponentModel)this.model).setConstraint(this.newBounds.getCopy());
				
        		((ComponentEditPart)((ComponentModel)this.model).getEditPart())
    			.propertyChange(new PropertyChangeEvent(this,"SCALE",this.oldBounds,this.newBounds));
          
            }
	
    }

    /**
     * Actually, this Method is only DUMMY-Method.
     * In future, you will be able to undo an execute.
     */
    @Override
    public void undo(){
      ((ComponentModel)this.model).setConstraint(this.oldBounds);
    }
}