/*
 * Copyright (C) 2006 Johannes Baumgarten, Bjoern Gebhardt, Mathias Kellner, Stephan Kluge, Svetlana Li, Jurij Merker, Daniel Meyer, Andre Schucht, Ling Wu
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License. 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 */
package Debug.Visualizer.model.commands;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;

import Debug.Visualizer.editparts.ComponentEditPart;
import Debug.Visualizer.editparts.ObjectEditPart;

import Debug.Visualizer.model.ObjectModel;

/**
 * This class controls Moving a grafical element on the window
 * The Command uses 'constraints' for getting location and size. Therefor you
 * need the XYLayout-Role for Moving and Resizing the 'constraint'
 * @author Andre Schucht
 * @version 1.1
 * @see org.eclipse.draw2d.XYLayout
 */
public class MoveObjectCommand extends Command {
    
    private Rectangle oldBounds;
    private Rectangle newBounds;
    private Object model;
    
    /**
     * The Parameters oldBounds and newBounds were passed, to
     * localize the former and new Loaction of the Model.
     * @param model1 the model to be localized
	  * @param oldBounds1 old bounds of the model
	  * @param newBounds1 new bounds of the model	       
	 */
    public MoveObjectCommand(Object model1, Rectangle oldBounds1, Rectangle newBounds1)
    {
        super();
        this.model = model1;
        this.oldBounds = oldBounds1;
        this.newBounds = newBounds1;
    }

    /**
     * This Methods execute the Moving of the Model.
     */
    @Override
    public void execute(){           
            
//            //update the  Pointlist
//            ((ComponentEditPart)((ObjectModel)this.model).getEditPart()
//                    .getParent())
//                    .propertyChange(new PropertyChangeEvent(
//                            ((ObjectModel)this.model).getEditPart(),
//                            ComponentEditPart.PROPERTY_CHILDISH_CHANGE,
//                            this.oldBounds, 
//                            this.newBounds));   
            
            Rectangle parent = ((ComponentEditPart)((ObjectEditPart)
            		((ObjectModel)this.model).getEditPart())
            		.getParent()).getFigure().getClientArea();
            
//            Insets in = ((ObjectEditPart)((ObjectModel)this.model)
//            		 		.getEditPart()).getFigure().getInsets();
//            
//            
            
            //compare the bounds of ComponentModel and HostModel, 
            //decide if it is removeble
            if (parent.x >= this.newBounds.x)  		this.newBounds.x = 0;
            if (parent.y >= this.newBounds.y)  		this.newBounds.y = 0;
            
            if ((newBounds.height == oldBounds.height) && (newBounds.width == oldBounds.width)){
            	//Move...
            	
            	((ObjectModel)this.model).setLocation(newBounds.getLocation());
            }
            else{
            	//resize
            	
				this.newBounds.setLocation(oldBounds.getLocation());
				
            	((ObjectModel)this.model).setSize(this.newBounds.getSize());
            	
    			((ObjectEditPart)((ObjectModel)this.model).getEditPart())
    			.propertyChange(new PropertyChangeEvent(this,"SCALE",this.oldBounds,this.newBounds));
            }
            

    }

    /**
     * Actually, this Method is only DUMMY-Method.
     * In future, you will be able to undo an execute.
     */
    @Override
    public void undo(){
      ((ObjectModel)this.model).setConstraint(this.oldBounds);
    }

}
